@@ -5,13 +5,14 @@ import (
 	"fmt"
 	"io"
 	"io/ioutil"
-	"log"
 	"math"
 	"net"
 	"os"
 	"path/filepath"
 	"reflect"
+	"runtime"
 	"strings"
+	"sync"
 	"testing"
 	"time"
 
@@ -20,6 +21,7 @@ import (
 	"github.com/hashicorp/nomad/client/config"
 	sframer "github.com/hashicorp/nomad/client/lib/streamframer"
 	cstructs "github.com/hashicorp/nomad/client/structs"
+	"github.com/hashicorp/nomad/helper/testlog"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad"
 	"github.com/hashicorp/nomad/nomad/mock"
@@ -32,7 +34,7 @@ import (
 // tempAllocDir returns a new alloc dir that is rooted in a temp dir. The caller
 // should destroy the temp dir.
 func tempAllocDir(t testing.TB) *allocdir.AllocDir {
-	dir, err := ioutil.TempDir("", "")
+	dir, err := ioutil.TempDir("", "nomadtest")
 	if err != nil {
 		t.Fatalf("TempDir() failed: %v", err)
 	}
@@ -41,7 +43,7 @@ func tempAllocDir(t testing.TB) *allocdir.AllocDir {
 		t.Fatalf("failed to chmod dir: %v", err)
 	}
 
-	return allocdir.NewAllocDir(log.New(os.Stderr, "", log.LstdFlags), dir)
+	return allocdir.NewAllocDir(testlog.HCLogger(t), dir)
 }
 
 type nopWriteCloser struct {
@@ -57,8 +59,8 @@ func TestFS_Stat_NoAlloc(t *testing.T) {
 	require := require.New(t)
 
 	// Start a client
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Make the request with bad allocation id
 	req := &cstructs.FsStatRequest{
@@ -77,29 +79,28 @@ func TestFS_Stat(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
 
-	// Start a client
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	// Start a server and client
+	s := nomad.TestServer(t, nil)
+	defer s.Shutdown()
+	testutil.WaitForLeader(t, s.RPC)
+
+	c, cleanup := TestClient(t, func(c *config.Config) {
+		c.Servers = []string{s.GetConfig().RPCAddr.String()}
+	})
+	defer cleanup()
 
 	// Create and add an alloc
-	a := mock.Alloc()
-	c.addAlloc(a, "")
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for": "500ms",
+	}
+	// Wait for alloc to be running
+	alloc := testutil.WaitForRunning(t, s.RPC, job)[0]
 
-	// Wait for the client to start it
-	testutil.WaitForResult(func() (bool, error) {
-		ar, ok := c.allocs[a.ID]
-		if !ok {
-			return false, fmt.Errorf("alloc doesn't exist")
-		}
-
-		return len(ar.tasks) != 0, fmt.Errorf("tasks not running")
-	}, func(err error) {
-		t.Fatal(err)
-	})
-
-	// Make the request with bad allocation id
+	// Make the request
 	req := &cstructs.FsStatRequest{
-		AllocID:      a.ID,
+		AllocID:      alloc.ID,
 		Path:         "/",
 		QueryOptions: structs.QueryOptions{Region: "global"},
 	}
@@ -113,18 +114,17 @@ func TestFS_Stat(t *testing.T) {
 
 func TestFS_Stat_ACL(t *testing.T) {
 	t.Parallel()
-	require := require.New(t)
 
 	// Start a server
 	s, root := nomad.TestACLServer(t, nil)
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	client := TestClient(t, func(c *config.Config) {
+	client, cleanup := TestClient(t, func(c *config.Config) {
 		c.ACLEnabled = true
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer client.Shutdown()
+	defer cleanup()
 
 	// Create a bad token
 	policyBad := mock.NamespacePolicy("other", "", []string{acl.NamespaceCapabilityDeny})
@@ -134,6 +134,15 @@ func TestFS_Stat_ACL(t *testing.T) {
 		[]string{acl.NamespaceCapabilityReadLogs, acl.NamespaceCapabilityReadFS})
 	tokenGood := mock.CreatePolicyAndToken(t, s.State(), 1009, "valid2", policyGood)
 
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for": "20s",
+	}
+
+	// Wait for client to be running job
+	alloc := testutil.WaitForRunningWithToken(t, s.RPC, job, root.SecretID)[0]
+
 	cases := []struct {
 		Name          string
 		Token         string
@@ -145,22 +154,19 @@ func TestFS_Stat_ACL(t *testing.T) {
 			ExpectedError: structs.ErrPermissionDenied.Error(),
 		},
 		{
-			Name:          "good token",
-			Token:         tokenGood.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "good token",
+			Token: tokenGood.SecretID,
 		},
 		{
-			Name:          "root token",
-			Token:         root.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "root token",
+			Token: root.SecretID,
 		},
 	}
 
 	for _, c := range cases {
 		t.Run(c.Name, func(t *testing.T) {
-			// Make the request with bad allocation id
 			req := &cstructs.FsStatRequest{
-				AllocID: uuid.Generate(),
+				AllocID: alloc.ID,
 				Path:    "/",
 				QueryOptions: structs.QueryOptions{
 					Region:    "global",
@@ -171,8 +177,12 @@ func TestFS_Stat_ACL(t *testing.T) {
 
 			var resp cstructs.FsStatResponse
 			err := client.ClientRPC("FileSystem.Stat", req, &resp)
-			require.NotNil(err)
-			require.Contains(err.Error(), c.ExpectedError)
+			if c.ExpectedError == "" {
+				require.NoError(t, err)
+			} else {
+				require.NotNil(t, err)
+				require.Contains(t, err.Error(), c.ExpectedError)
+			}
 		})
 	}
 }
@@ -182,8 +192,8 @@ func TestFS_List_NoAlloc(t *testing.T) {
 	require := require.New(t)
 
 	// Start a client
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Make the request with bad allocation id
 	req := &cstructs.FsListRequest{
@@ -202,29 +212,28 @@ func TestFS_List(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
 
-	// Start a client
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	// Start a server and client
+	s := nomad.TestServer(t, nil)
+	defer s.Shutdown()
+	testutil.WaitForLeader(t, s.RPC)
+
+	c, cleanup := TestClient(t, func(c *config.Config) {
+		c.Servers = []string{s.GetConfig().RPCAddr.String()}
+	})
+	defer cleanup()
 
 	// Create and add an alloc
-	a := mock.Alloc()
-	c.addAlloc(a, "")
-
-	// Wait for the client to start it
-	testutil.WaitForResult(func() (bool, error) {
-		ar, ok := c.allocs[a.ID]
-		if !ok {
-			return false, fmt.Errorf("alloc doesn't exist")
-		}
-
-		return len(ar.tasks) != 0, fmt.Errorf("tasks not running")
-	}, func(err error) {
-		t.Fatal(err)
-	})
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for": "500ms",
+	}
+	// Wait for alloc to be running
+	alloc := testutil.WaitForRunning(t, s.RPC, job)[0]
 
 	// Make the request
 	req := &cstructs.FsListRequest{
-		AllocID:      a.ID,
+		AllocID:      alloc.ID,
 		Path:         "/",
 		QueryOptions: structs.QueryOptions{Region: "global"},
 	}
@@ -238,18 +247,17 @@ func TestFS_List(t *testing.T) {
 
 func TestFS_List_ACL(t *testing.T) {
 	t.Parallel()
-	require := require.New(t)
 
 	// Start a server
 	s, root := nomad.TestACLServer(t, nil)
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	client := TestClient(t, func(c *config.Config) {
+	client, cleanup := TestClient(t, func(c *config.Config) {
 		c.ACLEnabled = true
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer client.Shutdown()
+	defer cleanup()
 
 	// Create a bad token
 	policyBad := mock.NamespacePolicy("other", "", []string{acl.NamespaceCapabilityDeny})
@@ -259,6 +267,15 @@ func TestFS_List_ACL(t *testing.T) {
 		[]string{acl.NamespaceCapabilityReadLogs, acl.NamespaceCapabilityReadFS})
 	tokenGood := mock.CreatePolicyAndToken(t, s.State(), 1009, "valid2", policyGood)
 
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for": "20s",
+	}
+
+	// Wait for client to be running job
+	alloc := testutil.WaitForRunningWithToken(t, s.RPC, job, root.SecretID)[0]
+
 	cases := []struct {
 		Name          string
 		Token         string
@@ -270,14 +287,12 @@ func TestFS_List_ACL(t *testing.T) {
 			ExpectedError: structs.ErrPermissionDenied.Error(),
 		},
 		{
-			Name:          "good token",
-			Token:         tokenGood.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "good token",
+			Token: tokenGood.SecretID,
 		},
 		{
-			Name:          "root token",
-			Token:         root.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "root token",
+			Token: root.SecretID,
 		},
 	}
 
@@ -285,7 +300,7 @@ func TestFS_List_ACL(t *testing.T) {
 		t.Run(c.Name, func(t *testing.T) {
 			// Make the request with bad allocation id
 			req := &cstructs.FsListRequest{
-				AllocID: uuid.Generate(),
+				AllocID: alloc.ID,
 				Path:    "/",
 				QueryOptions: structs.QueryOptions{
 					Region:    "global",
@@ -296,8 +311,11 @@ func TestFS_List_ACL(t *testing.T) {
 
 			var resp cstructs.FsListResponse
 			err := client.ClientRPC("FileSystem.List", req, &resp)
-			require.NotNil(err)
-			require.Contains(err.Error(), c.ExpectedError)
+			if c.ExpectedError == "" {
+				require.NoError(t, err)
+			} else {
+				require.EqualError(t, err, c.ExpectedError)
+			}
 		})
 	}
 }
@@ -307,8 +325,8 @@ func TestFS_Stream_NoAlloc(t *testing.T) {
 	require := require.New(t)
 
 	// Start a client
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Make the request with bad allocation id
 	req := &cstructs.FsStreamRequest{
@@ -379,18 +397,17 @@ OUTER:
 
 func TestFS_Stream_ACL(t *testing.T) {
 	t.Parallel()
-	require := require.New(t)
 
 	// Start a server
 	s, root := nomad.TestACLServer(t, nil)
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	client := TestClient(t, func(c *config.Config) {
+	client, cleanup := TestClient(t, func(c *config.Config) {
 		c.ACLEnabled = true
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer client.Shutdown()
+	defer cleanup()
 
 	// Create a bad token
 	policyBad := mock.NamespacePolicy("other", "", []string{acl.NamespaceCapabilityReadFS})
@@ -400,6 +417,15 @@ func TestFS_Stream_ACL(t *testing.T) {
 		[]string{acl.NamespaceCapabilityReadLogs, acl.NamespaceCapabilityReadFS})
 	tokenGood := mock.CreatePolicyAndToken(t, s.State(), 1009, "valid2", policyGood)
 
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for": "20s",
+	}
+
+	// Wait for client to be running job
+	alloc := testutil.WaitForRunningWithToken(t, s.RPC, job, root.SecretID)[0]
+
 	cases := []struct {
 		Name          string
 		Token         string
@@ -411,14 +437,12 @@ func TestFS_Stream_ACL(t *testing.T) {
 			ExpectedError: structs.ErrPermissionDenied.Error(),
 		},
 		{
-			Name:          "good token",
-			Token:         tokenGood.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "good token",
+			Token: tokenGood.SecretID,
 		},
 		{
-			Name:          "root token",
-			Token:         root.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "root token",
+			Token: root.SecretID,
 		},
 	}
 
@@ -426,7 +450,7 @@ func TestFS_Stream_ACL(t *testing.T) {
 		t.Run(c.Name, func(t *testing.T) {
 			// Make the request with bad allocation id
 			req := &cstructs.FsStreamRequest{
-				AllocID: uuid.Generate(),
+				AllocID: alloc.ID,
 				Path:    "foo",
 				Origin:  "start",
 				QueryOptions: structs.QueryOptions{
@@ -438,7 +462,7 @@ func TestFS_Stream_ACL(t *testing.T) {
 
 			// Get the handler
 			handler, err := client.StreamingRpcHandler("FileSystem.Stream")
-			require.Nil(err)
+			require.Nil(t, err)
 
 			// Create a pipe
 			p1, p2 := net.Pipe()
@@ -457,10 +481,8 @@ func TestFS_Stream_ACL(t *testing.T) {
 				for {
 					var msg cstructs.StreamErrWrapper
 					if err := decoder.Decode(&msg); err != nil {
-						if err == io.EOF || strings.Contains(err.Error(), "closed") {
-							return
-						}
-						errCh <- fmt.Errorf("error decoding: %v", err)
+						errCh <- err
+						return
 					}
 
 					streamMsg <- &msg
@@ -469,7 +491,7 @@ func TestFS_Stream_ACL(t *testing.T) {
 
 			// Send the request
 			encoder := codec.NewEncoder(p1, structs.MsgpackHandle)
-			require.Nil(encoder.Encode(req))
+			require.NoError(t, encoder.Encode(req))
 
 			timeout := time.After(5 * time.Second)
 
@@ -479,6 +501,11 @@ func TestFS_Stream_ACL(t *testing.T) {
 				case <-timeout:
 					t.Fatal("timeout")
 				case err := <-errCh:
+					eof := err == io.EOF || strings.Contains(err.Error(), "closed")
+					if c.ExpectedError == "" && eof {
+						// No error was expected!
+						return
+					}
 					t.Fatal(err)
 				case msg := <-streamMsg:
 					if msg.Error == nil {
@@ -505,72 +532,25 @@ func TestFS_Stream(t *testing.T) {
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	c := TestClient(t, func(c *config.Config) {
+	c, cleanup := TestClient(t, func(c *config.Config) {
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer c.Shutdown()
+	defer cleanup()
 
-	// Force an allocation onto the node
 	expected := "Hello from the other side"
-	a := mock.Alloc()
-	a.Job.Type = structs.JobTypeBatch
-	a.NodeID = c.NodeID()
-	a.Job.TaskGroups[0].Count = 1
-	a.Job.TaskGroups[0].Tasks[0] = &structs.Task{
-		Name:   "web",
-		Driver: "mock_driver",
-		Config: map[string]interface{}{
-			"run_for":       "2s",
-			"stdout_string": expected,
-		},
-		LogConfig: structs.DefaultLogConfig(),
-		Resources: &structs.Resources{
-			CPU:      500,
-			MemoryMB: 256,
-		},
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for":       "2s",
+		"stdout_string": expected,
 	}
 
-	// Wait for the client to connect
-	testutil.WaitForResult(func() (bool, error) {
-		node, err := s.State().NodeByID(nil, c.NodeID())
-		if err != nil {
-			return false, err
-		}
-		if node == nil {
-			return false, fmt.Errorf("unknown node")
-		}
-
-		return node.Status == structs.NodeStatusReady, fmt.Errorf("bad node status")
-	}, func(err error) {
-		t.Fatal(err)
-	})
-
-	// Upsert the allocation
-	state := s.State()
-	require.Nil(state.UpsertJob(999, a.Job))
-	require.Nil(state.UpsertAllocs(1003, []*structs.Allocation{a}))
-
-	// Wait for the client to run the allocation
-	testutil.WaitForResult(func() (bool, error) {
-		alloc, err := state.AllocByID(nil, a.ID)
-		if err != nil {
-			return false, err
-		}
-		if alloc == nil {
-			return false, fmt.Errorf("unknown alloc")
-		}
-		if alloc.ClientStatus != structs.AllocClientStatusComplete {
-			return false, fmt.Errorf("alloc client status: %v", alloc.ClientStatus)
-		}
-
-		return true, nil
-	}, func(err error) {
-		t.Fatalf("Alloc on node %q not finished: %v", c.NodeID(), err)
-	})
+	// Wait for alloc to be running
+	alloc := testutil.WaitForRunning(t, s.RPC, job)[0]
 
 	// Make the request
 	req := &cstructs.FsStreamRequest{
-		AllocID:      a.ID,
+		AllocID:      alloc.ID,
 		Path:         "alloc/logs/web.stdout.0",
 		PlainText:    true,
 		QueryOptions: structs.QueryOptions{Region: "global"},
@@ -637,6 +617,9 @@ OUTER:
 	}
 
 	testutil.WaitForResult(func() (bool, error) {
+		pipeChecker.l.Lock()
+		defer pipeChecker.l.Unlock()
+
 		return pipeChecker.Closed, nil
 	}, func(err error) {
 		t.Fatal("Pipe not closed")
@@ -645,11 +628,14 @@ OUTER:
 
 type ReadWriteCloseChecker struct {
 	io.ReadWriteCloser
+	l      sync.Mutex
 	Closed bool
 }
 
 func (r *ReadWriteCloseChecker) Close() error {
+	r.l.Lock()
 	r.Closed = true
+	r.l.Unlock()
 	return r.ReadWriteCloser.Close()
 }
 
@@ -662,76 +648,29 @@ func TestFS_Stream_Follow(t *testing.T) {
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	c := TestClient(t, func(c *config.Config) {
+	c, cleanup := TestClient(t, func(c *config.Config) {
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer c.Shutdown()
+	defer cleanup()
 
-	// Force an allocation onto the node
 	expectedBase := "Hello from the other side"
 	repeat := 10
 
-	a := mock.Alloc()
-	a.Job.Type = structs.JobTypeBatch
-	a.NodeID = c.NodeID()
-	a.Job.TaskGroups[0].Count = 1
-	a.Job.TaskGroups[0].Tasks[0] = &structs.Task{
-		Name:   "web",
-		Driver: "mock_driver",
-		Config: map[string]interface{}{
-			"run_for":                "20s",
-			"stdout_string":          expectedBase,
-			"stdout_repeat":          repeat,
-			"stdout_repeat_duration": 200 * time.Millisecond,
-		},
-		LogConfig: structs.DefaultLogConfig(),
-		Resources: &structs.Resources{
-			CPU:      500,
-			MemoryMB: 256,
-		},
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for":                "20s",
+		"stdout_string":          expectedBase,
+		"stdout_repeat":          repeat,
+		"stdout_repeat_duration": "200ms",
 	}
 
-	// Wait for the client to connect
-	testutil.WaitForResult(func() (bool, error) {
-		node, err := s.State().NodeByID(nil, c.NodeID())
-		if err != nil {
-			return false, err
-		}
-		if node == nil {
-			return false, fmt.Errorf("unknown node")
-		}
-
-		return node.Status == structs.NodeStatusReady, fmt.Errorf("bad node status")
-	}, func(err error) {
-		t.Fatal(err)
-	})
-
-	// Upsert the allocation
-	state := s.State()
-	require.Nil(state.UpsertJob(999, a.Job))
-	require.Nil(state.UpsertAllocs(1003, []*structs.Allocation{a}))
-
-	// Wait for the client to run the allocation
-	testutil.WaitForResult(func() (bool, error) {
-		alloc, err := state.AllocByID(nil, a.ID)
-		if err != nil {
-			return false, err
-		}
-		if alloc == nil {
-			return false, fmt.Errorf("unknown alloc")
-		}
-		if alloc.ClientStatus != structs.AllocClientStatusRunning {
-			return false, fmt.Errorf("alloc client status: %v", alloc.ClientStatus)
-		}
-
-		return true, nil
-	}, func(err error) {
-		t.Fatalf("Alloc on node %q not running: %v", c.NodeID(), err)
-	})
+	// Wait for alloc to be running
+	alloc := testutil.WaitForRunning(t, s.RPC, job)[0]
 
 	// Make the request
 	req := &cstructs.FsStreamRequest{
-		AllocID:      a.ID,
+		AllocID:      alloc.ID,
 		Path:         "alloc/logs/web.stdout.0",
 		PlainText:    true,
 		Follow:       true,
@@ -806,74 +745,27 @@ func TestFS_Stream_Limit(t *testing.T) {
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	c := TestClient(t, func(c *config.Config) {
+	c, cleanup := TestClient(t, func(c *config.Config) {
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer c.Shutdown()
+	defer cleanup()
 
-	// Force an allocation onto the node
 	var limit int64 = 5
 	full := "Hello from the other side"
 	expected := full[:limit]
-	a := mock.Alloc()
-	a.Job.Type = structs.JobTypeBatch
-	a.NodeID = c.NodeID()
-	a.Job.TaskGroups[0].Count = 1
-	a.Job.TaskGroups[0].Tasks[0] = &structs.Task{
-		Name:   "web",
-		Driver: "mock_driver",
-		Config: map[string]interface{}{
-			"run_for":       "2s",
-			"stdout_string": full,
-		},
-		LogConfig: structs.DefaultLogConfig(),
-		Resources: &structs.Resources{
-			CPU:      500,
-			MemoryMB: 256,
-		},
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for":       "2s",
+		"stdout_string": full,
 	}
 
-	// Wait for the client to connect
-	testutil.WaitForResult(func() (bool, error) {
-		node, err := s.State().NodeByID(nil, c.NodeID())
-		if err != nil {
-			return false, err
-		}
-		if node == nil {
-			return false, fmt.Errorf("unknown node")
-		}
-
-		return node.Status == structs.NodeStatusReady, fmt.Errorf("bad node status")
-	}, func(err error) {
-		t.Fatal(err)
-	})
-
-	// Upsert the allocation
-	state := s.State()
-	require.Nil(state.UpsertJob(999, a.Job))
-	require.Nil(state.UpsertAllocs(1003, []*structs.Allocation{a}))
-
-	// Wait for the client to run the allocation
-	testutil.WaitForResult(func() (bool, error) {
-		alloc, err := state.AllocByID(nil, a.ID)
-		if err != nil {
-			return false, err
-		}
-		if alloc == nil {
-			return false, fmt.Errorf("unknown alloc")
-		}
-		if alloc.ClientStatus != structs.AllocClientStatusComplete {
-			return false, fmt.Errorf("alloc client status: %v", alloc.ClientStatus)
-		}
-
-		return true, nil
-	}, func(err error) {
-		t.Fatalf("Alloc on node %q not finished: %v", c.NodeID(), err)
-	})
+	// Wait for alloc to be running
+	alloc := testutil.WaitForRunning(t, s.RPC, job)[0]
 
 	// Make the request
 	req := &cstructs.FsStreamRequest{
-		AllocID:      a.ID,
+		AllocID:      alloc.ID,
 		Path:         "alloc/logs/web.stdout.0",
 		PlainText:    true,
 		Limit:        limit,
@@ -943,8 +835,8 @@ func TestFS_Logs_NoAlloc(t *testing.T) {
 	require := require.New(t)
 
 	// Start a client
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Make the request with bad allocation id
 	req := &cstructs.FsLogsRequest{
@@ -1014,6 +906,123 @@ OUTER:
 	}
 }
 
+// TestFS_Logs_TaskPending asserts that trying to stream logs for tasks which
+// have not started returns a 404 error.
+func TestFS_Logs_TaskPending(t *testing.T) {
+	t.Parallel()
+	require := require.New(t)
+
+	// Start a server and client
+	s := nomad.TestServer(t, nil)
+	defer s.Shutdown()
+	testutil.WaitForLeader(t, s.RPC)
+
+	c, cleanup := TestClient(t, func(c *config.Config) {
+		c.Servers = []string{s.GetConfig().RPCAddr.String()}
+	})
+	defer cleanup()
+
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"start_block_for": "10s",
+	}
+
+	// Register job
+	args := &structs.JobRegisterRequest{}
+	args.Job = job
+	args.WriteRequest.Region = "global"
+	args.Namespace = job.Namespace
+	var jobResp structs.JobRegisterResponse
+	require.NoError(s.RPC("Job.Register", args, &jobResp))
+
+	// Get the allocation ID
+	var allocID string
+	testutil.WaitForResult(func() (bool, error) {
+		args := structs.AllocListRequest{}
+		args.Region = "global"
+		resp := structs.AllocListResponse{}
+		if err := s.RPC("Alloc.List", &args, &resp); err != nil {
+			return false, err
+		}
+
+		if len(resp.Allocations) != 1 {
+			return false, fmt.Errorf("expected 1 alloc, found %d", len(resp.Allocations))
+		}
+
+		allocID = resp.Allocations[0].ID
+
+		// wait for alloc runner to be created; otherwise, we get no alloc found error
+		if _, err := c.getAllocRunner(allocID); err != nil {
+			return false, fmt.Errorf("alloc runner was not created yet for %v", allocID)
+		}
+
+		return true, nil
+	}, func(err error) {
+		t.Fatalf("error getting alloc id: %v", err)
+	})
+
+	// Make the request
+	req := &cstructs.FsLogsRequest{
+		AllocID:      allocID,
+		Task:         job.TaskGroups[0].Tasks[0].Name,
+		LogType:      "stdout",
+		Origin:       "start",
+		PlainText:    true,
+		QueryOptions: structs.QueryOptions{Region: "global"},
+	}
+
+	// Get the handler
+	handler, err := c.StreamingRpcHandler("FileSystem.Logs")
+	require.Nil(err)
+
+	// Create a pipe
+	p1, p2 := net.Pipe()
+	defer p1.Close()
+	defer p2.Close()
+
+	errCh := make(chan error)
+	streamMsg := make(chan *cstructs.StreamErrWrapper)
+
+	// Start the handler
+	go handler(p2)
+
+	// Start the decoder
+	go func() {
+		decoder := codec.NewDecoder(p1, structs.MsgpackHandle)
+		for {
+			var msg cstructs.StreamErrWrapper
+			if err := decoder.Decode(&msg); err != nil {
+				if err == io.EOF || strings.Contains(err.Error(), "closed") {
+					return
+				}
+				errCh <- fmt.Errorf("error decoding: %v", err)
+			}
+
+			streamMsg <- &msg
+		}
+	}()
+
+	// Send the request
+	encoder := codec.NewEncoder(p1, structs.MsgpackHandle)
+	require.Nil(encoder.Encode(req))
+
+	for {
+		select {
+		case <-time.After(3 * time.Second):
+			t.Fatal("timeout")
+		case err := <-errCh:
+			t.Fatalf("unexpected stream error: %v", err)
+		case msg := <-streamMsg:
+			require.NotNil(msg.Error)
+			require.NotNil(msg.Error.Code)
+			require.EqualValues(404, *msg.Error.Code)
+			require.Contains(msg.Error.Message, "not started")
+			return
+		}
+	}
+}
+
 func TestFS_Logs_ACL(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
@@ -1023,11 +1032,11 @@ func TestFS_Logs_ACL(t *testing.T) {
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	client := TestClient(t, func(c *config.Config) {
+	client, cleanup := TestClient(t, func(c *config.Config) {
 		c.ACLEnabled = true
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer client.Shutdown()
+	defer cleanup()
 
 	// Create a bad token
 	policyBad := mock.NamespacePolicy("other", "", []string{acl.NamespaceCapabilityReadFS})
@@ -1037,6 +1046,15 @@ func TestFS_Logs_ACL(t *testing.T) {
 		[]string{acl.NamespaceCapabilityReadLogs, acl.NamespaceCapabilityReadFS})
 	tokenGood := mock.CreatePolicyAndToken(t, s.State(), 1009, "valid2", policyGood)
 
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for": "20s",
+	}
+
+	// Wait for client to be running job
+	alloc := testutil.WaitForRunningWithToken(t, s.RPC, job, root.SecretID)[0]
+
 	cases := []struct {
 		Name          string
 		Token         string
@@ -1048,14 +1066,12 @@ func TestFS_Logs_ACL(t *testing.T) {
 			ExpectedError: structs.ErrPermissionDenied.Error(),
 		},
 		{
-			Name:          "good token",
-			Token:         tokenGood.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "good token",
+			Token: tokenGood.SecretID,
 		},
 		{
-			Name:          "root token",
-			Token:         root.SecretID,
-			ExpectedError: structs.ErrUnknownAllocationPrefix,
+			Name:  "root token",
+			Token: root.SecretID,
 		},
 	}
 
@@ -1063,8 +1079,8 @@ func TestFS_Logs_ACL(t *testing.T) {
 		t.Run(c.Name, func(t *testing.T) {
 			// Make the request with bad allocation id
 			req := &cstructs.FsLogsRequest{
-				AllocID: uuid.Generate(),
-				Task:    "foo",
+				AllocID: alloc.ID,
+				Task:    job.TaskGroups[0].Tasks[0].Name,
 				LogType: "stdout",
 				Origin:  "start",
 				QueryOptions: structs.QueryOptions{
@@ -1095,10 +1111,8 @@ func TestFS_Logs_ACL(t *testing.T) {
 				for {
 					var msg cstructs.StreamErrWrapper
 					if err := decoder.Decode(&msg); err != nil {
-						if err == io.EOF || strings.Contains(err.Error(), "closed") {
-							return
-						}
-						errCh <- fmt.Errorf("error decoding: %v", err)
+						errCh <- err
+						return
 					}
 
 					streamMsg <- &msg
@@ -1117,6 +1131,11 @@ func TestFS_Logs_ACL(t *testing.T) {
 				case <-timeout:
 					t.Fatal("timeout")
 				case err := <-errCh:
+					eof := err == io.EOF || strings.Contains(err.Error(), "closed")
+					if c.ExpectedError == "" && eof {
+						// No error was expected!
+						return
+					}
 					t.Fatal(err)
 				case msg := <-streamMsg:
 					if msg.Error == nil {
@@ -1124,6 +1143,7 @@ func TestFS_Logs_ACL(t *testing.T) {
 					}
 
 					if strings.Contains(msg.Error.Error(), c.ExpectedError) {
+						// Ok! Error matched expectation.
 						break OUTER
 					} else {
 						t.Fatalf("Bad error: %v", msg.Error)
@@ -1143,73 +1163,34 @@ func TestFS_Logs(t *testing.T) {
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	c := TestClient(t, func(c *config.Config) {
+	c, cleanup := TestClient(t, func(c *config.Config) {
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer c.Shutdown()
+	defer cleanup()
 
-	// Force an allocation onto the node
-	expected := "Hello from the other side"
-	a := mock.Alloc()
-	a.Job.Type = structs.JobTypeBatch
-	a.NodeID = c.NodeID()
-	a.Job.TaskGroups[0].Count = 1
-	a.Job.TaskGroups[0].Tasks[0] = &structs.Task{
-		Name:   "web",
-		Driver: "mock_driver",
-		Config: map[string]interface{}{
-			"run_for":       "2s",
-			"stdout_string": expected,
-		},
-		LogConfig: structs.DefaultLogConfig(),
-		Resources: &structs.Resources{
-			CPU:      500,
-			MemoryMB: 256,
-		},
+	expected := "Hello from the other side\n"
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for":       "2s",
+		"stdout_string": expected,
 	}
 
-	// Wait for the client to connect
-	testutil.WaitForResult(func() (bool, error) {
-		node, err := s.State().NodeByID(nil, c.NodeID())
-		if err != nil {
-			return false, err
-		}
-		if node == nil {
-			return false, fmt.Errorf("unknown node")
-		}
+	// Wait for client to be running job
+	testutil.WaitForRunning(t, s.RPC, job)
 
-		return node.Status == structs.NodeStatusReady, fmt.Errorf("bad node status")
-	}, func(err error) {
-		t.Fatal(err)
-	})
-
-	// Upsert the allocation
-	state := s.State()
-	require.Nil(state.UpsertJob(999, a.Job))
-	require.Nil(state.UpsertAllocs(1003, []*structs.Allocation{a}))
-
-	// Wait for the client to run the allocation
-	testutil.WaitForResult(func() (bool, error) {
-		alloc, err := state.AllocByID(nil, a.ID)
-		if err != nil {
-			return false, err
-		}
-		if alloc == nil {
-			return false, fmt.Errorf("unknown alloc")
-		}
-		if alloc.ClientStatus != structs.AllocClientStatusComplete {
-			return false, fmt.Errorf("alloc client status: %v", alloc.ClientStatus)
-		}
-
-		return true, nil
-	}, func(err error) {
-		t.Fatalf("Alloc on node %q not finished: %v", c.NodeID(), err)
-	})
+	// Get the allocation ID
+	args := structs.AllocListRequest{}
+	args.Region = "global"
+	resp := structs.AllocListResponse{}
+	require.NoError(s.RPC("Alloc.List", &args, &resp))
+	require.Len(resp.Allocations, 1)
+	allocID := resp.Allocations[0].ID
 
 	// Make the request
 	req := &cstructs.FsLogsRequest{
-		AllocID:      a.ID,
-		Task:         a.Job.TaskGroups[0].Tasks[0].Name,
+		AllocID:      allocID,
+		Task:         job.TaskGroups[0].Tasks[0].Name,
 		LogType:      "stdout",
 		Origin:       "start",
 		PlainText:    true,
@@ -1283,77 +1264,30 @@ func TestFS_Logs_Follow(t *testing.T) {
 	defer s.Shutdown()
 	testutil.WaitForLeader(t, s.RPC)
 
-	c := TestClient(t, func(c *config.Config) {
+	c, cleanup := TestClient(t, func(c *config.Config) {
 		c.Servers = []string{s.GetConfig().RPCAddr.String()}
 	})
-	defer c.Shutdown()
+	defer cleanup()
 
-	// Force an allocation onto the node
-	expectedBase := "Hello from the other side"
+	expectedBase := "Hello from the other side\n"
 	repeat := 10
 
-	a := mock.Alloc()
-	a.Job.Type = structs.JobTypeBatch
-	a.NodeID = c.NodeID()
-	a.Job.TaskGroups[0].Count = 1
-	a.Job.TaskGroups[0].Tasks[0] = &structs.Task{
-		Name:   "web",
-		Driver: "mock_driver",
-		Config: map[string]interface{}{
-			"run_for":                "20s",
-			"stdout_string":          expectedBase,
-			"stdout_repeat":          repeat,
-			"stdout_repeat_duration": 200 * time.Millisecond,
-		},
-		LogConfig: structs.DefaultLogConfig(),
-		Resources: &structs.Resources{
-			CPU:      500,
-			MemoryMB: 256,
-		},
+	job := mock.BatchJob()
+	job.TaskGroups[0].Count = 1
+	job.TaskGroups[0].Tasks[0].Config = map[string]interface{}{
+		"run_for":                "20s",
+		"stdout_string":          expectedBase,
+		"stdout_repeat":          repeat,
+		"stdout_repeat_duration": "200ms",
 	}
 
-	// Wait for the client to connect
-	testutil.WaitForResult(func() (bool, error) {
-		node, err := s.State().NodeByID(nil, c.NodeID())
-		if err != nil {
-			return false, err
-		}
-		if node == nil {
-			return false, fmt.Errorf("unknown node")
-		}
-
-		return node.Status == structs.NodeStatusReady, fmt.Errorf("bad node status")
-	}, func(err error) {
-		t.Fatal(err)
-	})
-
-	// Upsert the allocation
-	state := s.State()
-	require.Nil(state.UpsertJob(999, a.Job))
-	require.Nil(state.UpsertAllocs(1003, []*structs.Allocation{a}))
-
-	// Wait for the client to run the allocation
-	testutil.WaitForResult(func() (bool, error) {
-		alloc, err := state.AllocByID(nil, a.ID)
-		if err != nil {
-			return false, err
-		}
-		if alloc == nil {
-			return false, fmt.Errorf("unknown alloc")
-		}
-		if alloc.ClientStatus != structs.AllocClientStatusRunning {
-			return false, fmt.Errorf("alloc client status: %v", alloc.ClientStatus)
-		}
-
-		return true, nil
-	}, func(err error) {
-		t.Fatalf("Alloc on node %q not running: %v", c.NodeID(), err)
-	})
+	// Wait for client to be running job
+	alloc := testutil.WaitForRunning(t, s.RPC, job)[0]
 
 	// Make the request
 	req := &cstructs.FsLogsRequest{
-		AllocID:      a.ID,
-		Task:         a.Job.TaskGroups[0].Tasks[0].Name,
+		AllocID:      alloc.ID,
+		Task:         job.TaskGroups[0].Tasks[0].Name,
 		LogType:      "stdout",
 		Origin:       "start",
 		PlainText:    true,
@@ -1363,7 +1297,7 @@ func TestFS_Logs_Follow(t *testing.T) {
 
 	// Get the handler
 	handler, err := c.StreamingRpcHandler("FileSystem.Logs")
-	require.Nil(err)
+	require.NoError(err)
 
 	// Create a pipe
 	p1, p2 := net.Pipe()
@@ -1628,8 +1562,8 @@ func TestFS_findClosest(t *testing.T) {
 func TestFS_streamFile_NoFile(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	ad := tempAllocDir(t)
 	defer os.RemoveAll(ad.AllocDir)
@@ -1642,14 +1576,18 @@ func TestFS_streamFile_NoFile(t *testing.T) {
 	err := c.endpoints.FileSystem.streamFile(
 		context.Background(), 0, "foo", 0, ad, framer, nil)
 	require.NotNil(err)
-	require.Contains(err.Error(), "no such file")
+	if runtime.GOOS == "windows" {
+		require.Contains(err.Error(), "cannot find the file")
+	} else {
+		require.Contains(err.Error(), "no such file")
+	}
 }
 
 func TestFS_streamFile_Modify(t *testing.T) {
 	t.Parallel()
 
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Get a temp alloc dir
 	ad := tempAllocDir(t)
@@ -1717,8 +1655,8 @@ func TestFS_streamFile_Modify(t *testing.T) {
 
 func TestFS_streamFile_Truncate(t *testing.T) {
 	t.Parallel()
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Get a temp alloc dir
 	ad := tempAllocDir(t)
@@ -1736,6 +1674,7 @@ func TestFS_streamFile_Truncate(t *testing.T) {
 
 	// Start the reader
 	truncateCh := make(chan struct{})
+	truncateClosed := false
 	dataPostTruncCh := make(chan struct{})
 	frames := make(chan *sframer.StreamFrame, 4)
 	go func() {
@@ -1746,8 +1685,9 @@ func TestFS_streamFile_Truncate(t *testing.T) {
 				continue
 			}
 
-			if frame.FileEvent == truncateEvent {
+			if frame.FileEvent == truncateEvent && !truncateClosed {
 				close(truncateCh)
+				truncateClosed = true
 			}
 
 			collected = append(collected, frame.Data...)
@@ -1818,10 +1758,13 @@ func TestFS_streamFile_Truncate(t *testing.T) {
 }
 
 func TestFS_streamImpl_Delete(t *testing.T) {
+	if runtime.GOOS == "windows" {
+		t.Skip("Windows does not allow us to delete a file while it is open")
+	}
 	t.Parallel()
 
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Get a temp alloc dir
 	ad := tempAllocDir(t)
@@ -1842,7 +1785,11 @@ func TestFS_streamImpl_Delete(t *testing.T) {
 	frames := make(chan *sframer.StreamFrame, 4)
 	go func() {
 		for {
-			frame := <-frames
+			frame, ok := <-frames
+			if !ok {
+				return
+			}
+
 			if frame.IsHeartbeat() {
 				continue
 			}
@@ -1888,8 +1835,8 @@ func TestFS_streamImpl_Delete(t *testing.T) {
 func TestFS_logsImpl_NoFollow(t *testing.T) {
 	t.Parallel()
 
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Get a temp alloc dir and create the log dir
 	ad := tempAllocDir(t)
@@ -1955,8 +1902,8 @@ func TestFS_logsImpl_NoFollow(t *testing.T) {
 func TestFS_logsImpl_Follow(t *testing.T) {
 	t.Parallel()
 
-	c := TestClient(t, nil)
-	defer c.Shutdown()
+	c, cleanup := TestClient(t, nil)
+	defer cleanup()
 
 	// Get a temp alloc dir and create the log dir
 	ad := tempAllocDir(t)
