@@ -2,7 +2,6 @@ package agent
 
 import (
 	"io/ioutil"
-	"log"
 	"os"
 	"path/filepath"
 	"strings"
@@ -11,7 +10,9 @@ import (
 
 	cstructs "github.com/hashicorp/nomad/client/structs"
 	"github.com/hashicorp/nomad/helper"
+	"github.com/hashicorp/nomad/helper/testlog"
 	"github.com/hashicorp/nomad/nomad/structs"
+	"github.com/hashicorp/nomad/nomad/structs/config"
 	sconfig "github.com/hashicorp/nomad/nomad/structs/config"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
@@ -53,34 +54,21 @@ func TestAgent_ServerConfig(t *testing.T) {
 		t.Fatalf("error normalizing config: %v", err)
 	}
 	out, err := a.serverConfig()
-	if err != nil {
-		t.Fatalf("err: %s", err)
-	}
+	require.NoError(t, err)
+
 	serfAddr := out.SerfConfig.MemberlistConfig.AdvertiseAddr
-	if serfAddr != "127.0.0.1" {
-		t.Fatalf("expect 127.0.0.1, got: %s", serfAddr)
-	}
+	require.Equal(t, "127.0.0.1", serfAddr)
+
 	serfPort := out.SerfConfig.MemberlistConfig.AdvertisePort
-	if serfPort != 4000 {
-		t.Fatalf("expected 4000, got: %d", serfPort)
-	}
-	if out.AuthoritativeRegion != "global" {
-		t.Fatalf("bad: %#v", out.AuthoritativeRegion)
-	}
-	if !out.ACLEnabled {
-		t.Fatalf("ACL not enabled")
-	}
+	require.Equal(t, 4000, serfPort)
+
+	require.Equal(t, "global", out.AuthoritativeRegion)
+	require.True(t, out.ACLEnabled)
 
 	// Assert addresses weren't changed
-	if addr := conf.AdvertiseAddrs.RPC; addr != "127.0.0.1:4001" {
-		t.Fatalf("bad rpc advertise addr: %#v", addr)
-	}
-	if addr := conf.AdvertiseAddrs.HTTP; addr != "10.10.11.1:4005" {
-		t.Fatalf("expect 10.11.11.1:4005, got: %v", addr)
-	}
-	if addr := conf.Addresses.RPC; addr != "0.0.0.0" {
-		t.Fatalf("expect 0.0.0.0, got: %v", addr)
-	}
+	require.Equal(t, "127.0.0.1:4001", conf.AdvertiseAddrs.RPC)
+	require.Equal(t, "10.10.11.1:4005", conf.AdvertiseAddrs.HTTP)
+	require.Equal(t, "0.0.0.0", conf.Addresses.RPC)
 
 	// Sets up the ports properly
 	conf.Addresses.RPC = ""
@@ -88,19 +76,12 @@ func TestAgent_ServerConfig(t *testing.T) {
 	conf.Ports.RPC = 4003
 	conf.Ports.Serf = 4004
 
-	if err := conf.normalizeAddrs(); err != nil {
-		t.Fatalf("error normalizing config: %v", err)
-	}
+	require.NoError(t, conf.normalizeAddrs())
+
 	out, err = a.serverConfig()
-	if err != nil {
-		t.Fatalf("err: %s", err)
-	}
-	if addr := out.RPCAddr.Port; addr != 4003 {
-		t.Fatalf("expect 4003, got: %d", out.RPCAddr.Port)
-	}
-	if port := out.SerfConfig.MemberlistConfig.BindPort; port != 4004 {
-		t.Fatalf("expect 4004, got: %d", port)
-	}
+	require.NoError(t, err)
+	require.Equal(t, 4003, out.RPCAddr.Port)
+	require.Equal(t, 4004, out.SerfConfig.MemberlistConfig.BindPort)
 
 	// Prefers advertise over bind addr
 	conf.BindAddr = "127.0.0.3"
@@ -111,88 +92,51 @@ func TestAgent_ServerConfig(t *testing.T) {
 	conf.AdvertiseAddrs.RPC = ""
 	conf.AdvertiseAddrs.Serf = "10.0.0.12:4004"
 
-	if err := conf.normalizeAddrs(); err != nil {
-		t.Fatalf("error normalizing config: %v", err)
-	}
+	require.NoError(t, conf.normalizeAddrs())
+
 	out, err = a.serverConfig()
-	if err != nil {
-		t.Fatalf("err: %s", err)
-	}
-	if addr := out.RPCAddr.IP.String(); addr != "127.0.0.2" {
-		t.Fatalf("expect 127.0.0.2, got: %s", addr)
-	}
-	if port := out.RPCAddr.Port; port != 4003 {
-		t.Fatalf("expect 4647, got: %d", port)
-	}
-	if addr := out.SerfConfig.MemberlistConfig.BindAddr; addr != "127.0.0.2" {
-		t.Fatalf("expect 127.0.0.2, got: %s", addr)
-	}
-	if port := out.SerfConfig.MemberlistConfig.BindPort; port != 4004 {
-		t.Fatalf("expect 4648, got: %d", port)
-	}
-	if addr := conf.Addresses.HTTP; addr != "127.0.0.2" {
-		t.Fatalf("expect 127.0.0.2, got: %s", addr)
-	}
-	if addr := conf.Addresses.RPC; addr != "127.0.0.2" {
-		t.Fatalf("expect 127.0.0.2, got: %s", addr)
-	}
-	if addr := conf.Addresses.Serf; addr != "127.0.0.2" {
-		t.Fatalf("expect 10.0.0.12, got: %s", addr)
-	}
-	if addr := conf.normalizedAddrs.HTTP; addr != "127.0.0.2:4646" {
-		t.Fatalf("expect 127.0.0.2:4646, got: %s", addr)
-	}
-	if addr := conf.normalizedAddrs.RPC; addr != "127.0.0.2:4003" {
-		t.Fatalf("expect 127.0.0.2:4003, got: %s", addr)
-	}
-	if addr := conf.normalizedAddrs.Serf; addr != "127.0.0.2:4004" {
-		t.Fatalf("expect 10.0.0.12:4004, got: %s", addr)
-	}
-	if addr := conf.AdvertiseAddrs.HTTP; addr != "10.0.0.10:4646" {
-		t.Fatalf("expect 10.0.0.10:4646, got: %s", addr)
-	}
-	if addr := conf.AdvertiseAddrs.RPC; addr != "127.0.0.2:4003" {
-		t.Fatalf("expect 127.0.0.2:4003, got: %s", addr)
-	}
-	if addr := conf.AdvertiseAddrs.Serf; addr != "10.0.0.12:4004" {
-		t.Fatalf("expect 10.0.0.12:4004, got: %s", addr)
-	}
+	require.Equal(t, "127.0.0.2", out.RPCAddr.IP.String())
+	require.Equal(t, 4003, out.RPCAddr.Port)
+	require.Equal(t, "127.0.0.2", out.SerfConfig.MemberlistConfig.BindAddr)
+	require.Equal(t, 4004, out.SerfConfig.MemberlistConfig.BindPort)
+	require.Equal(t, "127.0.0.2", conf.Addresses.HTTP)
+	require.Equal(t, "127.0.0.2", conf.Addresses.RPC)
+	require.Equal(t, "127.0.0.2", conf.Addresses.Serf)
+	require.Equal(t, "127.0.0.2:4646", conf.normalizedAddrs.HTTP)
+	require.Equal(t, "127.0.0.2:4003", conf.normalizedAddrs.RPC)
+	require.Equal(t, "127.0.0.2:4004", conf.normalizedAddrs.Serf)
+	require.Equal(t, "10.0.0.10:4646", conf.AdvertiseAddrs.HTTP)
+	require.Equal(t, "127.0.0.2:4003", conf.AdvertiseAddrs.RPC)
+	require.Equal(t, "10.0.0.12:4004", conf.AdvertiseAddrs.Serf)
 
 	conf.Server.NodeGCThreshold = "42g"
-	if err := conf.normalizeAddrs(); err != nil {
-		t.Fatalf("error normalizing config: %v", err)
-	}
-	out, err = a.serverConfig()
+	require.NoError(t, conf.normalizeAddrs())
+
+	_, err = a.serverConfig()
 	if err == nil || !strings.Contains(err.Error(), "unknown unit") {
 		t.Fatalf("expected unknown unit error, got: %#v", err)
 	}
 
 	conf.Server.NodeGCThreshold = "10s"
-	if err := conf.normalizeAddrs(); err != nil {
-		t.Fatalf("error normalizing config: %v", err)
-	}
+	require.NoError(t, conf.normalizeAddrs())
 	out, err = a.serverConfig()
-	if threshold := out.NodeGCThreshold; threshold != time.Second*10 {
-		t.Fatalf("expect 10s, got: %s", threshold)
-	}
+	require.NoError(t, err)
+	require.Equal(t, 10*time.Second, out.NodeGCThreshold)
 
 	conf.Server.HeartbeatGrace = 37 * time.Second
 	out, err = a.serverConfig()
-	if threshold := out.HeartbeatGrace; threshold != time.Second*37 {
-		t.Fatalf("expect 37s, got: %s", threshold)
-	}
+	require.NoError(t, err)
+	require.Equal(t, 37*time.Second, out.HeartbeatGrace)
 
 	conf.Server.MinHeartbeatTTL = 37 * time.Second
 	out, err = a.serverConfig()
-	if min := out.MinHeartbeatTTL; min != time.Second*37 {
-		t.Fatalf("expect 37s, got: %s", min)
-	}
+	require.NoError(t, err)
+	require.Equal(t, 37*time.Second, out.MinHeartbeatTTL)
 
 	conf.Server.MaxHeartbeatsPerSecond = 11.0
 	out, err = a.serverConfig()
-	if max := out.MaxHeartbeatsPerSecond; max != 11.0 {
-		t.Fatalf("expect 11, got: %v", max)
-	}
+	require.NoError(t, err)
+	require.Equal(t, float64(11.0), out.MaxHeartbeatsPerSecond)
 
 	// Defaults to the global bind addr
 	conf.Addresses.RPC = ""
@@ -204,110 +148,161 @@ func TestAgent_ServerConfig(t *testing.T) {
 	conf.Ports.HTTP = 4646
 	conf.Ports.RPC = 4647
 	conf.Ports.Serf = 4648
-	if err := conf.normalizeAddrs(); err != nil {
-		t.Fatalf("error normalizing config: %v", err)
-	}
+	require.NoError(t, conf.normalizeAddrs())
+
 	out, err = a.serverConfig()
-	if err != nil {
-		t.Fatalf("err: %s", err)
-	}
-	if addr := out.RPCAddr.IP.String(); addr != "127.0.0.3" {
-		t.Fatalf("expect 127.0.0.3, got: %s", addr)
-	}
-	if addr := out.SerfConfig.MemberlistConfig.BindAddr; addr != "127.0.0.3" {
-		t.Fatalf("expect 127.0.0.3, got: %s", addr)
-	}
-	if addr := conf.Addresses.HTTP; addr != "127.0.0.3" {
-		t.Fatalf("expect 127.0.0.3, got: %s", addr)
-	}
-	if addr := conf.Addresses.RPC; addr != "127.0.0.3" {
-		t.Fatalf("expect 127.0.0.3, got: %s", addr)
-	}
-	if addr := conf.Addresses.Serf; addr != "127.0.0.3" {
-		t.Fatalf("expect 127.0.0.3, got: %s", addr)
-	}
-	if addr := conf.normalizedAddrs.HTTP; addr != "127.0.0.3:4646" {
-		t.Fatalf("expect 127.0.0.3:4646, got: %s", addr)
-	}
-	if addr := conf.normalizedAddrs.RPC; addr != "127.0.0.3:4647" {
-		t.Fatalf("expect 127.0.0.3:4647, got: %s", addr)
-	}
-	if addr := conf.normalizedAddrs.Serf; addr != "127.0.0.3:4648" {
-		t.Fatalf("expect 127.0.0.3:4648, got: %s", addr)
-	}
+	require.NoError(t, err)
+
+	require.Equal(t, "127.0.0.3", out.RPCAddr.IP.String())
+	require.Equal(t, "127.0.0.3", out.SerfConfig.MemberlistConfig.BindAddr)
+	require.Equal(t, "127.0.0.3", conf.Addresses.HTTP)
+	require.Equal(t, "127.0.0.3", conf.Addresses.RPC)
+	require.Equal(t, "127.0.0.3", conf.Addresses.Serf)
+	require.Equal(t, "127.0.0.3:4646", conf.normalizedAddrs.HTTP)
+	require.Equal(t, "127.0.0.3:4647", conf.normalizedAddrs.RPC)
+	require.Equal(t, "127.0.0.3:4648", conf.normalizedAddrs.Serf)
 
 	// Properly handles the bootstrap flags
 	conf.Server.BootstrapExpect = 1
 	out, err = a.serverConfig()
-	if err != nil {
-		t.Fatalf("err: %s", err)
-	}
-	if !out.Bootstrap {
-		t.Fatalf("should have set bootstrap mode")
-	}
-	if out.BootstrapExpect != 0 {
-		t.Fatalf("bootstrap expect should be 0")
-	}
+	require.NoError(t, err)
+	require.True(t, out.Bootstrap)
+	require.Equal(t, int32(0), out.BootstrapExpect)
 
 	conf.Server.BootstrapExpect = 3
 	out, err = a.serverConfig()
-	if err != nil {
-		t.Fatalf("err: %s", err)
+	require.NoError(t, err)
+	require.False(t, out.Bootstrap)
+	require.Equal(t, int32(3), out.BootstrapExpect)
+}
+
+// TestAgent_ServerConfig_Limits_Errors asserts invalid Limits configurations
+// cause errors. This is the server-only (RPC) counterpart to
+// TestHTTPServer_Limits_Error.
+func TestAgent_ServerConfig_Limits_Error(t *testing.T) {
+	t.Parallel()
+
+	cases := []struct {
+		name        string
+		expectedErr string
+		limits      sconfig.Limits
+	}{
+		{
+			name:        "Negative Timeout",
+			expectedErr: "rpc_handshake_timeout must be >= 0",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "-5s",
+				RPCMaxConnsPerClient: helper.IntToPtr(100),
+			},
+		},
+		{
+			name:        "Invalid Timeout",
+			expectedErr: "error parsing rpc_handshake_timeout",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "s",
+				RPCMaxConnsPerClient: helper.IntToPtr(100),
+			},
+		},
+		{
+			name:        "Missing Timeout",
+			expectedErr: "error parsing rpc_handshake_timeout",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "",
+				RPCMaxConnsPerClient: helper.IntToPtr(100),
+			},
+		},
+		{
+			name:        "Negative Connection Limit",
+			expectedErr: "rpc_max_conns_per_client must be > 25; found: -100",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "5s",
+				RPCMaxConnsPerClient: helper.IntToPtr(-100),
+			},
+		},
+		{
+			name:        "Low Connection Limit",
+			expectedErr: "rpc_max_conns_per_client must be > 25; found: 20",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "5s",
+				RPCMaxConnsPerClient: helper.IntToPtr(sconfig.LimitsNonStreamingConnsPerClient),
+			},
+		},
 	}
-	if out.Bootstrap {
-		t.Fatalf("bootstrap mode should be disabled")
-	}
-	if out.BootstrapExpect != 3 {
-		t.Fatalf("should have bootstrap-expect = 3")
+
+	for i := range cases {
+		tc := cases[i]
+		t.Run(tc.name, func(t *testing.T) {
+			conf := DevConfig(nil)
+			require.NoError(t, conf.normalizeAddrs())
+
+			conf.Limits = tc.limits
+			serverConf, err := convertServerConfig(conf)
+			assert.Nil(t, serverConf)
+			require.Contains(t, err.Error(), tc.expectedErr)
+		})
 	}
 }
 
-func TestAgent_ClientConfig(t *testing.T) {
+// TestAgent_ServerConfig_Limits_OK asserts valid Limits configurations do not
+// cause errors. This is the server-only (RPC) counterpart to
+// TestHTTPServer_Limits_OK.
+func TestAgent_ServerConfig_Limits_OK(t *testing.T) {
 	t.Parallel()
-	conf := DefaultConfig()
-	conf.Client.Enabled = true
 
-	// For Clients HTTP and RPC must be set (Serf can be skipped)
-	conf.Addresses.HTTP = "169.254.0.1"
-	conf.Addresses.RPC = "169.254.0.1"
-	conf.Ports.HTTP = 5678
-	a := &Agent{config: conf}
-
-	if err := conf.normalizeAddrs(); err != nil {
-		t.Fatalf("error normalizing config: %v", err)
-	}
-	c, err := a.clientConfig()
-	if err != nil {
-		t.Fatalf("got err: %v", err)
+	cases := []struct {
+		name   string
+		limits sconfig.Limits
+	}{
+		{
+			name:   "Default",
+			limits: config.DefaultLimits(),
+		},
+		{
+			name: "Zero+nil is valid to disable",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "0",
+				RPCMaxConnsPerClient: nil,
+			},
+		},
+		{
+			name: "Zeros are valid",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "0s",
+				RPCMaxConnsPerClient: helper.IntToPtr(0),
+			},
+		},
+		{
+			name: "Low limits are valid",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "1ms",
+				RPCMaxConnsPerClient: helper.IntToPtr(26),
+			},
+		},
+		{
+			name: "High limits are valid",
+			limits: sconfig.Limits{
+				RPCHandshakeTimeout:  "5h",
+				RPCMaxConnsPerClient: helper.IntToPtr(100000),
+			},
+		},
 	}
 
-	expectedHttpAddr := "169.254.0.1:5678"
-	if c.Node.HTTPAddr != expectedHttpAddr {
-		t.Fatalf("Expected http addr: %v, got: %v", expectedHttpAddr, c.Node.HTTPAddr)
-	}
+	for i := range cases {
+		tc := cases[i]
+		t.Run(tc.name, func(t *testing.T) {
+			conf := DevConfig(nil)
+			require.NoError(t, conf.normalizeAddrs())
 
-	conf = DefaultConfig()
-	conf.DevMode = true
-	a = &Agent{config: conf}
-	conf.Client.Enabled = true
-	conf.Addresses.HTTP = "169.254.0.1"
-
-	if err := conf.normalizeAddrs(); err != nil {
-		t.Fatalf("error normalizing config: %v", err)
-	}
-	c, err = a.clientConfig()
-	if err != nil {
-		t.Fatalf("got err: %v", err)
-	}
-
-	expectedHttpAddr = "169.254.0.1:4646"
-	if c.Node.HTTPAddr != expectedHttpAddr {
-		t.Fatalf("Expected http addr: %v, got: %v", expectedHttpAddr, c.Node.HTTPAddr)
+			conf.Limits = tc.limits
+			serverConf, err := convertServerConfig(conf)
+			assert.NoError(t, err)
+			require.NotNil(t, serverConf)
+		})
 	}
 }
 
 // Clients should inherit telemetry configuration
-func TestAget_Client_TelemetryConfiguration(t *testing.T) {
+func TestAgent_Client_TelemetryConfiguration(t *testing.T) {
 	assert := assert.New(t)
 
 	conf := DefaultConfig()
@@ -333,10 +328,7 @@ func TestAget_Client_TelemetryConfiguration(t *testing.T) {
 // API health check depending on configuration.
 func TestAgent_HTTPCheck(t *testing.T) {
 	t.Parallel()
-	logger := log.New(ioutil.Discard, "", 0)
-	if testing.Verbose() {
-		logger = log.New(os.Stdout, "[TestAgent_HTTPCheck] ", log.Lshortfile)
-	}
+	logger := testlog.HCLogger(t)
 	agent := func() *Agent {
 		return &Agent{
 			logger: logger,
@@ -416,15 +408,12 @@ func TestAgent_HTTPCheckPath(t *testing.T) {
 	t.Parallel()
 	// Agent.agentHTTPCheck only needs a config and logger
 	a := &Agent{
-		config: DevConfig(),
-		logger: log.New(ioutil.Discard, "", 0),
+		config: DevConfig(nil),
+		logger: testlog.HCLogger(t),
 	}
 	if err := a.config.normalizeAddrs(); err != nil {
 		t.Fatalf("error normalizing config: %v", err)
 	}
-	if testing.Verbose() {
-		a.logger = log.New(os.Stderr, "", log.LstdFlags)
-	}
 
 	// Assert server check uses /v1/agent/health?type=server
 	isServer := true
@@ -447,6 +436,28 @@ func TestAgent_HTTPCheckPath(t *testing.T) {
 	}
 }
 
+// Here we validate that log levels get updated when the configuration is
+// reloaded. I can't find a good way to fetch this from the logger itself, so
+// we pull it only from the agents configuration struct, not the logger.
+func TestAgent_Reload_LogLevel(t *testing.T) {
+	t.Parallel()
+	assert := assert.New(t)
+
+	agent := NewTestAgent(t, t.Name(), func(c *Config) {
+		c.LogLevel = "INFO"
+	})
+	defer agent.Shutdown()
+
+	assert.Equal("INFO", agent.GetConfig().LogLevel)
+
+	newConfig := &Config{
+		LogLevel: "TRACE",
+	}
+
+	assert.Nil(agent.Reload(newConfig))
+	assert.Equal("TRACE", agent.GetConfig().LogLevel)
+}
+
 // This test asserts that the keyloader embedded in the TLS config is shared
 // across the Agent, Server, and Client. This is essential for certificate
 // reloading to work.
@@ -638,7 +649,7 @@ func TestServer_Reload_TLS_WithNilConfiguration(t *testing.T) {
 	t.Parallel()
 	assert := assert.New(t)
 
-	logger := log.New(ioutil.Discard, "", 0)
+	logger := testlog.HCLogger(t)
 
 	agent := &Agent{
 		logger: logger,
@@ -662,7 +673,7 @@ func TestServer_Reload_TLS_UpgradeToTLS(t *testing.T) {
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
 
-	logger := log.New(ioutil.Discard, "", 0)
+	logger := testlog.HCLogger(t)
 
 	agentConfig := &Config{
 		TLSConfig: &sconfig.TLSConfig{},
@@ -704,7 +715,7 @@ func TestServer_Reload_TLS_DowngradeFromTLS(t *testing.T) {
 	dir := tmpDir(t)
 	defer os.RemoveAll(dir)
 
-	logger := log.New(ioutil.Discard, "", 0)
+	logger := testlog.HCLogger(t)
 
 	agentConfig := &Config{
 		TLSConfig: &sconfig.TLSConfig{
@@ -769,10 +780,9 @@ func TestServer_ShouldReload_ReturnFalseForNoChanges(t *testing.T) {
 	})
 	defer agent.Shutdown()
 
-	shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := agent.ShouldReload(sameAgentConfig)
+	shouldReloadAgent, shouldReloadHTTP := agent.ShouldReload(sameAgentConfig)
 	assert.False(shouldReloadAgent)
 	assert.False(shouldReloadHTTP)
-	assert.False(shouldReloadRPC)
 }
 
 func TestServer_ShouldReload_ReturnTrueForOnlyHTTPChanges(t *testing.T) {
@@ -810,10 +820,9 @@ func TestServer_ShouldReload_ReturnTrueForOnlyHTTPChanges(t *testing.T) {
 	})
 	defer agent.Shutdown()
 
-	shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := agent.ShouldReload(sameAgentConfig)
+	shouldReloadAgent, shouldReloadHTTP := agent.ShouldReload(sameAgentConfig)
 	require.True(shouldReloadAgent)
 	require.True(shouldReloadHTTP)
-	require.False(shouldReloadRPC)
 }
 
 func TestServer_ShouldReload_ReturnTrueForOnlyRPCChanges(t *testing.T) {
@@ -851,10 +860,9 @@ func TestServer_ShouldReload_ReturnTrueForOnlyRPCChanges(t *testing.T) {
 	})
 	defer agent.Shutdown()
 
-	shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := agent.ShouldReload(sameAgentConfig)
+	shouldReloadAgent, shouldReloadHTTP := agent.ShouldReload(sameAgentConfig)
 	assert.True(shouldReloadAgent)
 	assert.False(shouldReloadHTTP)
-	assert.True(shouldReloadRPC)
 }
 
 func TestServer_ShouldReload_ReturnTrueForConfigChanges(t *testing.T) {
@@ -894,10 +902,9 @@ func TestServer_ShouldReload_ReturnTrueForConfigChanges(t *testing.T) {
 		},
 	}
 
-	shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := agent.ShouldReload(newConfig)
+	shouldReloadAgent, shouldReloadHTTP := agent.ShouldReload(newConfig)
 	assert.True(shouldReloadAgent)
 	assert.True(shouldReloadHTTP)
-	assert.True(shouldReloadRPC)
 }
 
 func TestServer_ShouldReload_ReturnTrueForFileChanges(t *testing.T) {
@@ -927,7 +934,7 @@ func TestServer_ShouldReload_ReturnTrueForFileChanges(t *testing.T) {
 	content := []byte(oldCertificate)
 	dir, err := ioutil.TempDir("", "certificate")
 	if err != nil {
-		log.Fatal(err)
+		t.Fatal(err)
 	}
 	defer os.RemoveAll(dir) // clean up
 
@@ -940,7 +947,7 @@ func TestServer_ShouldReload_ReturnTrueForFileChanges(t *testing.T) {
 		key    = "../../helper/tlsutil/testdata/nomad-foo-key.pem"
 	)
 
-	logger := log.New(ioutil.Discard, "", 0)
+	logger := testlog.HCLogger(t)
 
 	agentConfig := &Config{
 		TLSConfig: &sconfig.TLSConfig{
@@ -959,10 +966,9 @@ func TestServer_ShouldReload_ReturnTrueForFileChanges(t *testing.T) {
 	}
 	agent.config.TLSConfig.SetChecksum()
 
-	shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := agent.ShouldReload(agentConfig)
+	shouldReloadAgent, shouldReloadHTTP := agent.ShouldReload(agentConfig)
 	require.False(shouldReloadAgent)
 	require.False(shouldReloadHTTP)
-	require.False(shouldReloadRPC)
 
 	newCertificate := `
 	-----BEGIN CERTIFICATE-----
@@ -999,10 +1005,9 @@ func TestServer_ShouldReload_ReturnTrueForFileChanges(t *testing.T) {
 		},
 	}
 
-	shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC = agent.ShouldReload(newAgentConfig)
+	shouldReloadAgent, shouldReloadHTTP = agent.ShouldReload(newAgentConfig)
 	require.True(shouldReloadAgent)
 	require.True(shouldReloadHTTP)
-	require.True(shouldReloadRPC)
 }
 
 func TestServer_ShouldReload_ShouldHandleMultipleChanges(t *testing.T) {
@@ -1043,20 +1048,18 @@ func TestServer_ShouldReload_ShouldHandleMultipleChanges(t *testing.T) {
 	defer agent.Shutdown()
 
 	{
-		shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := agent.ShouldReload(sameAgentConfig)
+		shouldReloadAgent, shouldReloadHTTP := agent.ShouldReload(sameAgentConfig)
 		require.True(shouldReloadAgent)
 		require.True(shouldReloadHTTP)
-		require.True(shouldReloadRPC)
 	}
 
 	err := agent.Reload(sameAgentConfig)
 	require.Nil(err)
 
 	{
-		shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := agent.ShouldReload(sameAgentConfig)
+		shouldReloadAgent, shouldReloadHTTP := agent.ShouldReload(sameAgentConfig)
 		require.False(shouldReloadAgent)
 		require.False(shouldReloadHTTP)
-		require.False(shouldReloadRPC)
 	}
 }
 
