@@ -22,10 +22,14 @@ import (
 	"github.com/hashicorp/consul/lib"
 	checkpoint "github.com/hashicorp/go-checkpoint"
 	discover "github.com/hashicorp/go-discover"
+	hclog "github.com/hashicorp/go-hclog"
 	gsyslog "github.com/hashicorp/go-syslog"
 	"github.com/hashicorp/logutils"
+	"github.com/hashicorp/nomad/helper"
 	flaghelper "github.com/hashicorp/nomad/helper/flag-helpers"
 	gatedwriter "github.com/hashicorp/nomad/helper/gated-writer"
+	"github.com/hashicorp/nomad/helper/logging"
+	"github.com/hashicorp/nomad/helper/winsvc"
 	"github.com/hashicorp/nomad/nomad/structs/config"
 	"github.com/hashicorp/nomad/version"
 	"github.com/mitchellh/cli"
@@ -53,7 +57,7 @@ type Command struct {
 }
 
 func (c *Command) readConfig() *Config {
-	var dev bool
+	var dev *devModeConfig
 	var configPath []string
 	var servers string
 	var meta []string
@@ -74,7 +78,10 @@ func (c *Command) readConfig() *Config {
 	flags.Usage = func() { c.Ui.Error(c.Help()) }
 
 	// Role options
-	flags.BoolVar(&dev, "dev", false, "")
+	var devMode bool
+	var devConnectMode bool
+	flags.BoolVar(&devMode, "dev", false, "")
+	flags.BoolVar(&devConnectMode, "dev-connect", false, "")
 	flags.BoolVar(&cmdConfig.Server.Enabled, "server", false, "")
 	flags.BoolVar(&cmdConfig.Client.Enabled, "client", false, "")
 
@@ -105,8 +112,10 @@ func (c *Command) readConfig() *Config {
 	flags.StringVar(&cmdConfig.BindAddr, "bind", "", "")
 	flags.StringVar(&cmdConfig.Region, "region", "", "")
 	flags.StringVar(&cmdConfig.DataDir, "data-dir", "", "")
+	flags.StringVar(&cmdConfig.PluginDir, "plugin-dir", "", "")
 	flags.StringVar(&cmdConfig.Datacenter, "dc", "", "")
 	flags.StringVar(&cmdConfig.LogLevel, "log-level", "", "")
+	flags.BoolVar(&cmdConfig.LogJson, "log-json", false, "")
 	flags.StringVar(&cmdConfig.NodeName, "node", "", "")
 
 	// Consul options
@@ -117,6 +126,7 @@ func (c *Command) readConfig() *Config {
 	}), "consul-auto-advertise", "")
 	flags.StringVar(&cmdConfig.Consul.CAFile, "consul-ca-file", "", "")
 	flags.StringVar(&cmdConfig.Consul.CertFile, "consul-cert-file", "", "")
+	flags.StringVar(&cmdConfig.Consul.KeyFile, "consul-key-file", "", "")
 	flags.Var((flaghelper.FuncBoolVar)(func(b bool) error {
 		cmdConfig.Consul.ChecksUseAdvertise = &b
 		return nil
@@ -127,7 +137,6 @@ func (c *Command) readConfig() *Config {
 	}), "consul-client-auto-join", "")
 	flags.StringVar(&cmdConfig.Consul.ClientServiceName, "consul-client-service-name", "", "")
 	flags.StringVar(&cmdConfig.Consul.ClientHTTPCheckName, "consul-client-http-check-name", "", "")
-	flags.StringVar(&cmdConfig.Consul.KeyFile, "consul-key-file", "", "")
 	flags.StringVar(&cmdConfig.Consul.ServerServiceName, "consul-server-service-name", "", "")
 	flags.StringVar(&cmdConfig.Consul.ServerHTTPCheckName, "consul-server-http-check-name", "", "")
 	flags.StringVar(&cmdConfig.Consul.ServerSerfCheckName, "consul-server-serf-check-name", "", "")
@@ -158,6 +167,7 @@ func (c *Command) readConfig() *Config {
 	}), "vault-allow-unauthenticated", "")
 	flags.StringVar(&cmdConfig.Vault.Token, "vault-token", "", "")
 	flags.StringVar(&cmdConfig.Vault.Addr, "vault-address", "", "")
+	flags.StringVar(&cmdConfig.Vault.Namespace, "vault-namespace", "", "")
 	flags.StringVar(&cmdConfig.Vault.Role, "vault-create-from-role", "", "")
 	flags.StringVar(&cmdConfig.Vault.TLSCaFile, "vault-ca-file", "", "")
 	flags.StringVar(&cmdConfig.Vault.TLSCaPath, "vault-ca-path", "", "")
@@ -192,15 +202,19 @@ func (c *Command) readConfig() *Config {
 				c.Ui.Error(fmt.Sprintf("Error parsing Client.Meta value: %v", kv))
 				return nil
 			}
-
 			cmdConfig.Client.Meta[parts[0]] = parts[1]
 		}
 	}
 
 	// Load the configuration
+	dev, err := newDevModeConfig(devMode, devConnectMode)
+	if err != nil {
+		c.Ui.Error(err.Error())
+		return nil
+	}
 	var config *Config
-	if dev {
-		config = DevConfig()
+	if dev != nil {
+		config = DevConfig(dev)
 	} else {
 		config = DefaultConfig()
 	}
@@ -251,72 +265,33 @@ func (c *Command) readConfig() *Config {
 
 	// Check to see if we should read the Vault token from the environment
 	if config.Vault.Token == "" {
-		if token, ok := os.LookupEnv("VAULT_TOKEN"); ok {
-			config.Vault.Token = token
-		}
+		config.Vault.Token = os.Getenv("VAULT_TOKEN")
 	}
 
-	if dev {
-		// Skip validation for dev mode
-		return config
+	// Check to see if we should read the Vault namespace from the environment
+	if config.Vault.Namespace == "" {
+		config.Vault.Namespace = os.Getenv("VAULT_NAMESPACE")
 	}
 
-	if config.Server.EncryptKey != "" {
-		if _, err := config.Server.EncryptBytes(); err != nil {
-			c.Ui.Error(fmt.Sprintf("Invalid encryption key: %s", err))
-			return nil
-		}
-		keyfile := filepath.Join(config.DataDir, serfKeyring)
-		if _, err := os.Stat(keyfile); err == nil {
-			c.Ui.Warn("WARNING: keyring exists but -encrypt given, using keyring")
-		}
+	// Default the plugin directory to be under that of the data directory if it
+	// isn't explicitly specified.
+	if config.PluginDir == "" && config.DataDir != "" {
+		config.PluginDir = filepath.Join(config.DataDir, "plugins")
 	}
 
+	if !c.isValidConfig(config, cmdConfig) {
+		return nil
+	}
+
+	return config
+}
+
+func (c *Command) isValidConfig(config, cmdConfig *Config) bool {
+
 	// Check that the server is running in at least one mode.
 	if !(config.Server.Enabled || config.Client.Enabled) {
 		c.Ui.Error("Must specify either server, client or dev mode for the agent.")
-		return nil
-	}
-
-	// Verify the paths are absolute.
-	dirs := map[string]string{
-		"data-dir":  config.DataDir,
-		"alloc-dir": config.Client.AllocDir,
-		"state-dir": config.Client.StateDir,
-	}
-	for k, dir := range dirs {
-		if dir == "" {
-			continue
-		}
-
-		if !filepath.IsAbs(dir) {
-			c.Ui.Error(fmt.Sprintf("%s must be given as an absolute path: got %v", k, dir))
-			return nil
-		}
-	}
-
-	// Ensure that we have the directories we neet to run.
-	if config.Server.Enabled && config.DataDir == "" {
-		c.Ui.Error("Must specify data directory")
-		return nil
-	}
-
-	// The config is valid if the top-level data-dir is set or if both
-	// alloc-dir and state-dir are set.
-	if config.Client.Enabled && config.DataDir == "" {
-		if config.Client.AllocDir == "" || config.Client.StateDir == "" {
-			c.Ui.Error("Must specify both the state and alloc dir if data-dir is omitted.")
-			return nil
-		}
-	}
-
-	// Check the bootstrap flags
-	if config.Server.BootstrapExpect > 0 && !config.Server.Enabled {
-		c.Ui.Error("Bootstrap requires server mode to be enabled")
-		return nil
-	}
-	if config.Server.BootstrapExpect == 1 {
-		c.Ui.Error("WARNING: Bootstrap mode enabled! Potentially unsafe operation.")
+		return false
 	}
 
 	// Set up the TLS configuration properly if we have one.
@@ -328,11 +303,79 @@ func (c *Command) readConfig() *Config {
 		}
 	}
 
-	return config
+	if config.Server.EncryptKey != "" {
+		if _, err := config.Server.EncryptBytes(); err != nil {
+			c.Ui.Error(fmt.Sprintf("Invalid encryption key: %s", err))
+			return false
+		}
+		keyfile := filepath.Join(config.DataDir, serfKeyring)
+		if _, err := os.Stat(keyfile); err == nil {
+			c.Ui.Warn("WARNING: keyring exists but -encrypt given, using keyring")
+		}
+	}
+
+	// Verify the paths are absolute.
+	dirs := map[string]string{
+		"data-dir":   config.DataDir,
+		"plugin-dir": config.PluginDir,
+		"alloc-dir":  config.Client.AllocDir,
+		"state-dir":  config.Client.StateDir,
+	}
+	for k, dir := range dirs {
+		if dir == "" {
+			continue
+		}
+
+		if !filepath.IsAbs(dir) {
+			c.Ui.Error(fmt.Sprintf("%s must be given as an absolute path: got %v", k, dir))
+			return false
+		}
+	}
+
+	if config.Client.Enabled {
+		for k := range config.Client.Meta {
+			if !helper.IsValidInterpVariable(k) {
+				c.Ui.Error(fmt.Sprintf("Invalid Client.Meta key: %v", k))
+				return false
+			}
+		}
+	}
+
+	if config.DevMode {
+		// Skip the rest of the validation for dev mode
+		return true
+	}
+
+	// Ensure that we have the directories we need to run.
+	if config.Server.Enabled && config.DataDir == "" {
+		c.Ui.Error("Must specify data directory")
+		return false
+	}
+
+	// The config is valid if the top-level data-dir is set or if both
+	// alloc-dir and state-dir are set.
+	if config.Client.Enabled && config.DataDir == "" {
+		if config.Client.AllocDir == "" || config.Client.StateDir == "" || config.PluginDir == "" {
+			c.Ui.Error("Must specify the state, alloc dir, and plugin dir if data-dir is omitted.")
+			return false
+		}
+	}
+
+	// Check the bootstrap flags
+	if !config.Server.Enabled && cmdConfig.Server.BootstrapExpect > 0 {
+		// report an error if BootstrapExpect is set in CLI but server is disabled
+		c.Ui.Error("Bootstrap requires server mode to be enabled")
+		return false
+	}
+	if config.Server.Enabled && config.Server.BootstrapExpect == 1 {
+		c.Ui.Error("WARNING: Bootstrap mode enabled! Potentially unsafe operation.")
+	}
+
+	return true
 }
 
-// setupLoggers is used to setup the logGate, logWriter, and our logOutput
-func (c *Command) setupLoggers(config *Config) (*gatedwriter.Writer, *logWriter, io.Writer) {
+// setupLoggers is used to setup the logGate, and our logOutput
+func (c *Command) setupLoggers(config *Config) (*gatedwriter.Writer, io.Writer) {
 	// Setup logging. First create the gated log writer, which will
 	// store logs until we're ready to show them. Then create the level
 	// filter, filtering logs of the specified level.
@@ -347,37 +390,66 @@ func (c *Command) setupLoggers(config *Config) (*gatedwriter.Writer, *logWriter,
 		c.Ui.Error(fmt.Sprintf(
 			"Invalid log level: %s. Valid log levels are: %v",
 			c.logFilter.MinLevel, c.logFilter.Levels))
-		return nil, nil, nil
+		return nil, nil
 	}
 
+	// Create a log writer, and wrap a logOutput around it
+	writers := []io.Writer{c.logFilter}
+
 	// Check if syslog is enabled
-	var syslog io.Writer
 	if config.EnableSyslog {
 		l, err := gsyslog.NewLogger(gsyslog.LOG_NOTICE, config.SyslogFacility, "nomad")
 		if err != nil {
 			c.Ui.Error(fmt.Sprintf("Syslog setup failed: %v", err))
-			return nil, nil, nil
+			return nil, nil
 		}
-		syslog = &SyslogWrapper{l, c.logFilter}
+		writers = append(writers, &SyslogWrapper{l, c.logFilter})
 	}
 
-	// Create a log writer, and wrap a logOutput around it
-	logWriter := NewLogWriter(512)
-	var logOutput io.Writer
-	if syslog != nil {
-		logOutput = io.MultiWriter(c.logFilter, logWriter, syslog)
-	} else {
-		logOutput = io.MultiWriter(c.logFilter, logWriter)
+	// Check if file logging is enabled
+	if config.LogFile != "" {
+		dir, fileName := filepath.Split(config.LogFile)
+
+		// if a path is provided, but has no filename, then a default is used.
+		if fileName == "" {
+			fileName = "nomad.log"
+		}
+
+		// Try to enter the user specified log rotation duration first
+		var logRotateDuration time.Duration
+		if config.LogRotateDuration != "" {
+			duration, err := time.ParseDuration(config.LogRotateDuration)
+			if err != nil {
+				c.Ui.Error(fmt.Sprintf("Failed to parse log rotation duration: %v", err))
+				return nil, nil
+			}
+			logRotateDuration = duration
+		} else {
+			// Default to 24 hrs if no rotation period is specified
+			logRotateDuration = 24 * time.Hour
+		}
+
+		logFile := &logFile{
+			logFilter: c.logFilter,
+			fileName:  fileName,
+			logPath:   dir,
+			duration:  logRotateDuration,
+			MaxBytes:  config.LogRotateBytes,
+			MaxFiles:  config.LogRotateMaxFiles,
+		}
+
+		writers = append(writers, logFile)
 	}
-	c.logOutput = logOutput
-	log.SetOutput(logOutput)
-	return logGate, logWriter, logOutput
+
+	c.logOutput = io.MultiWriter(writers...)
+	log.SetOutput(c.logOutput)
+	return logGate, c.logOutput
 }
 
 // setupAgent is used to start the agent and various interfaces
-func (c *Command) setupAgent(config *Config, logOutput io.Writer, inmem *metrics.InmemSink) error {
+func (c *Command) setupAgent(config *Config, logger hclog.InterceptLogger, logOutput io.Writer, inmem *metrics.InmemSink) error {
 	c.Ui.Output("Starting Nomad agent...")
-	agent, err := NewAgent(config, logOutput, inmem)
+	agent, err := NewAgent(config, logger, logOutput, inmem)
 	if err != nil {
 		c.Ui.Error(fmt.Sprintf("Error starting agent: %s", err))
 		return err
@@ -446,7 +518,61 @@ func (c *Command) AutocompleteFlags() complete.Flags {
 		complete.PredictFiles("*.hcl"))
 
 	return map[string]complete.Predictor{
-		"-config": configFilePredictor,
+		"-dev":                           complete.PredictNothing,
+		"-dev-connect":                   complete.PredictNothing,
+		"-server":                        complete.PredictNothing,
+		"-client":                        complete.PredictNothing,
+		"-bootstrap-expect":              complete.PredictAnything,
+		"-encrypt":                       complete.PredictAnything,
+		"-raft-protocol":                 complete.PredictAnything,
+		"-rejoin":                        complete.PredictNothing,
+		"-join":                          complete.PredictAnything,
+		"-retry-join":                    complete.PredictAnything,
+		"-retry-max":                     complete.PredictAnything,
+		"-state-dir":                     complete.PredictDirs("*"),
+		"-alloc-dir":                     complete.PredictDirs("*"),
+		"-node-class":                    complete.PredictAnything,
+		"-servers":                       complete.PredictAnything,
+		"-meta":                          complete.PredictAnything,
+		"-config":                        configFilePredictor,
+		"-bind":                          complete.PredictAnything,
+		"-region":                        complete.PredictAnything,
+		"-data-dir":                      complete.PredictDirs("*"),
+		"-plugin-dir":                    complete.PredictDirs("*"),
+		"-dc":                            complete.PredictAnything,
+		"-log-level":                     complete.PredictAnything,
+		"-json-logs":                     complete.PredictNothing,
+		"-node":                          complete.PredictAnything,
+		"-consul-auth":                   complete.PredictAnything,
+		"-consul-auto-advertise":         complete.PredictNothing,
+		"-consul-ca-file":                complete.PredictAnything,
+		"-consul-cert-file":              complete.PredictAnything,
+		"-consul-key-file":               complete.PredictAnything,
+		"-consul-checks-use-advertise":   complete.PredictNothing,
+		"-consul-client-auto-join":       complete.PredictNothing,
+		"-consul-client-service-name":    complete.PredictAnything,
+		"-consul-client-http-check-name": complete.PredictAnything,
+		"-consul-server-service-name":    complete.PredictAnything,
+		"-consul-server-http-check-name": complete.PredictAnything,
+		"-consul-server-serf-check-name": complete.PredictAnything,
+		"-consul-server-rpc-check-name":  complete.PredictAnything,
+		"-consul-server-auto-join":       complete.PredictNothing,
+		"-consul-ssl":                    complete.PredictNothing,
+		"-consul-verify-ssl":             complete.PredictNothing,
+		"-consul-address":                complete.PredictAnything,
+		"-vault-enabled":                 complete.PredictNothing,
+		"-vault-allow-unauthenticated":   complete.PredictNothing,
+		"-vault-token":                   complete.PredictAnything,
+		"-vault-address":                 complete.PredictAnything,
+		"-vault-create-from-role":        complete.PredictAnything,
+		"-vault-ca-file":                 complete.PredictAnything,
+		"-vault-ca-path":                 complete.PredictAnything,
+		"-vault-cert-file":               complete.PredictAnything,
+		"-vault-key-file":                complete.PredictAnything,
+		"-vault-tls-skip-verify":         complete.PredictNothing,
+		"-vault-tls-server-name":         complete.PredictAnything,
+		"-acl-enabled":                   complete.PredictNothing,
+		"-acl-replication-token":         complete.PredictAnything,
 	}
 }
 
@@ -470,11 +596,24 @@ func (c *Command) Run(args []string) int {
 	}
 
 	// Setup the log outputs
-	logGate, _, logOutput := c.setupLoggers(config)
+	logGate, logOutput := c.setupLoggers(config)
 	if logGate == nil {
 		return 1
 	}
 
+	// Create logger
+	logger := hclog.NewInterceptLogger(&hclog.LoggerOptions{
+		Name:       "agent",
+		Level:      hclog.LevelFromString(config.LogLevel),
+		Output:     logOutput,
+		JSONFormat: config.LogJson,
+	})
+
+	// Swap out UI implementation if json logging is enabled
+	if config.LogJson {
+		c.Ui = &logging.HcLogUI{Log: logger}
+	}
+
 	// Log config files
 	if len(config.Files) > 0 {
 		c.Ui.Output(fmt.Sprintf("Loaded configuration from %s", strings.Join(config.Files, ", ")))
@@ -490,7 +629,7 @@ func (c *Command) Run(args []string) int {
 	}
 
 	// Create the agent
-	if err := c.setupAgent(config, logOutput, inmem); err != nil {
+	if err := c.setupAgent(config, logger, logOutput, inmem); err != nil {
 		logGate.Flush()
 		return 1
 	}
@@ -562,7 +701,7 @@ func (c *Command) handleRetryJoin(config *Config) error {
 		joiner := retryJoiner{
 			discover:      &discover.Discover{},
 			errCh:         c.retryJoinErrCh,
-			logger:        c.agent.logger,
+			logger:        c.agent.logger.Named("joiner"),
 			serverJoin:    c.agent.server.Join,
 			serverEnabled: true,
 		}
@@ -585,7 +724,7 @@ func (c *Command) handleRetryJoin(config *Config) error {
 			config.Server.RetryInterval = 0
 		}
 
-		c.agent.logger.Printf("[WARN] agent: Using deprecated retry_join fields. Upgrade configuration to use server_join")
+		c.agent.logger.Warn("using deprecated retry_join fields. Upgrade configuration to use server_join")
 	}
 
 	if config.Server.Enabled &&
@@ -595,7 +734,7 @@ func (c *Command) handleRetryJoin(config *Config) error {
 		joiner := retryJoiner{
 			discover:      &discover.Discover{},
 			errCh:         c.retryJoinErrCh,
-			logger:        c.agent.logger,
+			logger:        c.agent.logger.Named("joiner"),
 			serverJoin:    c.agent.server.Join,
 			serverEnabled: true,
 		}
@@ -613,7 +752,7 @@ func (c *Command) handleRetryJoin(config *Config) error {
 		joiner := retryJoiner{
 			discover:      &discover.Discover{},
 			errCh:         c.retryJoinErrCh,
-			logger:        c.agent.logger,
+			logger:        c.agent.logger.Named("joiner"),
 			clientJoin:    c.agent.client.SetServers,
 			clientEnabled: true,
 		}
@@ -639,6 +778,8 @@ WAIT:
 	select {
 	case s := <-signalCh:
 		sig = s
+	case <-winsvc.ShutdownChannel():
+		sig = os.Interrupt
 	case <-c.ShutdownCh:
 		sig = os.Interrupt
 	case <-c.retryJoinErrCh:
@@ -696,7 +837,7 @@ WAIT:
 // reloadHTTPServer shuts down the existing HTTP server and restarts it. This
 // is helpful when reloading the agent configuration.
 func (c *Command) reloadHTTPServer() error {
-	c.agent.logger.Println("[INFO] agent: Reloading HTTP server with new TLS configuration")
+	c.agent.logger.Info("reloading HTTP server with new TLS configuration")
 
 	c.httpServer.Shutdown()
 
@@ -731,43 +872,51 @@ func (c *Command) handleReload() {
 		newConf.LogLevel = c.agent.GetConfig().LogLevel
 	}
 
-	shouldReloadAgent, shouldReloadHTTP, shouldReloadRPC := c.agent.ShouldReload(newConf)
+	shouldReloadAgent, shouldReloadHTTP := c.agent.ShouldReload(newConf)
 	if shouldReloadAgent {
-		c.agent.logger.Printf("[DEBUG] agent: starting reload of agent config")
+		c.agent.logger.Debug("starting reload of agent config")
 		err := c.agent.Reload(newConf)
 		if err != nil {
-			c.agent.logger.Printf("[ERR] agent: failed to reload the config: %v", err)
+			c.agent.logger.Error("failed to reload the config", "error", err)
 			return
 		}
 	}
 
 	if s := c.agent.Server(); s != nil {
-		c.agent.logger.Printf("[DEBUG] agent: starting reload of server config")
-		sconf, err := convertServerConfig(newConf, c.logOutput)
+		c.agent.logger.Debug("starting reload of server config")
+		sconf, err := convertServerConfig(newConf)
 		if err != nil {
-			c.agent.logger.Printf("[ERR] agent: failed to convert server config: %v", err)
+			c.agent.logger.Error("failed to convert server config", "error", err)
+			return
+		}
+
+		// Finalize the config to get the agent objects injected in
+		c.agent.finalizeServerConfig(sconf)
+
+		// Reload the config
+		if err := s.Reload(sconf); err != nil {
+			c.agent.logger.Error("reloading server config failed", "error", err)
 			return
-		} else {
-			if err := s.Reload(sconf); err != nil {
-				c.agent.logger.Printf("[ERR] agent: reloading server config failed: %v", err)
-				return
-			}
 		}
 	}
 
-	if shouldReloadRPC {
+	if s := c.agent.Client(); s != nil {
+		c.agent.logger.Debug("starting reload of client config")
+		clientConfig, err := convertClientConfig(newConf)
+		if err != nil {
+			c.agent.logger.Error("failed to convert client config", "error", err)
+			return
+		}
 
-		if s := c.agent.Client(); s != nil {
-			clientConfig, err := c.agent.clientConfig()
-			c.agent.logger.Printf("[DEBUG] agent: starting reload of client config")
-			if err != nil {
-				c.agent.logger.Printf("[ERR] agent: reloading client config failed: %v", err)
-				return
-			}
-			if err := c.agent.Client().Reload(clientConfig); err != nil {
-				c.agent.logger.Printf("[ERR] agent: reloading client config failed: %v", err)
-				return
-			}
+		// Finalize the config to get the agent objects injected in
+		if err := c.agent.finalizeClientConfig(clientConfig); err != nil {
+			c.agent.logger.Error("failed to finalize client config", "error", err)
+			return
+		}
+
+		if err := c.agent.Client().Reload(clientConfig); err != nil {
+			c.agent.logger.Error("reloading client config failed", "error", err)
+			return
 		}
 	}
 
@@ -778,7 +927,7 @@ func (c *Command) handleReload() {
 	if shouldReloadHTTP {
 		err := c.reloadHTTPServer()
 		if err != nil {
-			c.agent.logger.Printf("[ERR] http: failed to reload the config: %v", err)
+			c.agent.httpLogger.Error("reloading config failed", "error", err)
 			return
 		}
 	}
@@ -1028,6 +1177,10 @@ General Options (clients and servers):
     downloaded artifacts used by drivers. On server nodes, the data
     dir is also used to store the replicated log.
 
+  -plugin-dir=<path>
+    The plugin directory is used to discover Nomad plugins. If not specified,
+    the plugin directory defaults to be that of <data-dir>/plugins/.
+
   -dc=<datacenter>
     The name of the datacenter this Nomad agent is a member of. By
     default this is set to "dc1".
@@ -1037,6 +1190,9 @@ General Options (clients and servers):
     DEBUG, INFO, and WARN, in decreasing order of verbosity. The
     default is INFO.
 
+  -log-json
+    Output logs in a JSON format. The default is false.
+
   -node=<name>
     The name of the local agent. This name is used to identify the node
     in the cluster. The name must be unique per region. The default is
@@ -1050,7 +1206,13 @@ General Options (clients and servers):
     Start the agent in development mode. This enables a pre-configured
     dual-role agent (client + server) which is useful for developing
     or testing Nomad. No other configuration is required to start the
-    agent in this mode.
+    agent in this mode, but you may pass an optional comma-separated
+    list of mode configurations:
+
+  -dev-connect
+	Start the agent in development mode, but bind to a public network
+	interface rather than localhost for using Consul Connect. This
+	mode is supported only on Linux as root.
 
 Server Options:
 
