@@ -7,10 +7,13 @@ import (
 	"os"
 	"path/filepath"
 	"reflect"
+	"runtime"
+	"strings"
 	"testing"
 	"time"
 
 	"github.com/hashicorp/consul/lib/freeport"
+	"github.com/hashicorp/nomad/client/testutil"
 	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/nomad/structs/config"
@@ -45,7 +48,9 @@ func TestConfig_Merge(t *testing.T) {
 		Datacenter:                "dc1",
 		NodeName:                  "node1",
 		DataDir:                   "/tmp/dir1",
+		PluginDir:                 "/tmp/pluginDir1",
 		LogLevel:                  "INFO",
+		LogJson:                   false,
 		EnableDebug:               false,
 		LeaveOnInt:                false,
 		LeaveOnTerm:               false,
@@ -86,18 +91,21 @@ func TestConfig_Merge(t *testing.T) {
 			Options: map[string]string{
 				"foo": "bar",
 			},
-			NetworkSpeed:   100,
-			CpuCompute:     100,
-			MemoryMB:       100,
-			MaxKillTimeout: "20s",
-			ClientMaxPort:  19996,
+			NetworkSpeed:      100,
+			CpuCompute:        100,
+			MemoryMB:          100,
+			MaxKillTimeout:    "20s",
+			ClientMaxPort:     19996,
+			DisableRemoteExec: false,
+			TemplateConfig: &ClientTemplateConfig{
+				FunctionBlacklist: []string{"plugin"},
+				DisableSandbox:    false,
+			},
 			Reserved: &Resources{
-				CPU:                 10,
-				MemoryMB:            10,
-				DiskMB:              10,
-				IOPS:                10,
-				ReservedPorts:       "1,10-30,55",
-				ParsedReservedPorts: []int{1, 2, 4},
+				CPU:           10,
+				MemoryMB:      10,
+				DiskMB:        10,
+				ReservedPorts: "1,10-30,55",
 			},
 		},
 		Server: &ServerConfig{
@@ -176,14 +184,25 @@ func TestConfig_Merge(t *testing.T) {
 			DisableUpgradeMigration: &falseValue,
 			EnableCustomUpgrades:    &falseValue,
 		},
+		Plugins: []*config.PluginConfig{
+			{
+				Name: "docker",
+				Args: []string{"foo"},
+				Config: map[string]interface{}{
+					"bar": 1,
+				},
+			},
+		},
 	}
 
 	c3 := &Config{
-		Region:                    "region2",
+		Region:                    "global",
 		Datacenter:                "dc2",
 		NodeName:                  "node2",
 		DataDir:                   "/tmp/dir2",
+		PluginDir:                 "/tmp/pluginDir2",
 		LogLevel:                  "DEBUG",
+		LogJson:                   true,
 		EnableDebug:               true,
 		LeaveOnInt:                true,
 		LeaveOnTerm:               true,
@@ -233,20 +252,23 @@ func TestConfig_Merge(t *testing.T) {
 				"foo": "bar",
 				"baz": "zip",
 			},
-			ChrootEnv:      map[string]string{},
-			ClientMaxPort:  20000,
-			ClientMinPort:  22000,
-			NetworkSpeed:   105,
-			CpuCompute:     105,
-			MemoryMB:       105,
-			MaxKillTimeout: "50s",
+			ChrootEnv:         map[string]string{},
+			ClientMaxPort:     20000,
+			ClientMinPort:     22000,
+			NetworkSpeed:      105,
+			CpuCompute:        105,
+			MemoryMB:          105,
+			MaxKillTimeout:    "50s",
+			DisableRemoteExec: false,
+			TemplateConfig: &ClientTemplateConfig{
+				FunctionBlacklist: []string{"plugin"},
+				DisableSandbox:    false,
+			},
 			Reserved: &Resources{
-				CPU:                 15,
-				MemoryMB:            15,
-				DiskMB:              15,
-				IOPS:                15,
-				ReservedPorts:       "2,10-30,55",
-				ParsedReservedPorts: []int{1, 2, 3},
+				CPU:           15,
+				MemoryMB:      15,
+				DiskMB:        15,
+				ReservedPorts: "2,10-30,55",
 			},
 			GCInterval:            6 * time.Second,
 			GCParallelDestroys:    6,
@@ -345,6 +367,22 @@ func TestConfig_Merge(t *testing.T) {
 			DisableUpgradeMigration: &trueValue,
 			EnableCustomUpgrades:    &trueValue,
 		},
+		Plugins: []*config.PluginConfig{
+			{
+				Name: "docker",
+				Args: []string{"bam"},
+				Config: map[string]interface{}{
+					"baz": 2,
+				},
+			},
+			{
+				Name: "exec",
+				Args: []string{"arg"},
+				Config: map[string]interface{}{
+					"config": true,
+				},
+			},
+		},
 	}
 
 	result := c0.Merge(c1)
@@ -585,6 +623,62 @@ func TestConfig_Listener(t *testing.T) {
 	}
 }
 
+func TestConfig_DevModeFlag(t *testing.T) {
+	cases := []struct {
+		dev         bool
+		connect     bool
+		expected    *devModeConfig
+		expectedErr string
+	}{}
+	if runtime.GOOS != "linux" {
+		cases = []struct {
+			dev         bool
+			connect     bool
+			expected    *devModeConfig
+			expectedErr string
+		}{
+			{false, false, nil, ""},
+			{true, false, &devModeConfig{defaultMode: true, connectMode: false}, ""},
+			{true, true, nil, "-dev-connect is only supported on linux"},
+			{false, true, nil, "-dev-connect is only supported on linux"},
+		}
+	}
+	if runtime.GOOS == "linux" {
+		testutil.RequireRoot(t)
+		cases = []struct {
+			dev         bool
+			connect     bool
+			expected    *devModeConfig
+			expectedErr string
+		}{
+			{false, false, nil, ""},
+			{true, false, &devModeConfig{defaultMode: true, connectMode: false}, ""},
+			{true, true, &devModeConfig{defaultMode: true, connectMode: true}, ""},
+			{false, true, &devModeConfig{defaultMode: false, connectMode: true}, ""},
+		}
+	}
+	for _, c := range cases {
+		t.Run("", func(t *testing.T) {
+			mode, err := newDevModeConfig(c.dev, c.connect)
+			if err != nil && c.expectedErr == "" {
+				t.Fatalf("unexpected error: %v", err)
+			}
+			if err != nil && !strings.Contains(err.Error(), c.expectedErr) {
+				t.Fatalf("expected %s; got %v", c.expectedErr, err)
+			}
+			if mode == nil && c.expected != nil {
+				t.Fatalf("expected %+v but got nil", c.expected)
+			}
+			if mode != nil {
+				if c.expected.defaultMode != mode.defaultMode ||
+					c.expected.connectMode != mode.connectMode {
+					t.Fatalf("expected %+v, got %+v", c.expected, mode)
+				}
+			}
+		})
+	}
+}
+
 // TestConfig_normalizeAddrs_DevMode asserts that normalizeAddrs allows
 // advertising localhost in dev mode.
 func TestConfig_normalizeAddrs_DevMode(t *testing.T) {
@@ -853,54 +947,6 @@ func TestConfig_normalizeAddrs(t *testing.T) {
 	}
 }
 
-func TestResources_ParseReserved(t *testing.T) {
-	cases := []struct {
-		Input  string
-		Parsed []int
-		Err    bool
-	}{
-		{
-			"1,2,3",
-			[]int{1, 2, 3},
-			false,
-		},
-		{
-			"3,1,2,1,2,3,1-3",
-			[]int{1, 2, 3},
-			false,
-		},
-		{
-			"3-1",
-			nil,
-			true,
-		},
-		{
-			"1-3,2-4",
-			[]int{1, 2, 3, 4},
-			false,
-		},
-		{
-			"1-3,4,5-5,6,7,8-10",
-			[]int{1, 2, 3, 4, 5, 6, 7, 8, 9, 10},
-			false,
-		},
-	}
-
-	for i, tc := range cases {
-		r := &Resources{ReservedPorts: tc.Input}
-		err := r.ParseReserved()
-		if (err != nil) != tc.Err {
-			t.Fatalf("test case %d: %v", i, err)
-			continue
-		}
-
-		if !reflect.DeepEqual(r.ParsedReservedPorts, tc.Parsed) {
-			t.Fatalf("test case %d: \n\n%#v\n\n%#v", i, r.ParsedReservedPorts, tc.Parsed)
-		}
-
-	}
-}
-
 func TestIsMissingPort(t *testing.T) {
 	_, _, err := net.SplitHostPort("localhost")
 	if missing := isMissingPort(err); !missing {
