@@ -3,7 +3,6 @@ package agent
 import (
 	"encoding/base64"
 	"fmt"
-	"io"
 	"io/ioutil"
 	"net/http"
 	"net/http/httptest"
@@ -12,6 +11,7 @@ import (
 	"time"
 
 	cstructs "github.com/hashicorp/nomad/client/structs"
+	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad/mock"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/testutil"
@@ -145,25 +145,22 @@ func TestHTTP_FS_ReadAt_MissingParams(t *testing.T) {
 	require := require.New(t)
 	httpTest(t, nil, func(s *TestAgent) {
 		req, err := http.NewRequest("GET", "/v1/client/fs/readat/", nil)
-		require.Nil(err)
-		respW := httptest.NewRecorder()
+		require.NoError(err)
 
-		_, err = s.Server.FileReadAtRequest(respW, req)
-		require.NotNil(err)
+		_, err = s.Server.FileReadAtRequest(httptest.NewRecorder(), req)
+		require.Error(err)
 
 		req, err = http.NewRequest("GET", "/v1/client/fs/readat/foo", nil)
-		require.Nil(err)
-		respW = httptest.NewRecorder()
+		require.NoError(err)
 
-		_, err = s.Server.FileReadAtRequest(respW, req)
-		require.NotNil(err)
+		_, err = s.Server.FileReadAtRequest(httptest.NewRecorder(), req)
+		require.Error(err)
 
 		req, err = http.NewRequest("GET", "/v1/client/fs/readat/foo?path=/path/to/file", nil)
-		require.Nil(err)
-		respW = httptest.NewRecorder()
+		require.NoError(err)
 
-		_, err = s.Server.FileReadAtRequest(respW, req)
-		require.NotNil(err)
+		_, err = s.Server.FileReadAtRequest(httptest.NewRecorder(), req)
+		require.Error(err)
 	})
 }
 
@@ -192,59 +189,70 @@ func TestHTTP_FS_Stream_MissingParams(t *testing.T) {
 	require := require.New(t)
 	httpTest(t, nil, func(s *TestAgent) {
 		req, err := http.NewRequest("GET", "/v1/client/fs/stream/", nil)
-		require.Nil(err)
+		require.NoError(err)
 		respW := httptest.NewRecorder()
 
 		_, err = s.Server.Stream(respW, req)
 		require.EqualError(err, allocIDNotPresentErr.Error())
 
 		req, err = http.NewRequest("GET", "/v1/client/fs/stream/foo", nil)
-		require.Nil(err)
+		require.NoError(err)
 		respW = httptest.NewRecorder()
 
 		_, err = s.Server.Stream(respW, req)
 		require.EqualError(err, fileNameNotPresentErr.Error())
 
 		req, err = http.NewRequest("GET", "/v1/client/fs/stream/foo?path=/path/to/file", nil)
-		require.Nil(err)
+		require.NoError(err)
 		respW = httptest.NewRecorder()
 
 		_, err = s.Server.Stream(respW, req)
-		require.Nil(err)
+		require.Error(err)
+		require.Contains(err.Error(), "alloc lookup failed")
 	})
 }
 
+// TestHTTP_FS_Logs_MissingParams asserts proper error codes and messages are
+// returned for incorrect parameters (eg missing tasks).
 func TestHTTP_FS_Logs_MissingParams(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
 	httpTest(t, nil, func(s *TestAgent) {
+		// AllocID Not Present
 		req, err := http.NewRequest("GET", "/v1/client/fs/logs/", nil)
-		require.Nil(err)
+		require.NoError(err)
 		respW := httptest.NewRecorder()
 
-		_, err = s.Server.Logs(respW, req)
-		require.EqualError(err, allocIDNotPresentErr.Error())
+		s.Server.mux.ServeHTTP(respW, req)
+		require.Equal(respW.Body.String(), allocIDNotPresentErr.Error())
+		require.Equal(400, respW.Code)
 
+		// Task Not Present
 		req, err = http.NewRequest("GET", "/v1/client/fs/logs/foo", nil)
-		require.Nil(err)
+		require.NoError(err)
 		respW = httptest.NewRecorder()
 
-		_, err = s.Server.Logs(respW, req)
-		require.EqualError(err, taskNotPresentErr.Error())
+		s.Server.mux.ServeHTTP(respW, req)
+		require.Equal(respW.Body.String(), taskNotPresentErr.Error())
+		require.Equal(400, respW.Code)
 
+		// Log Type Not Present
 		req, err = http.NewRequest("GET", "/v1/client/fs/logs/foo?task=foo", nil)
-		require.Nil(err)
+		require.NoError(err)
 		respW = httptest.NewRecorder()
 
-		_, err = s.Server.Logs(respW, req)
-		require.EqualError(err, logTypeNotPresentErr.Error())
+		s.Server.mux.ServeHTTP(respW, req)
+		require.Equal(respW.Body.String(), logTypeNotPresentErr.Error())
+		require.Equal(400, respW.Code)
 
+		// case where all parameters are set but alloc isn't found
 		req, err = http.NewRequest("GET", "/v1/client/fs/logs/foo?task=foo&type=stdout", nil)
-		require.Nil(err)
+		require.NoError(err)
 		respW = httptest.NewRecorder()
 
-		_, err = s.Server.Logs(respW, req)
-		require.Nil(err)
+		s.Server.mux.ServeHTTP(respW, req)
+		require.Equal(500, respW.Code)
+		require.Contains(respW.Body.String(), "alloc lookup failed")
 	})
 }
 
@@ -335,7 +343,51 @@ func TestHTTP_FS_Cat(t *testing.T) {
 	})
 }
 
-func TestHTTP_FS_Stream(t *testing.T) {
+func TestHTTP_FS_Stream_NoFollow(t *testing.T) {
+	t.Parallel()
+	require := require.New(t)
+	httpTest(t, nil, func(s *TestAgent) {
+		a := mockFSAlloc(s.client.NodeID(), nil)
+		addAllocToClient(s, a, terminalClientAlloc)
+
+		offset := 4
+		expectation := base64.StdEncoding.EncodeToString(
+			[]byte(defaultLoggerMockDriverStdout[len(defaultLoggerMockDriverStdout)-offset:]))
+		path := fmt.Sprintf("/v1/client/fs/stream/%s?path=alloc/logs/web.stdout.0&offset=%d&origin=end&follow=false",
+			a.ID, offset)
+
+		req, err := http.NewRequest("GET", path, nil)
+		require.Nil(err)
+		respW := testutil.NewResponseRecorder()
+		doneCh := make(chan struct{})
+		go func() {
+			_, err = s.Server.Stream(respW, req)
+			require.Nil(err)
+			close(doneCh)
+		}()
+
+		out := ""
+		testutil.WaitForResult(func() (bool, error) {
+			output, err := ioutil.ReadAll(respW)
+			if err != nil {
+				return false, err
+			}
+
+			out += string(output)
+			return strings.Contains(out, expectation), fmt.Errorf("%q doesn't contain %q", out, expectation)
+		}, func(err error) {
+			t.Fatal(err)
+		})
+
+		select {
+		case <-doneCh:
+		case <-time.After(1 * time.Second):
+			t.Fatal("should close but did not")
+		}
+	})
+}
+
+func TestHTTP_FS_Stream_Follow(t *testing.T) {
 	t.Parallel()
 	require := require.New(t)
 	httpTest(t, nil, func(s *TestAgent) {
@@ -348,9 +400,7 @@ func TestHTTP_FS_Stream(t *testing.T) {
 		path := fmt.Sprintf("/v1/client/fs/stream/%s?path=alloc/logs/web.stdout.0&offset=%d&origin=end",
 			a.ID, offset)
 
-		p, _ := io.Pipe()
-
-		req, err := http.NewRequest("GET", path, p)
+		req, err := http.NewRequest("GET", path, nil)
 		require.Nil(err)
 		respW := httptest.NewRecorder()
 		doneCh := make(chan struct{})
@@ -378,8 +428,6 @@ func TestHTTP_FS_Stream(t *testing.T) {
 			t.Fatal("shouldn't close")
 		case <-time.After(1 * time.Second):
 		}
-
-		p.Close()
 	})
 }
 
@@ -395,10 +443,9 @@ func TestHTTP_FS_Logs(t *testing.T) {
 		path := fmt.Sprintf("/v1/client/fs/logs/%s?type=stdout&task=web&offset=%d&origin=end&plain=true",
 			a.ID, offset)
 
-		p, _ := io.Pipe()
-		req, err := http.NewRequest("GET", path, p)
+		req, err := http.NewRequest("GET", path, nil)
 		require.Nil(err)
-		respW := httptest.NewRecorder()
+		respW := testutil.NewResponseRecorder()
 		go func() {
 			_, err = s.Server.Logs(respW, req)
 			require.Nil(err)
@@ -406,7 +453,7 @@ func TestHTTP_FS_Logs(t *testing.T) {
 
 		out := ""
 		testutil.WaitForResult(func() (bool, error) {
-			output, err := ioutil.ReadAll(respW.Body)
+			output, err := ioutil.ReadAll(respW)
 			if err != nil {
 				return false, err
 			}
@@ -416,8 +463,6 @@ func TestHTTP_FS_Logs(t *testing.T) {
 		}, func(err error) {
 			t.Fatal(err)
 		})
-
-		p.Close()
 	})
 }
 
@@ -433,10 +478,9 @@ func TestHTTP_FS_Logs_Follow(t *testing.T) {
 		path := fmt.Sprintf("/v1/client/fs/logs/%s?type=stdout&task=web&offset=%d&origin=end&plain=true&follow=true",
 			a.ID, offset)
 
-		p, _ := io.Pipe()
-		req, err := http.NewRequest("GET", path, p)
+		req, err := http.NewRequest("GET", path, nil)
 		require.Nil(err)
-		respW := httptest.NewRecorder()
+		respW := testutil.NewResponseRecorder()
 		errCh := make(chan error)
 		go func() {
 			_, err := s.Server.Logs(respW, req)
@@ -445,7 +489,7 @@ func TestHTTP_FS_Logs_Follow(t *testing.T) {
 
 		out := ""
 		testutil.WaitForResult(func() (bool, error) {
-			output, err := ioutil.ReadAll(respW.Body)
+			output, err := ioutil.ReadAll(respW)
 			if err != nil {
 				return false, err
 			}
@@ -461,7 +505,23 @@ func TestHTTP_FS_Logs_Follow(t *testing.T) {
 			t.Fatalf("shouldn't exit: %v", err)
 		case <-time.After(1 * time.Second):
 		}
+	})
+}
 
-		p.Close()
+func TestHTTP_FS_Logs_PropagatesErrors(t *testing.T) {
+	t.Parallel()
+	httpTest(t, nil, func(s *TestAgent) {
+		path := fmt.Sprintf("/v1/client/fs/logs/%s?type=stdout&task=web&offset=0&origin=end&plain=true",
+			uuid.Generate())
+
+		req, err := http.NewRequest("GET", path, nil)
+		require.NoError(t, err)
+		respW := testutil.NewResponseRecorder()
+
+		_, err = s.Server.Logs(respW, req)
+		require.Error(t, err)
+
+		_, ok := err.(HTTPCodedError)
+		require.Truef(t, ok, "expected a coded error but found: %#+v", err)
 	})
 }
