@@ -15,6 +15,7 @@ import (
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/kr/pretty"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 func TestHTTP_JobsList(t *testing.T) {
@@ -130,7 +131,7 @@ func TestHTTP_JobsRegister(t *testing.T) {
 	t.Parallel()
 	httpTest(t, nil, func(s *TestAgent) {
 		// Create the job
-		job := api.MockJob()
+		job := MockJob()
 		args := api.JobRegisterRequest{
 			Job:          job,
 			WriteRequest: api.WriteRequest{Region: "global"},
@@ -185,7 +186,7 @@ func TestHTTP_JobsRegister_ACL(t *testing.T) {
 	t.Parallel()
 	httpACLTest(t, nil, func(s *TestAgent) {
 		// Create the job
-		job := api.MockJob()
+		job := MockJob()
 		args := api.JobRegisterRequest{
 			Job: job,
 			WriteRequest: api.WriteRequest{
@@ -215,7 +216,7 @@ func TestHTTP_JobsRegister_Defaulting(t *testing.T) {
 	t.Parallel()
 	httpTest(t, nil, func(s *TestAgent) {
 		// Create the job
-		job := api.MockJob()
+		job := MockJob()
 
 		// Do not set its priority
 		job.Priority = nil
@@ -411,7 +412,7 @@ func TestHTTP_JobUpdate(t *testing.T) {
 	t.Parallel()
 	httpTest(t, nil, func(s *TestAgent) {
 		// Create the job
-		job := api.MockJob()
+		job := MockJob()
 		args := api.JobRegisterRequest{
 			Job: job,
 			WriteRequest: api.WriteRequest{
@@ -464,6 +465,105 @@ func TestHTTP_JobUpdate(t *testing.T) {
 	})
 }
 
+func TestHTTP_JobUpdateRegion(t *testing.T) {
+	t.Parallel()
+
+	cases := []struct {
+		Name           string
+		ConfigRegion   string
+		APIRegion      string
+		ExpectedRegion string
+	}{
+		{
+			Name:           "api region takes precedence",
+			ConfigRegion:   "not-global",
+			APIRegion:      "north-america",
+			ExpectedRegion: "north-america",
+		},
+		{
+			Name:           "config region is set",
+			ConfigRegion:   "north-america",
+			APIRegion:      "",
+			ExpectedRegion: "north-america",
+		},
+		{
+			Name:           "api region is set",
+			ConfigRegion:   "",
+			APIRegion:      "north-america",
+			ExpectedRegion: "north-america",
+		},
+		{
+			Name:           "defaults to node region global if no region is provided",
+			ConfigRegion:   "",
+			APIRegion:      "",
+			ExpectedRegion: "global",
+		},
+		{
+			Name:           "defaults to node region not-global if no region is provided",
+			ConfigRegion:   "",
+			APIRegion:      "",
+			ExpectedRegion: "not-global",
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.Name, func(t *testing.T) {
+			httpTest(t, func(c *Config) { c.Region = tc.ExpectedRegion }, func(s *TestAgent) {
+				// Create the job
+				job := MockRegionalJob()
+
+				if tc.ConfigRegion == "" {
+					job.Region = nil
+				} else {
+					job.Region = &tc.ConfigRegion
+				}
+
+				args := api.JobRegisterRequest{
+					Job: job,
+					WriteRequest: api.WriteRequest{
+						Namespace: api.DefaultNamespace,
+						Region:    tc.APIRegion,
+					},
+				}
+
+				buf := encodeReq(args)
+
+				// Make the HTTP request
+				url := "/v1/job/" + *job.ID
+
+				req, err := http.NewRequest("PUT", url, buf)
+				require.NoError(t, err)
+				respW := httptest.NewRecorder()
+
+				// Make the request
+				obj, err := s.Server.JobSpecificRequest(respW, req)
+				require.NoError(t, err)
+
+				// Check the response
+				dereg := obj.(structs.JobRegisterResponse)
+				require.NotEmpty(t, dereg.EvalID)
+
+				// Check for the index
+				require.NotEmpty(t, respW.HeaderMap.Get("X-Nomad-Index"), "missing index")
+
+				// Check the job is registered
+				getReq := structs.JobSpecificRequest{
+					JobID: *job.ID,
+					QueryOptions: structs.QueryOptions{
+						Region:    tc.ExpectedRegion,
+						Namespace: structs.DefaultNamespace,
+					},
+				}
+				var getResp structs.SingleJobResponse
+				err = s.Agent.RPC("Job.GetJob", &getReq, &getResp)
+				require.NoError(t, err)
+				require.NotNil(t, getResp.Job, "job does not exist")
+				require.Equal(t, tc.ExpectedRegion, getResp.Job.Region)
+			})
+		})
+	}
+}
+
 func TestHTTP_JobDelete(t *testing.T) {
 	t.Parallel()
 	httpTest(t, nil, func(s *TestAgent) {
@@ -796,6 +896,8 @@ func TestHTTP_JobDeployments(t *testing.T) {
 		state := s.Agent.server.State()
 		d := mock.Deployment()
 		d.JobID = j.ID
+		d.JobCreateIndex = resp.JobModifyIndex
+
 		assert.Nil(state.UpsertDeployment(1000, d), "UpsertDeployment")
 
 		// Make the HTTP request
@@ -838,6 +940,7 @@ func TestHTTP_JobDeployment(t *testing.T) {
 		state := s.Agent.server.State()
 		d := mock.Deployment()
 		d.JobID = j.ID
+		d.JobCreateIndex = resp.JobModifyIndex
 		assert.Nil(state.UpsertDeployment(1000, d), "UpsertDeployment")
 
 		// Make the HTTP request
@@ -985,7 +1088,7 @@ func TestHTTP_JobPlan(t *testing.T) {
 	t.Parallel()
 	httpTest(t, nil, func(s *TestAgent) {
 		// Create the job
-		job := api.MockJob()
+		job := MockJob()
 		args := api.JobPlanRequest{
 			Job:  job,
 			Diff: true,
@@ -1021,6 +1124,81 @@ func TestHTTP_JobPlan(t *testing.T) {
 	})
 }
 
+func TestHTTP_JobPlanRegion(t *testing.T) {
+	t.Parallel()
+
+	cases := []struct {
+		Name           string
+		ConfigRegion   string
+		APIRegion      string
+		ExpectedRegion string
+	}{
+		{
+			Name:           "api region takes precedence",
+			ConfigRegion:   "not-global",
+			APIRegion:      "north-america",
+			ExpectedRegion: "north-america",
+		},
+		{
+			Name:           "config region is set",
+			ConfigRegion:   "north-america",
+			APIRegion:      "",
+			ExpectedRegion: "north-america",
+		},
+		{
+			Name:           "api region is set",
+			ConfigRegion:   "",
+			APIRegion:      "north-america",
+			ExpectedRegion: "north-america",
+		},
+		{
+			Name:           "falls back to default if no region is provided",
+			ConfigRegion:   "",
+			APIRegion:      "",
+			ExpectedRegion: "global",
+		},
+	}
+
+	for _, tc := range cases {
+		t.Run(tc.Name, func(t *testing.T) {
+			httpTest(t, func(c *Config) { c.Region = tc.ExpectedRegion }, func(s *TestAgent) {
+				// Create the job
+				job := MockRegionalJob()
+
+				if tc.ConfigRegion == "" {
+					job.Region = nil
+				} else {
+					job.Region = &tc.ConfigRegion
+				}
+
+				args := api.JobPlanRequest{
+					Job:  job,
+					Diff: true,
+					WriteRequest: api.WriteRequest{
+						Region:    tc.APIRegion,
+						Namespace: api.DefaultNamespace,
+					},
+				}
+				buf := encodeReq(args)
+
+				// Make the HTTP request
+				req, err := http.NewRequest("PUT", "/v1/job/"+*job.ID+"/plan", buf)
+				require.NoError(t, err)
+				respW := httptest.NewRecorder()
+
+				// Make the request
+				obj, err := s.Server.JobSpecificRequest(respW, req)
+				require.NoError(t, err)
+
+				// Check the response
+				plan := obj.(structs.JobPlanResponse)
+				require.NotNil(t, plan.Annotations)
+				require.NotNil(t, plan.Diff)
+			})
+		})
+	}
+}
+
 func TestHTTP_JobDispatch(t *testing.T) {
 	t.Parallel()
 	httpTest(t, nil, func(s *TestAgent) {
@@ -1211,6 +1389,14 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 				Operand: "c",
 			},
 		},
+		Affinities: []*api.Affinity{
+			{
+				LTarget: "a",
+				RTarget: "b",
+				Operand: "c",
+				Weight:  helper.Int8ToPtr(50),
+			},
+		},
 		Update: &api.UpdateStrategy{
 			Stagger:          helper.TimeToPtr(1 * time.Second),
 			MaxParallel:      helper.IntToPtr(5),
@@ -1221,6 +1407,18 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 			AutoRevert:       helper.BoolToPtr(false),
 			Canary:           helper.IntToPtr(1),
 		},
+		Spreads: []*api.Spread{
+			{
+				Attribute: "${meta.rack}",
+				Weight:    helper.Int8ToPtr(100),
+				SpreadTarget: []*api.SpreadTarget{
+					{
+						Value:   "r1",
+						Percent: 50,
+					},
+				},
+			},
+		},
 		Periodic: &api.PeriodicConfig{
 			Enabled:         helper.BoolToPtr(true),
 			Spec:            helper.StringToPtr("spec"),
@@ -1248,6 +1446,14 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 						Operand: "z",
 					},
 				},
+				Affinities: []*api.Affinity{
+					{
+						LTarget: "x",
+						RTarget: "y",
+						Operand: "z",
+						Weight:  helper.Int8ToPtr(100),
+					},
+				},
 				RestartPolicy: &api.RestartPolicy{
 					Interval: helper.TimeToPtr(1 * time.Second),
 					Attempts: helper.IntToPtr(5),
@@ -1268,6 +1474,18 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 					MinHealthyTime:  helper.TimeToPtr(12 * time.Hour),
 					HealthyDeadline: helper.TimeToPtr(12 * time.Hour),
 				},
+				Spreads: []*api.Spread{
+					{
+						Attribute: "${node.datacenter}",
+						Weight:    helper.Int8ToPtr(100),
+						SpreadTarget: []*api.SpreadTarget{
+							{
+								Value:   "dc1",
+								Percent: 100,
+							},
+						},
+					},
+				},
 				EphemeralDisk: &api.EphemeralDisk{
 					SizeMB:  helper.IntToPtr(100),
 					Sticky:  helper.BoolToPtr(true),
@@ -1280,10 +1498,54 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 					ProgressDeadline: helper.TimeToPtr(5 * time.Minute),
 					AutoRevert:       helper.BoolToPtr(true),
 				},
-
 				Meta: map[string]string{
 					"key": "value",
 				},
+				Services: []*api.Service{
+					{
+						Name:       "groupserviceA",
+						Tags:       []string{"a", "b"},
+						CanaryTags: []string{"d", "e"},
+						PortLabel:  "1234",
+						Meta: map[string]string{
+							"servicemeta": "foobar",
+						},
+						CheckRestart: &api.CheckRestart{
+							Limit: 4,
+							Grace: helper.TimeToPtr(11 * time.Second),
+						},
+						Checks: []api.ServiceCheck{
+							{
+								Id:            "hello",
+								Name:          "bar",
+								Type:          "http",
+								Command:       "foo",
+								Args:          []string{"a", "b"},
+								Path:          "/check",
+								Protocol:      "http",
+								PortLabel:     "foo",
+								AddressMode:   "driver",
+								GRPCService:   "foo.Bar",
+								GRPCUseTLS:    true,
+								Interval:      4 * time.Second,
+								Timeout:       2 * time.Second,
+								InitialStatus: "ok",
+								CheckRestart: &api.CheckRestart{
+									Limit:          3,
+									IgnoreWarnings: true,
+								},
+								TaskName: "task1",
+							},
+						},
+						Connect: &api.ConsulConnect{
+							Native: false,
+							SidecarService: &api.ConsulSidecarService{
+								Tags: []string{"f", "g"},
+								Port: "9000",
+							},
+						},
+					},
+				},
 				Tasks: []*api.Task{
 					{
 						Name:   "task1",
@@ -1303,6 +1565,14 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 								Operand: "z",
 							},
 						},
+						Affinities: []*api.Affinity{
+							{
+								LTarget: "a",
+								RTarget: "b",
+								Operand: "c",
+								Weight:  helper.Int8ToPtr(50),
+							},
+						},
 
 						Services: []*api.Service{
 							{
@@ -1311,6 +1581,9 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 								Tags:       []string{"1", "2"},
 								CanaryTags: []string{"3", "4"},
 								PortLabel:  "foo",
+								Meta: map[string]string{
+									"servicemeta": "foobar",
+								},
 								CheckRestart: &api.CheckRestart{
 									Limit: 4,
 									Grace: helper.TimeToPtr(11 * time.Second),
@@ -1368,6 +1641,31 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 									},
 								},
 							},
+							Devices: []*api.RequestedDevice{
+								{
+									Name:  "nvidia/gpu",
+									Count: helper.Uint64ToPtr(4),
+									Constraints: []*api.Constraint{
+										{
+											LTarget: "x",
+											RTarget: "y",
+											Operand: "z",
+										},
+									},
+									Affinities: []*api.Affinity{
+										{
+											LTarget: "a",
+											RTarget: "b",
+											Operand: "c",
+											Weight:  helper.Int8ToPtr(50),
+										},
+									},
+								},
+								{
+									Name:  "gpu",
+									Count: nil,
+								},
+							},
 						},
 						Meta: map[string]string{
 							"lol": "code",
@@ -1443,6 +1741,26 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 				Operand: "c",
 			},
 		},
+		Affinities: []*structs.Affinity{
+			{
+				LTarget: "a",
+				RTarget: "b",
+				Operand: "c",
+				Weight:  50,
+			},
+		},
+		Spreads: []*structs.Spread{
+			{
+				Attribute: "${meta.rack}",
+				Weight:    100,
+				SpreadTarget: []*structs.SpreadTarget{
+					{
+						Value:   "r1",
+						Percent: 50,
+					},
+				},
+			},
+		},
 		Update: structs.UpdateStrategy{
 			Stagger:     1 * time.Second,
 			MaxParallel: 5,
@@ -1474,12 +1792,32 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 						Operand: "z",
 					},
 				},
+				Affinities: []*structs.Affinity{
+					{
+						LTarget: "x",
+						RTarget: "y",
+						Operand: "z",
+						Weight:  100,
+					},
+				},
 				RestartPolicy: &structs.RestartPolicy{
 					Interval: 1 * time.Second,
 					Attempts: 5,
 					Delay:    10 * time.Second,
 					Mode:     "delay",
 				},
+				Spreads: []*structs.Spread{
+					{
+						Attribute: "${node.datacenter}",
+						Weight:    100,
+						SpreadTarget: []*structs.SpreadTarget{
+							{
+								Value:   "dc1",
+								Percent: 100,
+							},
+						},
+					},
+				},
 				ReschedulePolicy: &structs.ReschedulePolicy{
 					Interval:      12 * time.Hour,
 					Attempts:      5,
@@ -1507,11 +1845,54 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 					HealthyDeadline:  5 * time.Minute,
 					ProgressDeadline: 5 * time.Minute,
 					AutoRevert:       true,
+					AutoPromote:      false,
 					Canary:           1,
 				},
 				Meta: map[string]string{
 					"key": "value",
 				},
+				Services: []*structs.Service{
+					{
+						Name:        "groupserviceA",
+						Tags:        []string{"a", "b"},
+						CanaryTags:  []string{"d", "e"},
+						PortLabel:   "1234",
+						AddressMode: "auto",
+						Meta: map[string]string{
+							"servicemeta": "foobar",
+						},
+						Checks: []*structs.ServiceCheck{
+							{
+								Name:          "bar",
+								Type:          "http",
+								Command:       "foo",
+								Args:          []string{"a", "b"},
+								Path:          "/check",
+								Protocol:      "http",
+								PortLabel:     "foo",
+								AddressMode:   "driver",
+								GRPCService:   "foo.Bar",
+								GRPCUseTLS:    true,
+								Interval:      4 * time.Second,
+								Timeout:       2 * time.Second,
+								InitialStatus: "ok",
+								CheckRestart: &structs.CheckRestart{
+									Grace:          11 * time.Second,
+									Limit:          3,
+									IgnoreWarnings: true,
+								},
+								TaskName: "task1",
+							},
+						},
+						Connect: &structs.ConsulConnect{
+							Native: false,
+							SidecarService: &structs.ConsulSidecarService{
+								Tags: []string{"f", "g"},
+								Port: "9000",
+							},
+						},
+					},
+				},
 				Tasks: []*structs.Task{
 					{
 						Name:   "task1",
@@ -1528,6 +1909,14 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 								Operand: "z",
 							},
 						},
+						Affinities: []*structs.Affinity{
+							{
+								LTarget: "a",
+								RTarget: "b",
+								Operand: "c",
+								Weight:  50,
+							},
+						},
 						Env: map[string]string{
 							"hello": "world",
 						},
@@ -1538,6 +1927,9 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 								CanaryTags:  []string{"3", "4"},
 								PortLabel:   "foo",
 								AddressMode: "auto",
+								Meta: map[string]string{
+									"servicemeta": "foobar",
+								},
 								Checks: []*structs.ServiceCheck{
 									{
 										Name:          "bar",
@@ -1594,6 +1986,31 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 									},
 								},
 							},
+							Devices: []*structs.RequestedDevice{
+								{
+									Name:  "nvidia/gpu",
+									Count: 4,
+									Constraints: []*structs.Constraint{
+										{
+											LTarget: "x",
+											RTarget: "y",
+											Operand: "z",
+										},
+									},
+									Affinities: []*structs.Affinity{
+										{
+											LTarget: "a",
+											RTarget: "b",
+											Operand: "c",
+											Weight:  50,
+										},
+									},
+								},
+								{
+									Name:  "gpu",
+									Count: 1,
+								},
+							},
 						},
 						Meta: map[string]string{
 							"lol": "code",
@@ -1888,3 +2305,119 @@ func TestJobs_ApiJobToStructsJob(t *testing.T) {
 		t.Fatalf("bad:\n%s", strings.Join(diff, "\n"))
 	}
 }
+
+func TestJobs_ApiJobToStructsJobUpdate(t *testing.T) {
+	apiJob := &api.Job{
+		Update: &api.UpdateStrategy{
+			Stagger:          helper.TimeToPtr(1 * time.Second),
+			MaxParallel:      helper.IntToPtr(5),
+			HealthCheck:      helper.StringToPtr(structs.UpdateStrategyHealthCheck_Manual),
+			MinHealthyTime:   helper.TimeToPtr(1 * time.Minute),
+			HealthyDeadline:  helper.TimeToPtr(3 * time.Minute),
+			ProgressDeadline: helper.TimeToPtr(3 * time.Minute),
+			AutoRevert:       helper.BoolToPtr(false),
+			AutoPromote:      nil,
+			Canary:           helper.IntToPtr(1),
+		},
+		TaskGroups: []*api.TaskGroup{
+			{
+				Update: &api.UpdateStrategy{
+					Canary:     helper.IntToPtr(2),
+					AutoRevert: helper.BoolToPtr(true),
+				},
+			}, {
+				Update: &api.UpdateStrategy{
+					Canary:      helper.IntToPtr(3),
+					AutoPromote: helper.BoolToPtr(true),
+				},
+			},
+		},
+	}
+
+	structsJob := ApiJobToStructJob(apiJob)
+
+	// Update has been moved from job down to the groups
+	jobUpdate := structs.UpdateStrategy{
+		Stagger:          1000000000,
+		MaxParallel:      5,
+		HealthCheck:      "",
+		MinHealthyTime:   0,
+		HealthyDeadline:  0,
+		ProgressDeadline: 0,
+		AutoRevert:       false,
+		AutoPromote:      false,
+		Canary:           0,
+	}
+
+	// But the groups inherit settings from the job update
+	group1 := structs.UpdateStrategy{
+		Stagger:          1000000000,
+		MaxParallel:      5,
+		HealthCheck:      "manual",
+		MinHealthyTime:   60000000000,
+		HealthyDeadline:  180000000000,
+		ProgressDeadline: 180000000000,
+		AutoRevert:       true,
+		AutoPromote:      false,
+		Canary:           2,
+	}
+
+	group2 := structs.UpdateStrategy{
+		Stagger:          1000000000,
+		MaxParallel:      5,
+		HealthCheck:      "manual",
+		MinHealthyTime:   60000000000,
+		HealthyDeadline:  180000000000,
+		ProgressDeadline: 180000000000,
+		AutoRevert:       false,
+		AutoPromote:      true,
+		Canary:           3,
+	}
+
+	require.Equal(t, jobUpdate, structsJob.Update)
+	require.Equal(t, group1, *structsJob.TaskGroups[0].Update)
+	require.Equal(t, group2, *structsJob.TaskGroups[1].Update)
+}
+
+// TestHTTP_JobValidate_SystemMigrate asserts that a system job with a migrate
+// stanza fails to validate but does not panic (see #5477).
+func TestHTTP_JobValidate_SystemMigrate(t *testing.T) {
+	t.Parallel()
+	httpTest(t, nil, func(s *TestAgent) {
+		// Create the job
+		job := &api.Job{
+			Region:      helper.StringToPtr("global"),
+			Datacenters: []string{"dc1"},
+			ID:          helper.StringToPtr("systemmigrate"),
+			Name:        helper.StringToPtr("systemmigrate"),
+			TaskGroups: []*api.TaskGroup{
+				{Name: helper.StringToPtr("web")},
+			},
+
+			// System job...
+			Type: helper.StringToPtr("system"),
+
+			// ...with an empty migrate stanza
+			Migrate: &api.MigrateStrategy{},
+		}
+
+		args := api.JobValidateRequest{
+			Job:          job,
+			WriteRequest: api.WriteRequest{Region: "global"},
+		}
+		buf := encodeReq(args)
+
+		// Make the HTTP request
+		req, err := http.NewRequest("PUT", "/v1/validate/job", buf)
+		require.NoError(t, err)
+		respW := httptest.NewRecorder()
+
+		// Make the request
+		obj, err := s.Server.ValidateJobRequest(respW, req)
+		require.NoError(t, err)
+
+		// Check the response
+		resp := obj.(structs.JobValidateResponse)
+		require.Contains(t, resp.Error, `Job type "system" does not allow migrate block`)
+	})
+}
