@@ -9,10 +9,9 @@ import (
 	"time"
 
 	humanize "github.com/dustin/go-humanize"
-
 	"github.com/hashicorp/nomad/api"
 	"github.com/hashicorp/nomad/api/contexts"
-	"github.com/hashicorp/nomad/client"
+	"github.com/hashicorp/nomad/client/allocrunner/taskrunner/restarts"
 	"github.com/posener/complete"
 )
 
@@ -192,6 +191,11 @@ func (c *AllocStatusCommand) Run(args []string) int {
 	}
 	c.Ui.Output(output)
 
+	if len(alloc.AllocatedResources.Shared.Networks) > 0 && alloc.AllocatedResources.Shared.Networks[0].HasPorts() {
+		c.Ui.Output("")
+		c.Ui.Output(formatAllocNetworkInfo(alloc))
+	}
+
 	if short {
 		c.shortTaskStatus(alloc)
 	} else {
@@ -230,12 +234,13 @@ func formatAllocBasicInfo(alloc *api.Allocation, client *api.Client, uuidLength
 	}
 
 	basic := []string{
-		fmt.Sprintf("ID|%s", limit(alloc.ID, uuidLength)),
+		fmt.Sprintf("ID|%s", alloc.ID),
 		fmt.Sprintf("Eval ID|%s", limit(alloc.EvalID, uuidLength)),
 		fmt.Sprintf("Name|%s", alloc.Name),
 		fmt.Sprintf("Node ID|%s", limit(alloc.NodeID, uuidLength)),
+		fmt.Sprintf("Node Name|%s", alloc.NodeName),
 		fmt.Sprintf("Job ID|%s", alloc.JobID),
-		fmt.Sprintf("Job Version|%d", getVersion(alloc.Job)),
+		fmt.Sprintf("Job Version|%d", *alloc.Job.Version),
 		fmt.Sprintf("Client Status|%s", alloc.ClientStatus),
 		fmt.Sprintf("Client Description|%s", alloc.ClientDescription),
 		fmt.Sprintf("Desired Status|%s", alloc.DesiredStatus),
@@ -299,6 +304,32 @@ func formatAllocBasicInfo(alloc *api.Allocation, client *api.Client, uuidLength
 	return formatKV(basic), nil
 }
 
+func formatAllocNetworkInfo(alloc *api.Allocation) string {
+	nw := alloc.AllocatedResources.Shared.Networks[0]
+	addrs := make([]string, len(nw.DynamicPorts)+len(nw.ReservedPorts)+1)
+	addrs[0] = "Label|Dynamic|Address"
+	portFmt := func(port *api.Port, dyn string) string {
+		s := fmt.Sprintf("%s|%s|%s:%d", port.Label, dyn, nw.IP, port.Value)
+		if port.To > 0 {
+			s += fmt.Sprintf(" -> %d", port.To)
+		}
+		return s
+	}
+	for idx, port := range nw.DynamicPorts {
+		addrs[idx+1] = portFmt(&port, "yes")
+	}
+	for idx, port := range nw.ReservedPorts {
+		addrs[idx+1+len(nw.DynamicPorts)] = portFmt(&port, "yes")
+	}
+
+	var mode string
+	if nw.Mode != "" {
+		mode = fmt.Sprintf(" (mode = %q)", nw.Mode)
+	}
+
+	return fmt.Sprintf("Allocation Addresses%s\n%s", mode, formatList(addrs))
+}
+
 // futureEvalTimePretty returns when the eval is eligible to reschedule
 // relative to current time, based on the WaitUntil field
 func futureEvalTimePretty(evalID string, client *api.Client) string {
@@ -425,7 +456,7 @@ func buildDisplayMessage(event *api.TaskEvent) string {
 		desc = strings.Join(parts, ", ")
 	case api.TaskRestarting:
 		in := fmt.Sprintf("Task restarting in %v", time.Duration(event.StartDelay))
-		if event.RestartReason != "" && event.RestartReason != client.ReasonWithinPolicy {
+		if event.RestartReason != "" && event.RestartReason != restarts.ReasonWithinPolicy {
 			desc = fmt.Sprintf("%s - %s", event.RestartReason, in)
 		} else {
 			desc = in
@@ -488,8 +519,9 @@ func (c *AllocStatusCommand) outputTaskResources(alloc *api.Allocation, task str
 			addr = append(addr, fmt.Sprintf("%v: %v:%v\n", port.Label, nw.IP, port.Value))
 		}
 	}
+
 	var resourcesOutput []string
-	resourcesOutput = append(resourcesOutput, "CPU|Memory|Disk|IOPS|Addresses")
+	resourcesOutput = append(resourcesOutput, "CPU|Memory|Disk|Addresses")
 	firstAddr := ""
 	if len(addr) > 0 {
 		firstAddr = addr[0]
@@ -498,6 +530,8 @@ func (c *AllocStatusCommand) outputTaskResources(alloc *api.Allocation, task str
 	// Display the rolled up stats. If possible prefer the live statistics
 	cpuUsage := strconv.Itoa(*resource.CPU)
 	memUsage := humanize.IBytes(uint64(*resource.MemoryMB * bytesPerMegabyte))
+	var deviceStats []*api.DeviceGroupStats
+
 	if stats != nil {
 		if ru, ok := stats.Tasks[task]; ok && ru != nil && ru.ResourceUsage != nil {
 			if cs := ru.ResourceUsage.CpuStats; cs != nil {
@@ -506,19 +540,25 @@ func (c *AllocStatusCommand) outputTaskResources(alloc *api.Allocation, task str
 			if ms := ru.ResourceUsage.MemoryStats; ms != nil {
 				memUsage = fmt.Sprintf("%v/%v", humanize.IBytes(ms.RSS), memUsage)
 			}
+			deviceStats = ru.ResourceUsage.DeviceStats
 		}
 	}
-	resourcesOutput = append(resourcesOutput, fmt.Sprintf("%v MHz|%v|%v|%v|%v",
+	resourcesOutput = append(resourcesOutput, fmt.Sprintf("%v MHz|%v|%v|%v",
 		cpuUsage,
 		memUsage,
 		humanize.IBytes(uint64(*alloc.Resources.DiskMB*bytesPerMegabyte)),
-		*resource.IOPS,
 		firstAddr))
 	for i := 1; i < len(addr); i++ {
 		resourcesOutput = append(resourcesOutput, fmt.Sprintf("||||%v", addr[i]))
 	}
 	c.Ui.Output(formatListWithSpaces(resourcesOutput))
 
+	if len(deviceStats) > 0 {
+		c.Ui.Output("")
+		c.Ui.Output("Device Stats")
+		c.Ui.Output(formatList(getDeviceResources(deviceStats)))
+	}
+
 	if stats != nil {
 		if ru, ok := stats.Tasks[task]; ok && ru != nil && displayStats && ru.ResourceUsage != nil {
 			c.Ui.Output("")
@@ -532,6 +572,8 @@ func (c *AllocStatusCommand) outputTaskResources(alloc *api.Allocation, task str
 func (c *AllocStatusCommand) outputVerboseResourceUsage(task string, resourceUsage *api.ResourceUsage) {
 	memoryStats := resourceUsage.MemoryStats
 	cpuStats := resourceUsage.CpuStats
+	deviceStats := resourceUsage.DeviceStats
+
 	if memoryStats != nil && len(memoryStats.Measured) > 0 {
 		c.Ui.Output("Memory Stats")
 
@@ -547,6 +589,8 @@ func (c *AllocStatusCommand) outputVerboseResourceUsage(task string, resourceUsa
 				measuredStats = append(measuredStats, humanize.IBytes(memoryStats.Cache))
 			case "Swap":
 				measuredStats = append(measuredStats, humanize.IBytes(memoryStats.Swap))
+			case "Usage":
+				measuredStats = append(measuredStats, humanize.IBytes(memoryStats.Usage))
 			case "Max Usage":
 				measuredStats = append(measuredStats, humanize.IBytes(memoryStats.MaxUsage))
 			case "Kernel Usage":
@@ -593,6 +637,13 @@ func (c *AllocStatusCommand) outputVerboseResourceUsage(task string, resourceUsa
 		out[1] = strings.Join(measuredStats, "|")
 		c.Ui.Output(formatList(out))
 	}
+
+	if len(deviceStats) > 0 {
+		c.Ui.Output("")
+		c.Ui.Output("Device Stats")
+
+		printDeviceStats(c.Ui, deviceStats)
+	}
 }
 
 // shortTaskStatus prints out the current state of each task.
