@@ -421,10 +421,10 @@ func (j *JobGetter) ApiJob(jpath string) (*api.Job, error) {
 			return nil, fmt.Errorf("Error getting jobfile from %q: %v", jpath, err)
 		} else {
 			file, err := os.Open(job.Name())
-			defer file.Close()
 			if err != nil {
 				return nil, fmt.Errorf("Error opening file %q: %v", jpath, err)
 			}
+			defer file.Close()
 			jobfile = file
 		}
 	}
@@ -438,32 +438,6 @@ func (j *JobGetter) ApiJob(jpath string) (*api.Job, error) {
 	return jobStruct, nil
 }
 
-// COMPAT: Remove in 0.7.0
-// Nomad 0.6.0 introduces the submit time field so CLI's interacting with
-// older versions of Nomad would SEGFAULT as reported here:
-// https://github.com/hashicorp/nomad/issues/2918
-// getSubmitTime returns a submit time of the job converting to time.Time
-func getSubmitTime(job *api.Job) time.Time {
-	if job.SubmitTime != nil {
-		return time.Unix(0, *job.SubmitTime)
-	}
-
-	return time.Time{}
-}
-
-// COMPAT: Remove in 0.7.0
-// Nomad 0.6.0 introduces job Versions so CLI's interacting with
-// older versions of Nomad would SEGFAULT as reported here:
-// https://github.com/hashicorp/nomad/issues/2918
-// getVersion returns a version of the job in safely.
-func getVersion(job *api.Job) uint64 {
-	if job.Version != nil {
-		return *job.Version
-	}
-
-	return 0
-}
-
 // mergeAutocompleteFlags is used to join multiple flag completion sets.
 func mergeAutocompleteFlags(flags ...complete.Flags) complete.Flags {
 	merged := make(map[string]complete.Predictor, len(flags))
