@@ -59,8 +59,7 @@ func TestNodeDrainCommand_Detach(t *testing.T) {
 						Name:   "mock_task",
 						Driver: "mock_driver",
 						Config: map[string]interface{}{
-							"run_for":    "10m",
-							"exit_after": "10m",
+							"run_for": "10m",
 						},
 					},
 				},
@@ -114,6 +113,9 @@ func TestNodeDrainCommand_Monitor(t *testing.T) {
 		if len(nodes) == 0 {
 			return false, fmt.Errorf("missing node")
 		}
+		if _, ok := nodes[0].Drivers["mock_driver"]; !ok {
+			return false, fmt.Errorf("mock_driver not ready")
+		}
 		nodeID = nodes[0].ID
 		return true, nil
 	}, func(err error) {
@@ -215,9 +217,7 @@ func TestNodeDrainCommand_Monitor(t *testing.T) {
 	cmd := &NodeDrainCommand{Meta: Meta{Ui: ui}}
 	args := []string{"-address=" + url, "-self", "-enable", "-deadline", "1s", "-ignore-system"}
 	t.Logf("Running: %v", args)
-	if code := cmd.Run(args); code != 0 {
-		t.Fatalf("expected exit 0, got: %d\n%s", code, outBuf.String())
-	}
+	require.Zero(cmd.Run(args))
 
 	out := outBuf.String()
 	t.Logf("Output:\n%s", out)
@@ -226,7 +226,7 @@ func TestNodeDrainCommand_Monitor(t *testing.T) {
 	// monitor goroutines may start only after some or all the allocs have been
 	// migrated.
 	if !testutil.IsTravis() {
-		require.Contains(out, "marked all allocations for migration")
+		require.Contains(out, "Drain complete for node")
 		for _, a := range allocs {
 			if *a.Job.Type == "system" {
 				if strings.Contains(out, a.ID) {
@@ -238,7 +238,7 @@ func TestNodeDrainCommand_Monitor(t *testing.T) {
 			require.Contains(out, fmt.Sprintf("Alloc %q draining", a.ID))
 		}
 
-		expected := fmt.Sprintf("All allocations on node %q have stopped.\n", nodeID)
+		expected := fmt.Sprintf("All allocations on node %q have stopped\n", nodeID)
 		if !strings.HasSuffix(out, expected) {
 			t.Fatalf("expected output to end with:\n%s", expected)
 		}
@@ -248,9 +248,7 @@ func TestNodeDrainCommand_Monitor(t *testing.T) {
 	outBuf.Reset()
 	args = []string{"-address=" + url, "-self", "-monitor", "-ignore-system"}
 	t.Logf("Running: %v", args)
-	if code := cmd.Run(args); code != 0 {
-		t.Fatalf("expected exit 0, got: %d\n%s", code, outBuf.String())
-	}
+	require.Zero(cmd.Run(args))
 
 	out = outBuf.String()
 	t.Logf("Output:\n%s", out)
@@ -296,7 +294,6 @@ func TestNodeDrainCommand_Monitor_NoDrainStrategy(t *testing.T) {
 	out := outBuf.String()
 	t.Logf("Output:\n%s", out)
 
-	require.Contains(out, "Monitoring node")
 	require.Contains(out, "No drain strategy set")
 }
 
