@@ -7,12 +7,11 @@ import (
 	"testing"
 	"time"
 
+	capi "github.com/hashicorp/consul/api"
 	"github.com/hashicorp/nomad/api"
 	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/kr/pretty"
-
-	capi "github.com/hashicorp/consul/api"
 )
 
 func TestParse(t *testing.T) {
@@ -44,6 +43,37 @@ func TestParse(t *testing.T) {
 						RTarget: "windows",
 						Operand: "=",
 					},
+					{
+						LTarget: "${attr.vault.version}",
+						RTarget: ">= 0.6.1",
+						Operand: "semver",
+					},
+				},
+
+				Affinities: []*api.Affinity{
+					{
+						LTarget: "${meta.team}",
+						RTarget: "mobile",
+						Operand: "=",
+						Weight:  helper.Int8ToPtr(50),
+					},
+				},
+
+				Spreads: []*api.Spread{
+					{
+						Attribute: "${meta.rack}",
+						Weight:    helper.Int8ToPtr(100),
+						SpreadTarget: []*api.SpreadTarget{
+							{
+								Value:   "r1",
+								Percent: 40,
+							},
+							{
+								Value:   "r2",
+								Percent: 60,
+							},
+						},
+					},
 				},
 
 				Update: &api.UpdateStrategy{
@@ -54,12 +84,14 @@ func TestParse(t *testing.T) {
 					HealthyDeadline:  helper.TimeToPtr(10 * time.Minute),
 					ProgressDeadline: helper.TimeToPtr(10 * time.Minute),
 					AutoRevert:       helper.BoolToPtr(true),
+					AutoPromote:      helper.BoolToPtr(true),
 					Canary:           helper.IntToPtr(1),
 				},
 
 				TaskGroups: []*api.TaskGroup{
 					{
 						Name: helper.StringToPtr("outside"),
+
 						Tasks: []*api.Task{
 							{
 								Name:   "outside",
@@ -84,6 +116,21 @@ func TestParse(t *testing.T) {
 								Operand: "=",
 							},
 						},
+
+						Volumes: map[string]*api.VolumeRequest{
+							"foo": {
+								Name: "foo",
+								Type: "host",
+							},
+						},
+						Affinities: []*api.Affinity{
+							{
+								LTarget: "${node.datacenter}",
+								RTarget: "dc2",
+								Operand: "=",
+								Weight:  helper.Int8ToPtr(100),
+							},
+						},
 						Meta: map[string]string{
 							"elb_mode":     "tcp",
 							"elb_interval": "10",
@@ -95,6 +142,26 @@ func TestParse(t *testing.T) {
 							Delay:    helper.TimeToPtr(15 * time.Second),
 							Mode:     helper.StringToPtr("delay"),
 						},
+						Spreads: []*api.Spread{
+							{
+								Attribute: "${node.datacenter}",
+								Weight:    helper.Int8ToPtr(50),
+								SpreadTarget: []*api.SpreadTarget{
+									{
+										Value:   "dc1",
+										Percent: 50,
+									},
+									{
+										Value:   "dc2",
+										Percent: 25,
+									},
+									{
+										Value:   "dc3",
+										Percent: 25,
+									},
+								},
+							},
+						},
 						ReschedulePolicy: &api.ReschedulePolicy{
 							Interval: helper.TimeToPtr(12 * time.Hour),
 							Attempts: helper.IntToPtr(5),
@@ -110,6 +177,7 @@ func TestParse(t *testing.T) {
 							HealthyDeadline:  helper.TimeToPtr(1 * time.Minute),
 							ProgressDeadline: helper.TimeToPtr(1 * time.Minute),
 							AutoRevert:       helper.BoolToPtr(false),
+							AutoPromote:      helper.BoolToPtr(false),
 							Canary:           helper.IntToPtr(2),
 						},
 						Migrate: &api.MigrateStrategy{
@@ -123,6 +191,7 @@ func TestParse(t *testing.T) {
 								Name:   "binstore",
 								Driver: "docker",
 								User:   "bob",
+								Kind:   "connect-proxy:test",
 								Config: map[string]interface{}{
 									"image": "hashicorp/binstore",
 									"labels": []map[string]interface{}{
@@ -131,6 +200,20 @@ func TestParse(t *testing.T) {
 										},
 									},
 								},
+								VolumeMounts: []*api.VolumeMount{
+									{
+										Volume:      helper.StringToPtr("foo"),
+										Destination: helper.StringToPtr("/mnt/foo"),
+									},
+								},
+								Affinities: []*api.Affinity{
+									{
+										LTarget: "${meta.foo}",
+										RTarget: "a,b,c",
+										Operand: "set_contains",
+										Weight:  helper.Int8ToPtr(25),
+									},
+								},
 								Services: []*api.Service{
 									{
 										Tags:       []string{"foo", "bar"},
@@ -168,6 +251,31 @@ func TestParse(t *testing.T) {
 											DynamicPorts:  []api.Port{{Label: "http", Value: 0}, {Label: "https", Value: 0}, {Label: "admin", Value: 0}},
 										},
 									},
+									Devices: []*api.RequestedDevice{
+										{
+											Name:  "nvidia/gpu",
+											Count: helper.Uint64ToPtr(10),
+											Constraints: []*api.Constraint{
+												{
+													LTarget: "${device.attr.memory}",
+													RTarget: "2GB",
+													Operand: ">",
+												},
+											},
+											Affinities: []*api.Affinity{
+												{
+													LTarget: "${device.model}",
+													RTarget: "1080ti",
+													Operand: "=",
+													Weight:  helper.Int8ToPtr(50),
+												},
+											},
+										},
+										{
+											Name:  "intel/gpu",
+											Count: nil,
+										},
+									},
 								},
 								KillTimeout:   helper.TimeToPtr(22 * time.Second),
 								ShutdownDelay: 11 * time.Second,
@@ -230,7 +338,6 @@ func TestParse(t *testing.T) {
 								Resources: &api.Resources{
 									CPU:      helper.IntToPtr(500),
 									MemoryMB: helper.IntToPtr(128),
-									IOPS:     helper.IntToPtr(30),
 								},
 								Constraints: []*api.Constraint{
 									{
@@ -682,6 +789,33 @@ func TestParse(t *testing.T) {
 			},
 			false,
 		},
+		{
+			"service-meta.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("service_meta"),
+				Name: helper.StringToPtr("service_meta"),
+				Type: helper.StringToPtr("service"),
+				TaskGroups: []*api.TaskGroup{
+					{
+						Name: helper.StringToPtr("group"),
+						Tasks: []*api.Task{
+							{
+								Name: "task",
+								Services: []*api.Service{
+									{
+										Name: "http-service",
+										Meta: map[string]string{
+											"foo": "bar",
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			false,
+		},
 		{
 			"reschedule-job.hcl",
 			&api.Job{
@@ -784,6 +918,126 @@ func TestParse(t *testing.T) {
 			},
 			false,
 		},
+		{
+			"tg-network.hcl",
+			&api.Job{
+				ID:          helper.StringToPtr("foo"),
+				Name:        helper.StringToPtr("foo"),
+				Datacenters: []string{"dc1"},
+				TaskGroups: []*api.TaskGroup{
+					{
+						Name:  helper.StringToPtr("bar"),
+						Count: helper.IntToPtr(3),
+						Networks: []*api.NetworkResource{
+							{
+								Mode: "bridge",
+								ReservedPorts: []api.Port{
+									{
+										Label: "http",
+										Value: 80,
+										To:    8080,
+									},
+								},
+							},
+						},
+						Services: []*api.Service{
+							{
+								Name:       "connect-service",
+								Tags:       []string{"foo", "bar"},
+								CanaryTags: []string{"canary", "bam"},
+								PortLabel:  "1234",
+								Connect: &api.ConsulConnect{
+									SidecarService: &api.ConsulSidecarService{
+										Tags: []string{"side1", "side2"},
+										Proxy: &api.ConsulProxy{
+											Upstreams: []*api.ConsulUpstream{
+												{
+													DestinationName: "other-service",
+													LocalBindPort:   4567,
+												},
+											},
+										},
+									},
+									SidecarTask: &api.SidecarTask{
+										Resources: &api.Resources{
+											CPU:      helper.IntToPtr(500),
+											MemoryMB: helper.IntToPtr(1024),
+										},
+										Env: map[string]string{
+											"FOO": "abc",
+										},
+										ShutdownDelay: helper.TimeToPtr(5 * time.Second),
+									},
+								},
+							},
+						},
+						Tasks: []*api.Task{
+							{
+								Name:   "bar",
+								Driver: "raw_exec",
+								Config: map[string]interface{}{
+									"command": "bash",
+									"args":    []interface{}{"-c", "echo hi"},
+								},
+								Resources: &api.Resources{
+									Networks: []*api.NetworkResource{
+										{
+											MBits: helper.IntToPtr(10),
+										},
+									},
+								},
+							},
+						},
+					},
+				},
+			},
+			false,
+		},
+
+		{
+			"tg-service-check.hcl",
+			&api.Job{
+				ID:   helper.StringToPtr("group_service_check_script"),
+				Name: helper.StringToPtr("group_service_check_script"),
+				TaskGroups: []*api.TaskGroup{
+					{
+						Name:  helper.StringToPtr("group"),
+						Count: helper.IntToPtr(1),
+						Networks: []*api.NetworkResource{
+							{
+								Mode: "bridge",
+								ReservedPorts: []api.Port{
+									{
+										Label: "http",
+										Value: 80,
+										To:    8080,
+									},
+								},
+							},
+						},
+						Services: []*api.Service{
+							{
+								Name:      "foo-service",
+								PortLabel: "http",
+								Checks: []api.ServiceCheck{
+									{
+										Name:          "check-name",
+										Type:          "script",
+										Command:       "/bin/true",
+										Interval:      time.Duration(10 * time.Second),
+										Timeout:       time.Duration(2 * time.Second),
+										InitialStatus: "passing",
+										TaskName:      "foo",
+									},
+								},
+							},
+						},
+						Tasks: []*api.Task{{Name: "foo"}},
+					},
+				},
+			},
+			false,
+		},
 	}
 
 	for _, tc := range cases {
@@ -852,7 +1106,7 @@ func TestIncorrectKey(t *testing.T) {
 		t.Fatalf("Expected an error")
 	}
 
-	if !strings.Contains(err.Error(), "* group: 'binsl', task: 'binstore', service: 'foo', check -> invalid key: nterval") {
+	if !strings.Contains(err.Error(), "* group: 'binsl', task: 'binstore', service (0): 'foo', check -> invalid key: nterval") {
 		t.Fatalf("Expected key error; got %v", err)
 	}
 }
