@@ -8,7 +8,10 @@ import (
 	"runtime"
 	"time"
 
+	log "github.com/hashicorp/go-hclog"
+
 	"github.com/hashicorp/memberlist"
+	"github.com/hashicorp/nomad/helper/pluginutils/loader"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/nomad/structs/config"
@@ -72,6 +75,9 @@ type Config struct {
 	// logs will go to stderr.
 	LogOutput io.Writer
 
+	// Logger is the logger used by the server.
+	Logger log.InterceptLogger
+
 	// ProtocolVersion is the protocol version to speak. This must be between
 	// ProtocolVersionMin and ProtocolVersionMax.
 	ProtocolVersion uint8
@@ -274,6 +280,10 @@ type Config struct {
 	// key/value/tag format, or simply a key/value format
 	DisableTaggedMetrics bool
 
+	// DisableDispatchedJobSummaryMetrics allows for ignore dispatched jobs when
+	// publishing Job summary metrics
+	DisableDispatchedJobSummaryMetrics bool
+
 	// BackwardsCompatibleMetrics determines whether to show methods of
 	// displaying metrics for older versions, or to only show the new format
 	BackwardsCompatibleMetrics bool
@@ -290,6 +300,27 @@ type Config struct {
 	// autopilot tasks, such as promoting eligible non-voters and removing
 	// dead servers.
 	AutopilotInterval time.Duration
+
+	// PluginLoader is used to load plugins.
+	PluginLoader loader.PluginCatalog
+
+	// PluginSingletonLoader is a plugin loader that will returns singleton
+	// instances of the plugins.
+	PluginSingletonLoader loader.PluginCatalog
+
+	// RPCHandshakeTimeout is the deadline by which RPC handshakes must
+	// complete. The RPC handshake includes the first byte read as well as
+	// the TLS handshake and subsequent byte read if TLS is enabled.
+	//
+	// The deadline is reset after the first byte is read so when TLS is
+	// enabled RPC connections may take (timeout * 2) to complete.
+	//
+	// 0 means no timeout.
+	RPCHandshakeTimeout time.Duration
+
+	// RPCMaxConnsPerClient is the maximum number of concurrent RPC
+	// connections from a single IP address. nil/0 means no limit.
+	RPCMaxConnsPerClient int
 }
 
 // CheckVersion is used to check if the ProtocolVersion is valid
@@ -304,7 +335,8 @@ func (c *Config) CheckVersion() error {
 	return nil
 }
 
-// DefaultConfig returns the default configuration
+// DefaultConfig returns the default configuration. Only used as the basis for
+// merging agent or test parameters.
 func DefaultConfig() *Config {
 	hostname, err := os.Hostname()
 	if err != nil {
@@ -381,8 +413,7 @@ func DefaultConfig() *Config {
 	// Disable shutdown on removal
 	c.RaftConfig.ShutdownOnRemove = false
 
-	// Enable interoperability with new raft APIs, requires all servers
-	// to be on raft v1 or higher.
+	// Default to Raft v2, update to v3 to enable new Raft and autopilot features.
 	c.RaftConfig.ProtocolVersion = 2
 
 	return c
