@@ -7,13 +7,14 @@ import (
 	"strings"
 	"time"
 
-	"github.com/armon/go-metrics"
+	metrics "github.com/armon/go-metrics"
+	log "github.com/hashicorp/go-hclog"
+	memdb "github.com/hashicorp/go-memdb"
+	multierror "github.com/hashicorp/go-multierror"
+
 	"github.com/golang/snappy"
 	"github.com/hashicorp/consul/lib"
-	memdb "github.com/hashicorp/go-memdb"
-	"github.com/hashicorp/go-multierror"
 	"github.com/hashicorp/nomad/acl"
-	"github.com/hashicorp/nomad/client/driver"
 	"github.com/hashicorp/nomad/helper"
 	"github.com/hashicorp/nomad/helper/uuid"
 	"github.com/hashicorp/nomad/nomad/state"
@@ -32,13 +33,6 @@ const (
 )
 
 var (
-	// vaultConstraint is the implicit constraint added to jobs requesting a
-	// Vault token
-	vaultConstraint = &structs.Constraint{
-		LTarget: "${attr.vault.version}",
-		RTarget: ">= 0.6.1",
-		Operand: structs.ConstraintVersion,
-	}
 
 	// allowRescheduleTransition is the transition that allows failed
 	// allocations to be force rescheduled. We create a one off
@@ -50,7 +44,29 @@ var (
 
 // Job endpoint is used for job interactions
 type Job struct {
-	srv *Server
+	srv    *Server
+	logger log.Logger
+
+	// builtin admission controllers
+	mutators   []jobMutator
+	validators []jobValidator
+}
+
+// NewJobEndpoints creates a new job endpoint with builtin admission controllers
+func NewJobEndpoints(s *Server) *Job {
+	return &Job{
+		srv:    s,
+		logger: s.logger.Named("job"),
+		mutators: []jobMutator{
+			jobConnectHook{},
+			jobCanonicalizer{},
+			jobImpliedConstraints{},
+		},
+		validators: []jobValidator{
+			jobConnectHook{},
+			jobValidate{},
+		},
+	}
 }
 
 // Register is used to upsert a job for scheduling
@@ -65,20 +81,20 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 		return fmt.Errorf("missing job for registration")
 	}
 
-	// Initialize the job fields (sets defaults and any necessary init work).
-	canonicalizeWarnings := args.Job.Canonicalize()
+	// defensive check; http layer and RPC requester should ensure namespaces are set consistently
+	if args.RequestNamespace() != args.Job.Namespace {
+		return fmt.Errorf("mismatched request namespace in request: %q, %q", args.RequestNamespace(), args.Job.Namespace)
+	}
 
-	// Add implicit constraints
-	setImplicitConstraints(args.Job)
-
-	// Validate the job and capture any warnings
-	err, warnings := validateJob(args.Job)
+	// Run admission controllers
+	job, warnings, err := j.admissionControllers(args.Job)
 	if err != nil {
 		return err
 	}
+	args.Job = job
 
 	// Set the warning message
-	reply.Warnings = structs.MergeMultierrorWarnings(warnings, canonicalizeWarnings)
+	reply.Warnings = structs.MergeMultierrorWarnings(warnings...)
 
 	// Check job submission permissions
 	if aclObj, err := j.srv.ResolveToken(args.AuthToken); err != nil {
@@ -87,13 +103,47 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 		if !aclObj.AllowNsOp(args.RequestNamespace(), acl.NamespaceCapabilitySubmitJob) {
 			return structs.ErrPermissionDenied
 		}
+
+		// Validate Volume Permsissions
+		for _, tg := range args.Job.TaskGroups {
+			for _, vol := range tg.Volumes {
+				if vol.Type != structs.VolumeTypeHost {
+					return structs.ErrPermissionDenied
+				}
+
+				// If a volume is readonly, then we allow access if the user has ReadOnly
+				// or ReadWrite access to the volume. Otherwise we only allow access if
+				// they have ReadWrite access.
+				if vol.ReadOnly {
+					if !aclObj.AllowHostVolumeOperation(vol.Source, acl.HostVolumeCapabilityMountReadOnly) &&
+						!aclObj.AllowHostVolumeOperation(vol.Source, acl.HostVolumeCapabilityMountReadWrite) {
+						return structs.ErrPermissionDenied
+					}
+				} else {
+					if !aclObj.AllowHostVolumeOperation(vol.Source, acl.HostVolumeCapabilityMountReadWrite) {
+						return structs.ErrPermissionDenied
+					}
+				}
+			}
+
+			for _, t := range tg.Tasks {
+				for _, vm := range t.VolumeMounts {
+					vol := tg.Volumes[vm.Volume]
+					if vm.PropagationMode == structs.VolumeMountPropagationBidirectional &&
+						!aclObj.AllowHostVolumeOperation(vol.Source, acl.HostVolumeCapabilityMountReadWrite) {
+						return structs.ErrPermissionDenied
+					}
+				}
+			}
+		}
+
 		// Check if override is set and we do not have permissions
 		if args.PolicyOverride {
 			if !aclObj.AllowNsOp(args.RequestNamespace(), acl.NamespaceCapabilitySentinelOverride) {
-				j.srv.logger.Printf("[WARN] nomad.job: policy override attempted without permissions for Job %q", args.Job.ID)
+				j.logger.Warn("policy override attempted without permissions for job", "job", args.Job.ID)
 				return structs.ErrPermissionDenied
 			}
-			j.srv.logger.Printf("[WARN] nomad.job: policy override set for Job %q", args.Job.ID)
+			j.logger.Warn("policy override set for job", "job", args.Job.ID)
 		}
 	}
 
@@ -171,8 +221,8 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 		return err
 	}
 	if policyWarnings != nil {
-		reply.Warnings = structs.MergeMultierrorWarnings(warnings,
-			canonicalizeWarnings, policyWarnings)
+		warnings = append(warnings, policyWarnings)
+		reply.Warnings = structs.MergeMultierrorWarnings(warnings...)
 	}
 
 	// Clear the Vault token
@@ -186,11 +236,11 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 		// Commit this update via Raft
 		fsmErr, index, err := j.srv.raftApply(structs.JobRegisterRequestType, args)
 		if err, ok := fsmErr.(error); ok && err != nil {
-			j.srv.logger.Printf("[ERR] nomad.job: Register failed: %v", err)
+			j.logger.Error("registering job failed", "error", err, "fsm", true)
 			return err
 		}
 		if err != nil {
-			j.srv.logger.Printf("[ERR] nomad.job: Register failed: %v", err)
+			j.logger.Error("registering job failed", "error", err, "raft", true)
 			return err
 		}
 
@@ -206,6 +256,7 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 	}
 
 	// Create a new evaluation
+	now := time.Now().UTC().UnixNano()
 	eval := &structs.Evaluation{
 		ID:             uuid.Generate(),
 		Namespace:      args.RequestNamespace(),
@@ -215,6 +266,8 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 		JobID:          args.Job.ID,
 		JobModifyIndex: reply.JobModifyIndex,
 		Status:         structs.EvalStatusPending,
+		CreateTime:     now,
+		ModifyTime:     now,
 	}
 	update := &structs.EvalUpdateRequest{
 		Evals:        []*structs.Evaluation{eval},
@@ -226,7 +279,7 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 	// but that the EvalUpdate does not.
 	_, evalIndex, err := j.srv.raftApply(structs.EvalUpdateRequestType, update)
 	if err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: Eval create failed: %v", err)
+		j.logger.Error("eval create failed", "error", err, "method", "register")
 		return err
 	}
 
@@ -237,67 +290,6 @@ func (j *Job) Register(args *structs.JobRegisterRequest, reply *structs.JobRegis
 	return nil
 }
 
-// setImplicitConstraints adds implicit constraints to the job based on the
-// features it is requesting.
-func setImplicitConstraints(j *structs.Job) {
-	// Get the required Vault Policies
-	policies := j.VaultPolicies()
-
-	// Get the required signals
-	signals := j.RequiredSignals()
-
-	// Hot path
-	if len(signals) == 0 && len(policies) == 0 {
-		return
-	}
-
-	// Add Vault constraints
-	for _, tg := range j.TaskGroups {
-		_, ok := policies[tg.Name]
-		if !ok {
-			// Not requesting Vault
-			continue
-		}
-
-		found := false
-		for _, c := range tg.Constraints {
-			if c.Equal(vaultConstraint) {
-				found = true
-				break
-			}
-		}
-
-		if !found {
-			tg.Constraints = append(tg.Constraints, vaultConstraint)
-		}
-	}
-
-	// Add signal constraints
-	for _, tg := range j.TaskGroups {
-		tgSignals, ok := signals[tg.Name]
-		if !ok {
-			// Not requesting Vault
-			continue
-		}
-
-		// Flatten the signals
-		required := helper.MapStringStringSliceValueSet(tgSignals)
-		sigConstraint := getSignalConstraint(required)
-
-		found := false
-		for _, c := range tg.Constraints {
-			if c.Equal(sigConstraint) {
-				found = true
-				break
-			}
-		}
-
-		if !found {
-			tg.Constraints = append(tg.Constraints, sigConstraint)
-		}
-	}
-}
-
 // getSignalConstraint builds a suitable constraint based on the required
 // signals
 func getSignalConstraint(signals []string) *structs.Constraint {
@@ -360,6 +352,17 @@ func (j *Job) Summary(args *structs.JobSummaryRequest,
 func (j *Job) Validate(args *structs.JobValidateRequest, reply *structs.JobValidateResponse) error {
 	defer metrics.MeasureSince([]string{"nomad", "job", "validate"}, time.Now())
 
+	// defensive check; http layer and RPC requester should ensure namespaces are set consistently
+	if args.RequestNamespace() != args.Job.Namespace {
+		return fmt.Errorf("mismatched request namespace in request: %q, %q", args.RequestNamespace(), args.Job.Namespace)
+	}
+
+	job, mutateWarnings, err := j.admissionMutators(args.Job)
+	if err != nil {
+		return err
+	}
+	args.Job = job
+
 	// Check for read-job permissions
 	if aclObj, err := j.srv.ResolveToken(args.AuthToken); err != nil {
 		return err
@@ -367,14 +370,8 @@ func (j *Job) Validate(args *structs.JobValidateRequest, reply *structs.JobValid
 		return structs.ErrPermissionDenied
 	}
 
-	// Initialize the job fields (sets defaults and any necessary init work).
-	canonicalizeWarnings := args.Job.Canonicalize()
-
-	// Add implicit constraints
-	setImplicitConstraints(args.Job)
-
 	// Validate the job and capture any warnings
-	err, warnings := validateJob(args.Job)
+	validateWarnings, err := j.admissionValidators(args.Job)
 	if err != nil {
 		if merr, ok := err.(*multierror.Error); ok {
 			for _, err := range merr.Errors {
@@ -387,8 +384,10 @@ func (j *Job) Validate(args *structs.JobValidateRequest, reply *structs.JobValid
 		}
 	}
 
+	validateWarnings = append(validateWarnings, mutateWarnings...)
+
 	// Set the warning message
-	reply.Warnings = structs.MergeMultierrorWarnings(warnings, canonicalizeWarnings)
+	reply.Warnings = structs.MergeMultierrorWarnings(validateWarnings...)
 	reply.DriverConfigValidated = true
 	return nil
 }
@@ -439,8 +438,11 @@ func (j *Job) Revert(args *structs.JobRevertRequest, reply *structs.JobRegisterR
 	}
 
 	// Build the register request
+	revJob := jobV.Copy()
+	// Use Vault Token from revert request to perform registration of reverted job.
+	revJob.VaultToken = args.VaultToken
 	reg := &structs.JobRegisterRequest{
-		Job:          jobV.Copy(),
+		Job:          revJob,
 		WriteRequest: args.WriteRequest,
 	}
 
@@ -495,7 +497,7 @@ func (j *Job) Stable(args *structs.JobStabilityRequest, reply *structs.JobStabil
 	// Commit this stability request via Raft
 	_, modifyIndex, err := j.srv.raftApply(structs.JobStabilityRequestType, args)
 	if err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: Job stability request failed: %v", err)
+		j.logger.Error("submitting job stability request failed", "error", err)
 		return err
 	}
 
@@ -566,6 +568,7 @@ func (j *Job) Evaluate(args *structs.JobEvaluateRequest, reply *structs.JobRegis
 	}
 
 	// Create a new evaluation
+	now := time.Now().UTC().UnixNano()
 	eval := &structs.Evaluation{
 		ID:             uuid.Generate(),
 		Namespace:      args.RequestNamespace(),
@@ -575,6 +578,8 @@ func (j *Job) Evaluate(args *structs.JobEvaluateRequest, reply *structs.JobRegis
 		JobID:          job.ID,
 		JobModifyIndex: job.ModifyIndex,
 		Status:         structs.EvalStatusPending,
+		CreateTime:     now,
+		ModifyTime:     now,
 	}
 
 	// Create a AllocUpdateDesiredTransitionRequest request with the eval and any forced rescheduled allocs
@@ -585,7 +590,7 @@ func (j *Job) Evaluate(args *structs.JobEvaluateRequest, reply *structs.JobRegis
 	_, evalIndex, err := j.srv.raftApply(structs.AllocUpdateDesiredTransitionRequestType, updateTransitionReq)
 
 	if err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: Eval create failed: %v", err)
+		j.logger.Error("eval create failed", "error", err, "method", "evaluate")
 		return err
 	}
 
@@ -630,7 +635,7 @@ func (j *Job) Deregister(args *structs.JobDeregisterRequest, reply *structs.JobD
 	// Commit this update via Raft
 	_, index, err := j.srv.raftApply(structs.JobDeregisterRequestType, args)
 	if err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: Deregister failed: %v", err)
+		j.logger.Error("deregister failed", "error", err)
 		return err
 	}
 
@@ -645,6 +650,7 @@ func (j *Job) Deregister(args *structs.JobDeregisterRequest, reply *structs.JobD
 	// Create a new evaluation
 	// XXX: The job priority / type is strange for this, since it's not a high
 	// priority even if the job was.
+	now := time.Now().UTC().UnixNano()
 	eval := &structs.Evaluation{
 		ID:             uuid.Generate(),
 		Namespace:      args.RequestNamespace(),
@@ -654,6 +660,8 @@ func (j *Job) Deregister(args *structs.JobDeregisterRequest, reply *structs.JobD
 		JobID:          args.JobID,
 		JobModifyIndex: index,
 		Status:         structs.EvalStatusPending,
+		CreateTime:     now,
+		ModifyTime:     now,
 	}
 	update := &structs.EvalUpdateRequest{
 		Evals:        []*structs.Evaluation{eval},
@@ -663,7 +671,7 @@ func (j *Job) Deregister(args *structs.JobDeregisterRequest, reply *structs.JobD
 	// Commit this evaluation via Raft
 	_, evalIndex, err := j.srv.raftApply(structs.EvalUpdateRequestType, update)
 	if err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: Eval create failed: %v", err)
+		j.logger.Error("eval create failed", "error", err, "method", "deregister")
 		return err
 	}
 
@@ -733,6 +741,7 @@ func (j *Job) BatchDeregister(args *structs.JobBatchDeregisterRequest, reply *st
 		}
 
 		// Create a new evaluation
+		now := time.Now().UTC().UnixNano()
 		eval := &structs.Evaluation{
 			ID:          uuid.Generate(),
 			Namespace:   jobNS.Namespace,
@@ -741,6 +750,8 @@ func (j *Job) BatchDeregister(args *structs.JobBatchDeregisterRequest, reply *st
 			TriggeredBy: structs.EvalTriggerJobDeregister,
 			JobID:       jobNS.ID,
 			Status:      structs.EvalStatusPending,
+			CreateTime:  now,
+			ModifyTime:  now,
 		}
 		args.Evals = append(args.Evals, eval)
 	}
@@ -748,7 +759,7 @@ func (j *Job) BatchDeregister(args *structs.JobBatchDeregisterRequest, reply *st
 	// Commit this update via Raft
 	_, index, err := j.srv.raftApply(structs.JobBatchDeregisterRequestType, args)
 	if err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: batch deregister failed: %v", err)
+		j.logger.Error("batch deregister failed", "error", err)
 		return err
 	}
 
@@ -940,13 +951,19 @@ func (j *Job) Allocations(args *structs.JobSpecificRequest,
 		return structs.ErrPermissionDenied
 	}
 
+	// Ensure JobID is set otherwise everything works and never returns
+	// allocations which can hide bugs in request code.
+	if args.JobID == "" {
+		return fmt.Errorf("missing job ID")
+	}
+
 	// Setup the blocking query
 	opts := blockingOptions{
 		queryOpts: &args.QueryOptions,
 		queryMeta: &reply.QueryMeta,
 		run: func(ws memdb.WatchSet, state *state.StateStore) error {
 			// Capture the allocations
-			allocs, err := state.AllocsByJob(ws, args.RequestNamespace(), args.JobID, args.AllAllocs)
+			allocs, err := state.AllocsByJob(ws, args.RequestNamespace(), args.JobID, args.All)
 			if err != nil {
 				return err
 			}
@@ -1037,7 +1054,7 @@ func (j *Job) Deployments(args *structs.JobSpecificRequest,
 		queryMeta: &reply.QueryMeta,
 		run: func(ws memdb.WatchSet, state *state.StateStore) error {
 			// Capture the deployments
-			deploys, err := state.DeploymentsByJobID(ws, args.RequestNamespace(), args.JobID)
+			deploys, err := state.DeploymentsByJobID(ws, args.RequestNamespace(), args.JobID, args.All)
 			if err != nil {
 				return err
 			}
@@ -1079,7 +1096,7 @@ func (j *Job) LatestDeployment(args *structs.JobSpecificRequest,
 		queryMeta: &reply.QueryMeta,
 		run: func(ws memdb.WatchSet, state *state.StateStore) error {
 			// Capture the deployments
-			deploys, err := state.DeploymentsByJobID(ws, args.RequestNamespace(), args.JobID)
+			deploys, err := state.DeploymentsByJobID(ws, args.RequestNamespace(), args.JobID, args.All)
 			if err != nil {
 				return err
 			}
@@ -1118,20 +1135,15 @@ func (j *Job) Plan(args *structs.JobPlanRequest, reply *structs.JobPlanResponse)
 		return fmt.Errorf("Job required for plan")
 	}
 
-	// Initialize the job fields (sets defaults and any necessary init work).
-	canonicalizeWarnings := args.Job.Canonicalize()
-
-	// Add implicit constraints
-	setImplicitConstraints(args.Job)
-
-	// Validate the job and capture any warnings
-	err, warnings := validateJob(args.Job)
+	// Run admission controllers
+	job, warnings, err := j.admissionControllers(args.Job)
 	if err != nil {
 		return err
 	}
+	args.Job = job
 
 	// Set the warning message
-	reply.Warnings = structs.MergeMultierrorWarnings(warnings, canonicalizeWarnings)
+	reply.Warnings = structs.MergeMultierrorWarnings(warnings...)
 
 	// Check job submission permissions, which we assume is the same for plan
 	if aclObj, err := j.srv.ResolveToken(args.AuthToken); err != nil {
@@ -1154,8 +1166,8 @@ func (j *Job) Plan(args *structs.JobPlanRequest, reply *structs.JobPlanResponse)
 		return err
 	}
 	if policyWarnings != nil {
-		reply.Warnings = structs.MergeMultierrorWarnings(warnings,
-			canonicalizeWarnings, policyWarnings)
+		warnings = append(warnings, policyWarnings)
+		reply.Warnings = structs.MergeMultierrorWarnings(warnings...)
 	}
 
 	// Acquire a snapshot of the state
@@ -1190,6 +1202,7 @@ func (j *Job) Plan(args *structs.JobPlanRequest, reply *structs.JobPlanResponse)
 	}
 
 	// Create an eval and mark it as requiring annotations and insert that as well
+	now := time.Now().UTC().UnixNano()
 	eval := &structs.Evaluation{
 		ID:             uuid.Generate(),
 		Namespace:      args.RequestNamespace(),
@@ -1200,6 +1213,9 @@ func (j *Job) Plan(args *structs.JobPlanRequest, reply *structs.JobPlanResponse)
 		JobModifyIndex: updatedIndex,
 		Status:         structs.EvalStatusPending,
 		AnnotatePlan:   true,
+		// Timestamps are added for consistency but this eval is never persisted
+		CreateTime: now,
+		ModifyTime: now,
 	}
 
 	snap.UpsertEvals(100, []*structs.Evaluation{eval})
@@ -1211,7 +1227,7 @@ func (j *Job) Plan(args *structs.JobPlanRequest, reply *structs.JobPlanResponse)
 	}
 
 	// Create the scheduler and run it
-	sched, err := scheduler.NewScheduler(eval.Type, j.srv.logger, snap, planner)
+	sched, err := scheduler.NewScheduler(eval.Type, j.logger, snap, planner)
 	if err != nil {
 		return err
 	}
@@ -1259,70 +1275,6 @@ func (j *Job) Plan(args *structs.JobPlanRequest, reply *structs.JobPlanResponse)
 	return nil
 }
 
-// validateJob validates a Job and task drivers and returns an error if there is
-// a validation problem or if the Job is of a type a user is not allowed to
-// submit.
-func validateJob(job *structs.Job) (invalid, warnings error) {
-	validationErrors := new(multierror.Error)
-	if err := job.Validate(); err != nil {
-		multierror.Append(validationErrors, err)
-	}
-
-	// Get any warnings
-	warnings = job.Warnings()
-
-	// Get the signals required
-	signals := job.RequiredSignals()
-
-	// Validate the driver configurations.
-	for _, tg := range job.TaskGroups {
-		// Get the signals for the task group
-		tgSignals, tgOk := signals[tg.Name]
-
-		for _, task := range tg.Tasks {
-			d, err := driver.NewDriver(
-				task.Driver,
-				driver.NewEmptyDriverContext(),
-			)
-			if err != nil {
-				msg := "failed to create driver for task %q in group %q for validation: %v"
-				multierror.Append(validationErrors, fmt.Errorf(msg, tg.Name, task.Name, err))
-				continue
-			}
-
-			if err := d.Validate(task.Config); err != nil {
-				formatted := fmt.Errorf("group %q -> task %q -> config: %v", tg.Name, task.Name, err)
-				multierror.Append(validationErrors, formatted)
-			}
-
-			// The task group didn't have any task that required signals
-			if !tgOk {
-				continue
-			}
-
-			// This task requires signals. Ensure the driver is capable
-			if required, ok := tgSignals[task.Name]; ok {
-				abilities := d.Abilities()
-				if !abilities.SendSignals {
-					formatted := fmt.Errorf("group %q -> task %q: driver %q doesn't support sending signals. Requested signals are %v",
-						tg.Name, task.Name, task.Driver, strings.Join(required, ", "))
-					multierror.Append(validationErrors, formatted)
-				}
-			}
-		}
-	}
-
-	if job.Type == structs.JobTypeCore {
-		multierror.Append(validationErrors, fmt.Errorf("job type cannot be core"))
-	}
-
-	if len(job.Payload) != 0 {
-		multierror.Append(validationErrors, fmt.Errorf("job can't be submitted with a payload, only dispatched"))
-	}
-
-	return validationErrors.ErrorOrNil(), warnings
-}
-
 // validateJobUpdate ensures updates to a job are valid.
 func validateJobUpdate(old, new *structs.Job) error {
 	// Validate Dispatch not set on new Jobs
@@ -1433,11 +1385,11 @@ func (j *Job) Dispatch(args *structs.JobDispatchRequest, reply *structs.JobDispa
 	// Commit this update via Raft
 	fsmErr, jobCreateIndex, err := j.srv.raftApply(structs.JobRegisterRequestType, regReq)
 	if err, ok := fsmErr.(error); ok && err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: Dispatched job register failed: %v", err)
+		j.logger.Error("dispatched job register failed", "error", err, "fsm", true)
 		return err
 	}
 	if err != nil {
-		j.srv.logger.Printf("[ERR] nomad.job: Dispatched job register failed: %v", err)
+		j.logger.Error("dispatched job register failed", "error", err, "raft", true)
 		return err
 	}
 
@@ -1448,6 +1400,7 @@ func (j *Job) Dispatch(args *structs.JobDispatchRequest, reply *structs.JobDispa
 	// If the job is periodic, we don't create an eval.
 	if !dispatchJob.IsPeriodic() {
 		// Create a new evaluation
+		now := time.Now().UTC().UnixNano()
 		eval := &structs.Evaluation{
 			ID:             uuid.Generate(),
 			Namespace:      args.RequestNamespace(),
@@ -1457,6 +1410,8 @@ func (j *Job) Dispatch(args *structs.JobDispatchRequest, reply *structs.JobDispa
 			JobID:          dispatchJob.ID,
 			JobModifyIndex: jobCreateIndex,
 			Status:         structs.EvalStatusPending,
+			CreateTime:     now,
+			ModifyTime:     now,
 		}
 		update := &structs.EvalUpdateRequest{
 			Evals:        []*structs.Evaluation{eval},
@@ -1466,7 +1421,7 @@ func (j *Job) Dispatch(args *structs.JobDispatchRequest, reply *structs.JobDispa
 		// Commit this evaluation via Raft
 		_, evalIndex, err := j.srv.raftApply(structs.EvalUpdateRequestType, update)
 		if err != nil {
-			j.srv.logger.Printf("[ERR] nomad.job: Eval create failed: %v", err)
+			j.logger.Error("eval create failed", "error", err, "method", "dispatch")
 			return err
 		}
 
