@@ -2,9 +2,7 @@ package nomad
 
 import (
 	"fmt"
-	"log"
 	"math/rand"
-	"os"
 	"reflect"
 	"sort"
 	"strconv"
@@ -13,6 +11,7 @@ import (
 	"testing"
 	"time"
 
+	"github.com/hashicorp/nomad/helper/testlog"
 	"github.com/hashicorp/nomad/nomad/mock"
 	"github.com/hashicorp/nomad/nomad/structs"
 	"github.com/hashicorp/nomad/testutil"
@@ -78,8 +77,8 @@ func (t times) Less(i, j int) bool { return t[i].Before(t[j]) }
 
 // testPeriodicDispatcher returns an enabled PeriodicDispatcher which uses the
 // MockJobEvalDispatcher.
-func testPeriodicDispatcher() (*PeriodicDispatch, *MockJobEvalDispatcher) {
-	logger := log.New(os.Stderr, "", log.LstdFlags)
+func testPeriodicDispatcher(t *testing.T) (*PeriodicDispatch, *MockJobEvalDispatcher) {
+	logger := testlog.HCLogger(t)
 	m := NewMockJobEvalDispatcher()
 	d := NewPeriodicDispatch(logger, m)
 	d.SetEnabled(true)
@@ -105,7 +104,7 @@ func testPeriodicJob(times ...time.Time) *structs.Job {
 // This tests the reported issue: https://github.com/hashicorp/nomad/issues/2829
 func TestPeriodicDispatch_SetEnabled(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 
 	// SetEnabled has been called once but do it again.
 	p.SetEnabled(true)
@@ -128,7 +127,7 @@ func TestPeriodicDispatch_SetEnabled(t *testing.T) {
 
 func TestPeriodicDispatch_Add_NonPeriodic(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 	job := mock.Job()
 	if err := p.Add(job); err != nil {
 		t.Fatalf("Add of non-periodic job failed: %v; expect no-op", err)
@@ -142,7 +141,7 @@ func TestPeriodicDispatch_Add_NonPeriodic(t *testing.T) {
 
 func TestPeriodicDispatch_Add_Periodic_Parameterized(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 	job := mock.PeriodicJob()
 	job.ParameterizedJob = &structs.ParameterizedJobConfig{}
 	if err := p.Add(job); err != nil {
@@ -157,7 +156,7 @@ func TestPeriodicDispatch_Add_Periodic_Parameterized(t *testing.T) {
 
 func TestPeriodicDispatch_Add_Periodic_Stopped(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 	job := mock.PeriodicJob()
 	job.Stop = true
 	if err := p.Add(job); err != nil {
@@ -172,7 +171,7 @@ func TestPeriodicDispatch_Add_Periodic_Stopped(t *testing.T) {
 
 func TestPeriodicDispatch_Add_UpdateJob(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 	job := mock.PeriodicJob()
 	if err := p.Add(job); err != nil {
 		t.Fatalf("Add failed %v", err)
@@ -202,7 +201,7 @@ func TestPeriodicDispatch_Add_UpdateJob(t *testing.T) {
 func TestPeriodicDispatch_Add_Remove_Namespaced(t *testing.T) {
 	assert := assert.New(t)
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 	job := mock.PeriodicJob()
 	job2 := mock.PeriodicJob()
 	job2.Namespace = "test"
@@ -219,7 +218,7 @@ func TestPeriodicDispatch_Add_Remove_Namespaced(t *testing.T) {
 
 func TestPeriodicDispatch_Add_RemoveJob(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 	job := mock.PeriodicJob()
 	if err := p.Add(job); err != nil {
 		t.Fatalf("Add failed %v", err)
@@ -244,7 +243,7 @@ func TestPeriodicDispatch_Add_RemoveJob(t *testing.T) {
 
 func TestPeriodicDispatch_Add_TriggersUpdate(t *testing.T) {
 	t.Parallel()
-	p, m := testPeriodicDispatcher()
+	p, m := testPeriodicDispatcher(t)
 
 	// Create a job that won't be evaluated for a while.
 	job := testPeriodicJob(time.Now().Add(10 * time.Second))
@@ -287,7 +286,7 @@ func TestPeriodicDispatch_Add_TriggersUpdate(t *testing.T) {
 
 func TestPeriodicDispatch_Remove_Untracked(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 	if err := p.Remove("ns", "foo"); err != nil {
 		t.Fatalf("Remove failed %v; expected a no-op", err)
 	}
@@ -295,7 +294,7 @@ func TestPeriodicDispatch_Remove_Untracked(t *testing.T) {
 
 func TestPeriodicDispatch_Remove_Tracked(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 
 	job := mock.PeriodicJob()
 	if err := p.Add(job); err != nil {
@@ -319,7 +318,7 @@ func TestPeriodicDispatch_Remove_Tracked(t *testing.T) {
 
 func TestPeriodicDispatch_Remove_TriggersUpdate(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 
 	// Create a job that will be evaluated soon.
 	job := testPeriodicJob(time.Now().Add(1 * time.Second))
@@ -349,7 +348,7 @@ func TestPeriodicDispatch_Remove_TriggersUpdate(t *testing.T) {
 
 func TestPeriodicDispatch_ForceRun_Untracked(t *testing.T) {
 	t.Parallel()
-	p, _ := testPeriodicDispatcher()
+	p, _ := testPeriodicDispatcher(t)
 
 	if _, err := p.ForceRun("ns", "foo"); err == nil {
 		t.Fatal("ForceRun of untracked job should fail")
@@ -358,7 +357,7 @@ func TestPeriodicDispatch_ForceRun_Untracked(t *testing.T) {
 
 func TestPeriodicDispatch_ForceRun_Tracked(t *testing.T) {
 	t.Parallel()
-	p, m := testPeriodicDispatcher()
+	p, m := testPeriodicDispatcher(t)
 
 	// Create a job that won't be evaluated for a while.
 	job := testPeriodicJob(time.Now().Add(10 * time.Second))
@@ -387,7 +386,7 @@ func TestPeriodicDispatch_ForceRun_Tracked(t *testing.T) {
 
 func TestPeriodicDispatch_Run_DisallowOverlaps(t *testing.T) {
 	t.Parallel()
-	p, m := testPeriodicDispatcher()
+	p, m := testPeriodicDispatcher(t)
 
 	// Create a job that will trigger two launches but disallows overlapping.
 	launch1 := time.Now().Round(1 * time.Second).Add(1 * time.Second)
@@ -417,7 +416,7 @@ func TestPeriodicDispatch_Run_DisallowOverlaps(t *testing.T) {
 
 func TestPeriodicDispatch_Run_Multiple(t *testing.T) {
 	t.Parallel()
-	p, m := testPeriodicDispatcher()
+	p, m := testPeriodicDispatcher(t)
 
 	// Create a job that will be launched twice.
 	launch1 := time.Now().Round(1 * time.Second).Add(1 * time.Second)
@@ -449,7 +448,7 @@ func TestPeriodicDispatch_Run_Multiple(t *testing.T) {
 
 func TestPeriodicDispatch_Run_SameTime(t *testing.T) {
 	t.Parallel()
-	p, m := testPeriodicDispatcher()
+	p, m := testPeriodicDispatcher(t)
 
 	// Create two job that will be launched at the same time.
 	launch := time.Now().Round(1 * time.Second).Add(1 * time.Second)
@@ -487,7 +486,7 @@ func TestPeriodicDispatch_Run_SameTime(t *testing.T) {
 
 func TestPeriodicDispatch_Run_SameID_Different_Namespace(t *testing.T) {
 	t.Parallel()
-	p, m := testPeriodicDispatcher()
+	p, m := testPeriodicDispatcher(t)
 
 	// Create two job that will be launched at the same time.
 	launch := time.Now().Round(1 * time.Second).Add(1 * time.Second)
@@ -534,7 +533,7 @@ func TestPeriodicDispatch_Run_SameID_Different_Namespace(t *testing.T) {
 // behavior.
 func TestPeriodicDispatch_Complex(t *testing.T) {
 	t.Parallel()
-	p, m := testPeriodicDispatcher()
+	p, m := testPeriodicDispatcher(t)
 
 	// Create some jobs launching at different times.
 	now := time.Now().Round(1 * time.Second)
