@@ -3,6 +3,7 @@ package nomad
 import (
 	"reflect"
 	"testing"
+	"time"
 
 	memdb "github.com/hashicorp/go-memdb"
 	"github.com/hashicorp/nomad/helper/testlog"
@@ -12,6 +13,7 @@ import (
 	"github.com/hashicorp/nomad/testutil"
 	"github.com/hashicorp/raft"
 	"github.com/stretchr/testify/assert"
+	"github.com/stretchr/testify/require"
 )
 
 const (
@@ -61,6 +63,7 @@ func testRegisterJob(t *testing.T, s *Server, j *structs.Job) {
 	}
 }
 
+// COMPAT 0.11: Tests the older unoptimized code path for applyPlan
 func TestPlanApply_applyPlan(t *testing.T) {
 	t.Parallel()
 	s1 := TestServer(t, nil)
@@ -200,7 +203,9 @@ func TestPlanApply_applyPlan(t *testing.T) {
 
 	// Check that our optimistic view is updated
 	out, _ := snap.AllocByID(ws, allocEvict.ID)
-	assert.Equal(structs.AllocDesiredStatusEvict, out.DesiredStatus)
+	if out.DesiredStatus != structs.AllocDesiredStatusEvict && out.DesiredStatus != structs.AllocDesiredStatusStop {
+		assert.Equal(structs.AllocDesiredStatusEvict, out.DesiredStatus)
+	}
 
 	// Verify plan applies cleanly
 	index, err = planWaitFuture(future)
@@ -210,7 +215,10 @@ func TestPlanApply_applyPlan(t *testing.T) {
 	// Lookup the allocation
 	allocOut, err = s1.fsm.State().AllocByID(ws, alloc.ID)
 	assert.Nil(err)
-	assert.Equal(structs.AllocDesiredStatusEvict, allocOut.DesiredStatus)
+	if allocOut.DesiredStatus != structs.AllocDesiredStatusEvict && allocOut.DesiredStatus != structs.AllocDesiredStatusStop {
+		assert.Equal(structs.AllocDesiredStatusEvict, allocOut.DesiredStatus)
+	}
+
 	assert.NotNil(allocOut.Job)
 	assert.True(allocOut.ModifyTime > 0)
 
@@ -227,6 +235,158 @@ func TestPlanApply_applyPlan(t *testing.T) {
 	assert.Equal(index, evalOut.ModifyIndex)
 }
 
+// Verifies that applyPlan properly updates the constituent objects in MemDB,
+// when the plan contains normalized allocs.
+func TestPlanApply_applyPlanWithNormalizedAllocs(t *testing.T) {
+	t.Parallel()
+	s1 := TestServer(t, func(c *Config) {
+		c.Build = "0.9.2"
+	})
+	defer s1.Shutdown()
+	testutil.WaitForLeader(t, s1.RPC)
+
+	// Register node
+	node := mock.Node()
+	testRegisterNode(t, s1, node)
+
+	// Register a fake deployment
+	oldDeployment := mock.Deployment()
+	if err := s1.State().UpsertDeployment(900, oldDeployment); err != nil {
+		t.Fatalf("UpsertDeployment failed: %v", err)
+	}
+
+	// Create a deployment
+	dnew := mock.Deployment()
+
+	// Create a deployment update for the old deployment id
+	desiredStatus, desiredStatusDescription := "foo", "bar"
+	updates := []*structs.DeploymentStatusUpdate{
+		{
+			DeploymentID:      oldDeployment.ID,
+			Status:            desiredStatus,
+			StatusDescription: desiredStatusDescription,
+		},
+	}
+
+	// Register allocs, deployment and deployment update
+	alloc := mock.Alloc()
+	stoppedAlloc := mock.Alloc()
+	stoppedAllocDiff := &structs.Allocation{
+		ID:                 stoppedAlloc.ID,
+		DesiredDescription: "Desired Description",
+		ClientStatus:       structs.AllocClientStatusLost,
+	}
+	preemptedAlloc := mock.Alloc()
+	preemptedAllocDiff := &structs.Allocation{
+		ID:                    preemptedAlloc.ID,
+		PreemptedByAllocation: alloc.ID,
+	}
+	s1.State().UpsertJobSummary(1000, mock.JobSummary(alloc.JobID))
+	s1.State().UpsertAllocs(1100, []*structs.Allocation{stoppedAlloc, preemptedAlloc})
+	// Create an eval
+	eval := mock.Eval()
+	eval.JobID = alloc.JobID
+	if err := s1.State().UpsertEvals(1, []*structs.Evaluation{eval}); err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	timestampBeforeCommit := time.Now().UTC().UnixNano()
+	planRes := &structs.PlanResult{
+		NodeAllocation: map[string][]*structs.Allocation{
+			node.ID: {alloc},
+		},
+		NodeUpdate: map[string][]*structs.Allocation{
+			stoppedAlloc.NodeID: {stoppedAllocDiff},
+		},
+		NodePreemptions: map[string][]*structs.Allocation{
+			preemptedAlloc.NodeID: {preemptedAllocDiff},
+		},
+		Deployment:        dnew,
+		DeploymentUpdates: updates,
+	}
+
+	// Snapshot the state
+	snap, err := s1.State().Snapshot()
+	if err != nil {
+		t.Fatalf("err: %v", err)
+	}
+
+	// Create the plan with a deployment
+	plan := &structs.Plan{
+		Job:               alloc.Job,
+		Deployment:        dnew,
+		DeploymentUpdates: updates,
+		EvalID:            eval.ID,
+	}
+
+	require := require.New(t)
+	assert := assert.New(t)
+
+	// Apply the plan
+	future, err := s1.applyPlan(plan, planRes, snap)
+	require.NoError(err)
+
+	// Verify our optimistic snapshot is updated
+	ws := memdb.NewWatchSet()
+	allocOut, err := snap.AllocByID(ws, alloc.ID)
+	require.NoError(err)
+	require.NotNil(allocOut)
+
+	deploymentOut, err := snap.DeploymentByID(ws, plan.Deployment.ID)
+	require.NoError(err)
+	require.NotNil(deploymentOut)
+
+	// Check plan does apply cleanly
+	index, err := planWaitFuture(future)
+	require.NoError(err)
+	assert.NotEqual(0, index)
+
+	// Lookup the allocation
+	fsmState := s1.fsm.State()
+	allocOut, err = fsmState.AllocByID(ws, alloc.ID)
+	require.NoError(err)
+	require.NotNil(allocOut)
+	assert.True(allocOut.CreateTime > 0)
+	assert.True(allocOut.ModifyTime > 0)
+	assert.Equal(allocOut.CreateTime, allocOut.ModifyTime)
+
+	// Verify stopped alloc diff applied cleanly
+	updatedStoppedAlloc, err := fsmState.AllocByID(ws, stoppedAlloc.ID)
+	require.NoError(err)
+	require.NotNil(updatedStoppedAlloc)
+	assert.True(updatedStoppedAlloc.ModifyTime > timestampBeforeCommit)
+	assert.Equal(updatedStoppedAlloc.DesiredDescription, stoppedAllocDiff.DesiredDescription)
+	assert.Equal(updatedStoppedAlloc.ClientStatus, stoppedAllocDiff.ClientStatus)
+	assert.Equal(updatedStoppedAlloc.DesiredStatus, structs.AllocDesiredStatusStop)
+
+	// Verify preempted alloc diff applied cleanly
+	updatedPreemptedAlloc, err := fsmState.AllocByID(ws, preemptedAlloc.ID)
+	require.NoError(err)
+	require.NotNil(updatedPreemptedAlloc)
+	assert.True(updatedPreemptedAlloc.ModifyTime > timestampBeforeCommit)
+	assert.Equal(updatedPreemptedAlloc.DesiredDescription,
+		"Preempted by alloc ID "+preemptedAllocDiff.PreemptedByAllocation)
+	assert.Equal(updatedPreemptedAlloc.DesiredStatus, structs.AllocDesiredStatusEvict)
+
+	// Lookup the new deployment
+	dout, err := fsmState.DeploymentByID(ws, plan.Deployment.ID)
+	require.NoError(err)
+	require.NotNil(dout)
+
+	// Lookup the updated deployment
+	dout2, err := fsmState.DeploymentByID(ws, oldDeployment.ID)
+	require.NoError(err)
+	require.NotNil(dout2)
+	assert.Equal(desiredStatus, dout2.Status)
+	assert.Equal(desiredStatusDescription, dout2.StatusDescription)
+
+	// Lookup updated eval
+	evalOut, err := fsmState.EvalByID(ws, eval.ID)
+	require.NoError(err)
+	require.NotNil(evalOut)
+	assert.Equal(index, evalOut.ModifyIndex)
+}
+
 func TestPlanApply_EvalPlan_Simple(t *testing.T) {
 	t.Parallel()
 	state := testStateStore(t)
@@ -253,7 +413,7 @@ func TestPlanApply_EvalPlan_Simple(t *testing.T) {
 	pool := NewEvaluatePool(workerPoolSize, workerPoolBufferSize)
 	defer pool.Shutdown()
 
-	result, err := evaluatePlan(pool, snap, plan, testlog.Logger(t))
+	result, err := evaluatePlan(pool, snap, plan, testlog.HCLogger(t))
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -271,6 +431,120 @@ func TestPlanApply_EvalPlan_Simple(t *testing.T) {
 	}
 }
 
+func TestPlanApply_EvalPlan_Preemption(t *testing.T) {
+	t.Parallel()
+	state := testStateStore(t)
+	node := mock.Node()
+	node.NodeResources = &structs.NodeResources{
+		Cpu: structs.NodeCpuResources{
+			CpuShares: 2000,
+		},
+		Memory: structs.NodeMemoryResources{
+			MemoryMB: 4192,
+		},
+		Disk: structs.NodeDiskResources{
+			DiskMB: 30 * 1024,
+		},
+		Networks: []*structs.NetworkResource{
+			{
+				Device: "eth0",
+				CIDR:   "192.168.0.100/32",
+				MBits:  1000,
+			},
+		},
+	}
+	state.UpsertNode(1000, node)
+
+	preemptedAlloc := mock.Alloc()
+	preemptedAlloc.NodeID = node.ID
+	preemptedAlloc.AllocatedResources = &structs.AllocatedResources{
+		Shared: structs.AllocatedSharedResources{
+			DiskMB: 25 * 1024,
+		},
+		Tasks: map[string]*structs.AllocatedTaskResources{
+			"web": {
+				Cpu: structs.AllocatedCpuResources{
+					CpuShares: 1500,
+				},
+				Memory: structs.AllocatedMemoryResources{
+					MemoryMB: 4000,
+				},
+				Networks: []*structs.NetworkResource{
+					{
+						Device:        "eth0",
+						IP:            "192.168.0.100",
+						ReservedPorts: []structs.Port{{Label: "admin", Value: 5000}},
+						MBits:         800,
+						DynamicPorts:  []structs.Port{{Label: "http", Value: 9876}},
+					},
+				},
+			},
+		},
+	}
+
+	// Insert a preempted alloc such that the alloc will fit only after preemption
+	state.UpsertAllocs(1001, []*structs.Allocation{preemptedAlloc})
+
+	alloc := mock.Alloc()
+	alloc.AllocatedResources = &structs.AllocatedResources{
+		Shared: structs.AllocatedSharedResources{
+			DiskMB: 24 * 1024,
+		},
+		Tasks: map[string]*structs.AllocatedTaskResources{
+			"web": {
+				Cpu: structs.AllocatedCpuResources{
+					CpuShares: 1500,
+				},
+				Memory: structs.AllocatedMemoryResources{
+					MemoryMB: 3200,
+				},
+				Networks: []*structs.NetworkResource{
+					{
+						Device:        "eth0",
+						IP:            "192.168.0.100",
+						ReservedPorts: []structs.Port{{Label: "admin", Value: 5000}},
+						MBits:         800,
+						DynamicPorts:  []structs.Port{{Label: "http", Value: 9876}},
+					},
+				},
+			},
+		},
+	}
+	plan := &structs.Plan{
+		Job: alloc.Job,
+		NodeAllocation: map[string][]*structs.Allocation{
+			node.ID: {alloc},
+		},
+		NodePreemptions: map[string][]*structs.Allocation{
+			node.ID: {preemptedAlloc},
+		},
+		Deployment: mock.Deployment(),
+		DeploymentUpdates: []*structs.DeploymentStatusUpdate{
+			{
+				DeploymentID:      uuid.Generate(),
+				Status:            "foo",
+				StatusDescription: "bar",
+			},
+		},
+	}
+	snap, _ := state.Snapshot()
+
+	pool := NewEvaluatePool(workerPoolSize, workerPoolBufferSize)
+	defer pool.Shutdown()
+
+	result, err := evaluatePlan(pool, snap, plan, testlog.HCLogger(t))
+
+	require := require.New(t)
+	require.NoError(err)
+	require.NotNil(result)
+
+	require.Equal(result.NodeAllocation, plan.NodeAllocation)
+	require.Equal(result.Deployment, plan.Deployment)
+	require.Equal(result.DeploymentUpdates, plan.DeploymentUpdates)
+	require.Equal(result.NodePreemptions, plan.NodePreemptions)
+
+}
+
 func TestPlanApply_EvalPlan_Partial(t *testing.T) {
 	t.Parallel()
 	state := testStateStore(t)
@@ -282,7 +556,7 @@ func TestPlanApply_EvalPlan_Partial(t *testing.T) {
 
 	alloc := mock.Alloc()
 	alloc2 := mock.Alloc() // Ensure alloc2 does not fit
-	alloc2.Resources = node2.Resources
+	alloc2.AllocatedResources = structs.NodeResourcesToAllocatedResources(node2.NodeResources)
 
 	// Create a deployment where the allocs are markeda as canaries
 	d := mock.Deployment()
@@ -300,7 +574,7 @@ func TestPlanApply_EvalPlan_Partial(t *testing.T) {
 	pool := NewEvaluatePool(workerPoolSize, workerPoolBufferSize)
 	defer pool.Shutdown()
 
-	result, err := evaluatePlan(pool, snap, plan, testlog.Logger(t))
+	result, err := evaluatePlan(pool, snap, plan, testlog.HCLogger(t))
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -340,7 +614,7 @@ func TestPlanApply_EvalPlan_Partial_AllAtOnce(t *testing.T) {
 
 	alloc := mock.Alloc()
 	alloc2 := mock.Alloc() // Ensure alloc2 does not fit
-	alloc2.Resources = node2.Resources
+	alloc2.AllocatedResources = structs.NodeResourcesToAllocatedResources(node2.NodeResources)
 	plan := &structs.Plan{
 		Job:       alloc.Job,
 		AllAtOnce: true, // Require all to make progress
@@ -361,7 +635,7 @@ func TestPlanApply_EvalPlan_Partial_AllAtOnce(t *testing.T) {
 	pool := NewEvaluatePool(workerPoolSize, workerPoolBufferSize)
 	defer pool.Shutdown()
 
-	result, err := evaluatePlan(pool, snap, plan, testlog.Logger(t))
+	result, err := evaluatePlan(pool, snap, plan, testlog.HCLogger(t))
 	if err != nil {
 		t.Fatalf("err: %v", err)
 	}
@@ -494,9 +768,9 @@ func TestPlanApply_EvalNodePlan_NodeFull(t *testing.T) {
 	alloc := mock.Alloc()
 	state := testStateStore(t)
 	node := mock.Node()
+	node.ReservedResources = nil
 	alloc.NodeID = node.ID
-	node.Resources = alloc.Resources
-	node.Reserved = nil
+	alloc.AllocatedResources = structs.NodeResourcesToAllocatedResources(node.NodeResources)
 	state.UpsertJobSummary(999, mock.JobSummary(alloc.JobID))
 	state.UpsertNode(1000, node)
 	state.UpsertAllocs(1001, []*structs.Allocation{alloc})
@@ -525,14 +799,69 @@ func TestPlanApply_EvalNodePlan_NodeFull(t *testing.T) {
 	}
 }
 
+// Test that we detect device oversubscription
+func TestPlanApply_EvalNodePlan_NodeFull_Device(t *testing.T) {
+	t.Parallel()
+	require := require.New(t)
+	alloc := mock.Alloc()
+	state := testStateStore(t)
+	node := mock.NvidiaNode()
+	node.ReservedResources = nil
+
+	nvidia0 := node.NodeResources.Devices[0].Instances[0].ID
+
+	// Have the allocation use a Nvidia device
+	alloc.NodeID = node.ID
+	alloc.AllocatedResources.Tasks["web"].Devices = []*structs.AllocatedDeviceResource{
+		{
+			Type:      "gpu",
+			Vendor:    "nvidia",
+			Name:      "1080ti",
+			DeviceIDs: []string{nvidia0},
+		},
+	}
+
+	state.UpsertJobSummary(999, mock.JobSummary(alloc.JobID))
+	state.UpsertNode(1000, node)
+	state.UpsertAllocs(1001, []*structs.Allocation{alloc})
+
+	// Alloc2 tries to use the same device
+	alloc2 := mock.Alloc()
+	alloc2.AllocatedResources.Tasks["web"].Networks = nil
+	alloc2.AllocatedResources.Tasks["web"].Devices = []*structs.AllocatedDeviceResource{
+		{
+			Type:      "gpu",
+			Vendor:    "nvidia",
+			Name:      "1080ti",
+			DeviceIDs: []string{nvidia0},
+		},
+	}
+	alloc2.NodeID = node.ID
+	state.UpsertJobSummary(1200, mock.JobSummary(alloc2.JobID))
+
+	snap, _ := state.Snapshot()
+	plan := &structs.Plan{
+		Job: alloc.Job,
+		NodeAllocation: map[string][]*structs.Allocation{
+			node.ID: {alloc2},
+		},
+	}
+
+	fit, reason, err := evaluateNodePlan(snap, plan, node.ID)
+	require.NoError(err)
+	require.False(fit)
+	require.Equal("device oversubscribed", reason)
+}
+
 func TestPlanApply_EvalNodePlan_UpdateExisting(t *testing.T) {
 	t.Parallel()
 	alloc := mock.Alloc()
 	state := testStateStore(t)
 	node := mock.Node()
-	alloc.NodeID = node.ID
-	node.Resources = alloc.Resources
+	node.ReservedResources = nil
 	node.Reserved = nil
+	alloc.NodeID = node.ID
+	alloc.AllocatedResources = structs.NodeResourcesToAllocatedResources(node.NodeResources)
 	state.UpsertNode(1000, node)
 	state.UpsertAllocs(1001, []*structs.Allocation{alloc})
 	snap, _ := state.Snapshot()
@@ -561,9 +890,9 @@ func TestPlanApply_EvalNodePlan_NodeFull_Evict(t *testing.T) {
 	alloc := mock.Alloc()
 	state := testStateStore(t)
 	node := mock.Node()
+	node.ReservedResources = nil
 	alloc.NodeID = node.ID
-	node.Resources = alloc.Resources
-	node.Reserved = nil
+	alloc.AllocatedResources = structs.NodeResourcesToAllocatedResources(node.NodeResources)
 	state.UpsertNode(1000, node)
 	state.UpsertAllocs(1001, []*structs.Allocation{alloc})
 	snap, _ := state.Snapshot()
@@ -599,10 +928,10 @@ func TestPlanApply_EvalNodePlan_NodeFull_AllocEvict(t *testing.T) {
 	alloc := mock.Alloc()
 	state := testStateStore(t)
 	node := mock.Node()
+	node.ReservedResources = nil
 	alloc.NodeID = node.ID
 	alloc.DesiredStatus = structs.AllocDesiredStatusEvict
-	node.Resources = alloc.Resources
-	node.Reserved = nil
+	alloc.AllocatedResources = structs.NodeResourcesToAllocatedResources(node.NodeResources)
 	state.UpsertNode(1000, node)
 	state.UpsertAllocs(1001, []*structs.Allocation{alloc})
 	snap, _ := state.Snapshot()
@@ -633,8 +962,8 @@ func TestPlanApply_EvalNodePlan_NodeDown_EvictOnly(t *testing.T) {
 	state := testStateStore(t)
 	node := mock.Node()
 	alloc.NodeID = node.ID
-	node.Resources = alloc.Resources
-	node.Reserved = nil
+	alloc.AllocatedResources = structs.NodeResourcesToAllocatedResources(node.NodeResources)
+	node.ReservedResources = nil
 	node.Status = structs.NodeStatusDown
 	state.UpsertNode(1000, node)
 	state.UpsertAllocs(1001, []*structs.Allocation{alloc})
