@@ -1,14 +1,15 @@
 package state
 
 import (
+	testing "github.com/mitchellh/go-testing-interface"
+
 	"github.com/hashicorp/nomad/helper/testlog"
-	"github.com/mitchellh/go-testing-interface"
 )
 
 func TestStateStore(t testing.T) *StateStore {
 	config := &StateStoreConfig{
-		LogOutput: testlog.NewWriter(t),
-		Region:    "global",
+		Logger: testlog.HCLogger(t),
+		Region: "global",
 	}
 	state, err := NewStateStore(config)
 	if err != nil {
