@@ -21,78 +21,99 @@ import (
 type ConsulConfig struct {
 	// ServerServiceName is the name of the service that Nomad uses to register
 	// servers with Consul
-	ServerServiceName string `mapstructure:"server_service_name"`
+	ServerServiceName string `hcl:"server_service_name"`
 
 	// ServerHTTPCheckName is the name of the health check that Nomad uses
 	// to register the server HTTP health check with Consul
-	ServerHTTPCheckName string `mapstructure:"server_http_check_name"`
+	ServerHTTPCheckName string `hcl:"server_http_check_name"`
 
 	// ServerSerfCheckName is the name of the health check that Nomad uses
 	// to register the server Serf health check with Consul
-	ServerSerfCheckName string `mapstructure:"server_serf_check_name"`
+	ServerSerfCheckName string `hcl:"server_serf_check_name"`
 
 	// ServerRPCCheckName is the name of the health check that Nomad uses
 	// to register the server RPC health check with Consul
-	ServerRPCCheckName string `mapstructure:"server_rpc_check_name"`
+	ServerRPCCheckName string `hcl:"server_rpc_check_name"`
 
 	// ClientServiceName is the name of the service that Nomad uses to register
 	// clients with Consul
-	ClientServiceName string `mapstructure:"client_service_name"`
+	ClientServiceName string `hcl:"client_service_name"`
 
 	// ClientHTTPCheckName is the name of the health check that Nomad uses
 	// to register the client HTTP health check with Consul
-	ClientHTTPCheckName string `mapstructure:"client_http_check_name"`
+	ClientHTTPCheckName string `hcl:"client_http_check_name"`
+
+	// Tags are optional service tags that get registered with the service
+	// in Consul
+	Tags []string `hcl:"tags"`
 
 	// AutoAdvertise determines if this Nomad Agent will advertise its
 	// services via Consul.  When true, Nomad Agent will register
 	// services with Consul.
-	AutoAdvertise *bool `mapstructure:"auto_advertise"`
+	AutoAdvertise *bool `hcl:"auto_advertise"`
 
 	// ChecksUseAdvertise specifies that Consul checks should use advertise
 	// address instead of bind address
-	ChecksUseAdvertise *bool `mapstructure:"checks_use_advertise"`
+	ChecksUseAdvertise *bool `hcl:"checks_use_advertise"`
 
-	// Addr is the address of the local Consul agent
-	Addr string `mapstructure:"address"`
+	// Addr is the HTTP endpoint address of the local Consul agent
+	//
+	// Uses Consul's default and env var.
+	Addr string `hcl:"address"`
+
+	// GRPCAddr is the gRPC endpoint address of the local Consul agent
+	GRPCAddr string `hcl:"grpc_address"`
 
 	// Timeout is used by Consul HTTP Client
-	Timeout time.Duration `mapstructure:"timeout"`
+	Timeout    time.Duration
+	TimeoutHCL string `hcl:"timeout" json:"-"`
 
 	// Token is used to provide a per-request ACL token. This options overrides
 	// the agent's default token
-	Token string `mapstructure:"token"`
+	Token string `hcl:"token"`
 
 	// Auth is the information to use for http access to Consul agent
-	Auth string `mapstructure:"auth"`
+	Auth string `hcl:"auth"`
 
 	// EnableSSL sets the transport scheme to talk to the Consul agent as https
-	EnableSSL *bool `mapstructure:"ssl"`
+	//
+	// Uses Consul's default and env var.
+	EnableSSL *bool `hcl:"ssl"`
 
 	// VerifySSL enables or disables SSL verification when the transport scheme
 	// for the consul api client is https
-	VerifySSL *bool `mapstructure:"verify_ssl"`
+	//
+	// Uses Consul's default and env var.
+	VerifySSL *bool `hcl:"verify_ssl"`
 
-	// CAFile is the path to the ca certificate used for Consul communication
-	CAFile string `mapstructure:"ca_file"`
+	// CAFile is the path to the ca certificate used for Consul communication.
+	//
+	// Uses Consul's default and env var.
+	CAFile string `hcl:"ca_file"`
 
 	// CertFile is the path to the certificate for Consul communication
-	CertFile string `mapstructure:"cert_file"`
+	CertFile string `hcl:"cert_file"`
 
 	// KeyFile is the path to the private key for Consul communication
-	KeyFile string `mapstructure:"key_file"`
+	KeyFile string `hcl:"key_file"`
 
 	// ServerAutoJoin enables Nomad servers to find peers by querying Consul and
 	// joining them
-	ServerAutoJoin *bool `mapstructure:"server_auto_join"`
+	ServerAutoJoin *bool `hcl:"server_auto_join"`
 
 	// ClientAutoJoin enables Nomad servers to find addresses of Nomad servers
 	// and register with them
-	ClientAutoJoin *bool `mapstructure:"client_auto_join"`
+	ClientAutoJoin *bool `hcl:"client_auto_join"`
+
+	// ExtraKeysHCL is used by hcl to surface unexpected keys
+	ExtraKeysHCL []string `hcl:",unusedKeys" json:"-"`
 }
 
 // DefaultConsulConfig() returns the canonical defaults for the Nomad
-// `consul` configuration.
+// `consul` configuration. Uses Consul's default configuration which reads
+// environment variables.
 func DefaultConsulConfig() *ConsulConfig {
+	def := consul.DefaultConfig()
 	return &ConsulConfig{
 		ServerServiceName:   "nomad",
 		ServerHTTPCheckName: "Nomad Server HTTP Check",
@@ -102,11 +123,15 @@ func DefaultConsulConfig() *ConsulConfig {
 		ClientHTTPCheckName: "Nomad Client HTTP Check",
 		AutoAdvertise:       helper.BoolToPtr(true),
 		ChecksUseAdvertise:  helper.BoolToPtr(false),
-		EnableSSL:           helper.BoolToPtr(false),
-		VerifySSL:           helper.BoolToPtr(true),
 		ServerAutoJoin:      helper.BoolToPtr(true),
 		ClientAutoJoin:      helper.BoolToPtr(true),
 		Timeout:             5 * time.Second,
+
+		// From Consul api package defaults
+		Addr:      def.Address,
+		EnableSSL: helper.BoolToPtr(def.Scheme == "https"),
+		VerifySSL: helper.BoolToPtr(!def.TLSConfig.InsecureSkipVerify),
+		CAFile:    def.TLSConfig.CAFile,
 	}
 }
 
@@ -132,15 +157,22 @@ func (a *ConsulConfig) Merge(b *ConsulConfig) *ConsulConfig {
 	if b.ClientHTTPCheckName != "" {
 		result.ClientHTTPCheckName = b.ClientHTTPCheckName
 	}
+	result.Tags = append(result.Tags, b.Tags...)
 	if b.AutoAdvertise != nil {
 		result.AutoAdvertise = helper.BoolToPtr(*b.AutoAdvertise)
 	}
 	if b.Addr != "" {
 		result.Addr = b.Addr
 	}
+	if b.GRPCAddr != "" {
+		result.GRPCAddr = b.GRPCAddr
+	}
 	if b.Timeout != 0 {
 		result.Timeout = b.Timeout
 	}
+	if b.TimeoutHCL != "" {
+		result.TimeoutHCL = b.TimeoutHCL
+	}
 	if b.Token != "" {
 		result.Token = b.Token
 	}
@@ -226,7 +258,6 @@ func (c *ConsulConfig) ApiConfig() (*consul.Config, error) {
 		}
 		config.Transport.TLSClientConfig = tlsConfig
 	}
-
 	return config, nil
 }
 
