@@ -1,22 +1,27 @@
 import Component from '@ember/component';
 import { computed } from '@ember/object';
+import { on } from '@ember/object/evented';
 
 export default Component.extend({
   timerTicks: 0,
 
-  startTimer: function() {
+  startTimer: on('init', function() {
     this.set(
       'timer',
       setInterval(() => {
         this.incrementProperty('timerTicks');
       }, 500)
     );
-  }.on('init'),
+  }),
 
   willDestroy() {
-    clearInterval(this.get('timer'));
+    clearInterval(this.timer);
   },
 
+  distributionBarDatum: computed(() => {
+    return [{ label: 'one', value: 10 }];
+  }),
+
   distributionBarData: computed(() => {
     return [
       { label: 'one', value: 10 },
