@@ -10,6 +10,7 @@ export default Controller.extend(Sortable, Searchable, {
     searchTerm: 'search',
     sortProperty: 'sort',
     sortDescending: 'desc',
+    onlyPreemptions: 'preemptions',
   },
 
   currentPage: 1,
@@ -20,10 +21,25 @@ export default Controller.extend(Sortable, Searchable, {
 
   searchProps: computed(() => ['shortId', 'name']),
 
-  listToSort: alias('model.allocations'),
+  onlyPreemptions: false,
+
+  visibleAllocations: computed(
+    'model.allocations.[]',
+    'preemptions.[]',
+    'onlyPreemptions',
+    function() {
+      return this.onlyPreemptions ? this.preemptions : this.model.allocations;
+    }
+  ),
+
+  listToSort: alias('visibleAllocations'),
   listToSearch: alias('listSorted'),
   sortedAllocations: alias('listSearched'),
 
+  preemptions: computed('model.allocations.@each.wasPreempted', function() {
+    return this.model.allocations.filterBy('wasPreempted');
+  }),
+
   sortedEvents: computed('model.events.@each.time', function() {
     return this.get('model.events')
       .sortBy('time')
@@ -38,5 +54,9 @@ export default Controller.extend(Sortable, Searchable, {
     gotoAllocation(allocation) {
       this.transitionToRoute('allocations.allocation', allocation);
     },
+
+    setPreemptionFilter(value) {
+      this.set('onlyPreemptions', value);
+    },
   },
 });
