@@ -8,11 +8,14 @@ const Router = EmberRouter.extend({
 
 Router.map(function() {
   this.route('jobs', function() {
+    this.route('run');
     this.route('job', { path: '/:job_name' }, function() {
       this.route('task-group', { path: '/:name' });
       this.route('definition');
       this.route('versions');
       this.route('deployments');
+      this.route('evaluations');
+      this.route('allocations');
     });
   });
 
@@ -28,6 +31,8 @@ Router.map(function() {
     this.route('allocation', { path: '/:allocation_id' }, function() {
       this.route('task', { path: '/:name' }, function() {
         this.route('logs');
+        this.route('fs-root', { path: '/fs' });
+        this.route('fs', { path: '/fs/*path' });
       });
     });
   });
