@@ -1,4 +1,3 @@
-import { get } from '@ember/object';
 import { assign } from '@ember/polyfills';
 import { inject as service } from '@ember/service';
 import ApplicationSerializer from './application';
@@ -13,8 +12,9 @@ export default ApplicationSerializer.extend({
 
   normalize(modelClass, hash) {
     // Transform the map-based Drivers object into an array-based NodeDriver fragment list
-    hash.Drivers = Object.keys(get(hash, 'Drivers') || {}).map(key => {
-      return assign({}, get(hash, `Drivers.${key}`), { Name: key });
+    const drivers = hash.Drivers || {};
+    hash.Drivers = Object.keys(drivers).map(key => {
+      return assign({}, drivers[key], { Name: key });
     });
 
     return this._super(modelClass, hash);
