@@ -1,5 +1,11 @@
 /* eslint-env node */
 
+let USE_MIRAGE = true;
+
+if (process.env.USE_MIRAGE) {
+  USE_MIRAGE = process.env.USE_MIRAGE == 'true';
+}
+
 module.exports = function(environment) {
   var ENV = {
     modulePrefix: 'nomad-ui',
@@ -9,8 +15,7 @@ module.exports = function(environment) {
     EmberENV: {
       FEATURES: {
         // Here you can enable experimental features on an ember canary build
-        // e.g. 'with-controller': true
-        'ember-routing-router-service': true,
+        // e.g. EMBER_NATIVE_DECORATOR_SUPPORT: true
       },
       EXTEND_PROTOTYPES: {
         // Prevent Ember Data from overriding Date.parse.
@@ -19,8 +24,11 @@ module.exports = function(environment) {
     },
 
     APP: {
-      // Here you can pass flags/options to your application instance
-      // when it is created
+      blockingQueries: true,
+      mirageScenario: 'smallCluster',
+      mirageWithNamespaces: true,
+      mirageWithTokens: true,
+      mirageWithRegions: true,
     },
   };
 
@@ -32,7 +40,8 @@ module.exports = function(environment) {
     // ENV.APP.LOG_VIEW_LOOKUPS = true;
 
     ENV['ember-cli-mirage'] = {
-      // enabled: false,
+      enabled: USE_MIRAGE,
+      excludeFilesFromBuild: !USE_MIRAGE,
     };
   }
 
@@ -45,6 +54,7 @@ module.exports = function(environment) {
     ENV.APP.LOG_VIEW_LOOKUPS = false;
 
     ENV.APP.rootElement = '#ember-testing';
+    ENV.APP.autoboot = false;
 
     ENV.browserify = {
       tests: true,
@@ -55,8 +65,8 @@ module.exports = function(environment) {
     };
   }
 
-  if (environment === 'production') {
-  }
+  // if (environment === 'production') {
+  // }
 
   return ENV;
 };
